#!/bin/bash

# copy, modify and execute this script in the run directory

# argument (optional):  COMPARE=2 --> compare the 2 runs

COMPARE=$1

DIRTOOLS="../POST"

# DATASET 1
dirdata1='./OUT_Flux_S00'
subtitle1='GSEID 7.5e-16 dtmin=5, Rtol=0.025'
fileform1=ZSPEC.dat

# DATASET 2 (OPTIONAL)
dirdata2='./OUT_Flux_000'
subtitle2='DGESV dtmin=5, Rtol=0.025'
fileform2=ZSPEC.dat

subtitle1="$(echo $subtitle1 | sed -e 's/ /_/g' )"
subtitle2="$(echo $subtitle2 | sed -e 's/ /_/g' )"


case $COMPARE in
	2)
#         DATASET 1  vs  DATASET 2
PREF='-p C_'
dataset2="-c $dirdata2 -m $subtitle2 -e $fileform2"
echo " >>>>>>>>>>>>>   compare $dirdata1 and $dirdata2 results"
        ;;
        *)
#         DATASET 1 only
PREF=
dataset2=
echo " >>>>>>>>>>>>>   plot $dirdata1 results"
        ;;
esac

echo " >>>>>>>>>>>>>   plots in $dirdata1"

PSCRIPT="plotruns.sh $PREF -d $dirdata1 -n $subtitle1 -f $fileform1 $dataset2"


ln -f -s $DIRTOOLS/cexplot.par .

#----------------------------------	FAMILIES
SPEC=O
title="Ox_species"
ylabel='Volume_mixing_ratio'
legends=O3_O3P_O1D
$DIRTOOLS/$PSCRIPT -s $SPEC -t $title -y $ylabel -l $legends

SPEC=N
title="NOx_species"
ylabel='Volume_mixing_ratio'
legends=NO_NO2_NO3_N2O5_HNO3_NO+NO2
$DIRTOOLS/$PSCRIPT -s $SPEC -t $title -y $ylabel -l $legends

SPEC=H
title="HOx_species"
ylabel='Volume_mixing_ratio'
legends=H_OH_HO2_H2O2
$DIRTOOLS/$PSCRIPT -s $SPEC -t $title -y $ylabel -l $legends

SPEC=Cl
title="Clx_species"
ylabel='Volume_mixing_ratio'
legends=Cl2_Cl_ClO_Cl2O2
$DIRTOOLS/$PSCRIPT -s $SPEC -t $title -y $ylabel -l $legends

SPEC=Br
title="Brx_species"
ylabel='Volume_mixing_ratio'
legends=Br_BrO_BrCl_HBr_HOBr_BrONO2
$DIRTOOLS/$PSCRIPT -s $SPEC -t $title -y $ylabel -l $legends

SPEC=C
title="Cx_species"
ylabel='Volume_mixing_ratio'
legends=CO_HCHO_CH3_CH3O
$DIRTOOLS/$PSCRIPT -s $SPEC -t $title -y $ylabel -l $legends


#----------------------------------	INDIVIDUAL SPECIES
#SPECLST='ClO BrO O1D OH HO2 H2O2 HCHO'
SPECLST='ClO BrO O1D OH HO2 H2O2 HCHO ACO3 ADDT ALD ETHP ISOP PAN PHO OP1 OP2 OLIP OLNN KET MACR HC8P TOLP XO2'

for specie in $SPECLST ; do
SPEC=_${specie}
title=$SPEC
ylabel='Concentration_(molecules\/cm3)'
legends=${specie}
$DIRTOOLS/$PSCRIPT -s $SPEC -t $title -y $ylabel -l $legends
done
