      SUBROUTINE GAUSSEIDEL(A,B,X,N,IMAX,EPSI,DIFF,ITER,INFO)

!     SUBROUTINE FOR SOLVING AX=B USING GAUSS-SEIDEL METHOD
!     the convergence criterion is based on the residue norm with residue=AX-B

      USE TYPES, ONLY : JPIM,JPRB

      IMPLICIT NONE
      INTEGER (KIND=JPIM), INTENT(IN) :: N	! System size
      REAL(KIND=JPRB) , INTENT(IN) :: A(N,N),B(N)	! Matrix and RHS of the system
      INTEGER (KIND=JPIM), INTENT(IN) :: IMAX	! Max number of iterations
      REAL(KIND=JPRB) , INTENT(IN) :: EPSI	! Convergence criterion

      REAL(KIND=JPRB) , INTENT(INOUT) :: X(N)	! Guess / Solution of the system
      REAL(KIND=JPRB) , INTENT(OUT) :: DIFF	! Measure of the convergence
      INTEGER (KIND=JPIM), INTENT(OUT) :: ITER	! Number of iterations used
      INTEGER (KIND=JPIM), INTENT(OUT) :: INFO	! Convergence (0) or not (1)

! local variables
      REAL(KIND=JPRB) :: XINC(N)
      REAL(KIND=JPRB) :: R(N)                   ! Residue
      REAL(KIND=JPRB) :: SUMAX
      REAL(KIND=JPRB) :: XNEW
      REAL(KIND=JPRB) :: CHECK,PCHECK
      INTEGER (KIND=JPIM) :: I,J,NN

      REAL(KIND=JPRB) :: EPSI2
      REAL(KIND=JPRB) :: Rnorm2,Bnorm2

      EPSI2=EPSI*EPSI

      Bnorm2=dot_product(B,B)

! initial residue
      R=B - matmul(A,X)
      Rnorm2=dot_product(R,R)

      CHECK=Rnorm2/Bnorm2

      INFO=1

      ITER=0
      DO WHILE ( ITER.lt.IMAX .and. CHECK.gt.EPSI2 )
         ! calculate   x(k+1)= x(k) + L-1 * r(k)
         DO I=1,N
            SUMAX=0.0_JPRB
            DO J=1,I-1
               SUMAX=SUMAX+A(I,J)*XINC(J)
            ENDDO
            XINC(I)=(-SUMAX+R(I))/A(I,I)
         ENDDO
         X=X+XINC

         ITER = ITER + 1
         ! new residue and convergence criterion
         R=B - matmul(A,X)
         Rnorm2=dot_product(R,R)
         CHECK=Rnorm2/Bnorm2
      ENDDO


      IF ( CHECK.le.EPSI2 ) INFO=0        ! convergence is reached

      DIFF=SQRT(CHECK)

      RETURN
      END SUBROUTINE GAUSSEIDEL
