!  ASIS method 0D Fortran code demonstrator
!
!  Reference:
!    ASIS v1.0: an adaptative solver for the simulation of atmospheric chemistry
!    Daniel Cariolle, Philippe Moinat, Hubert Teyssèdre, Luc Giraud, Béatrice Josse, and Franck Lefèvre
!    Geosci. Model Dev. Discuss., doi:10.5194/gmd-2016-281, 2016
!    Manuscript accepted for publication in the journal Geosci. Model Dev.
!    Published: 21 November 2016
!
!  TEST CASE PARAMETERS
!    most of these parameters are read from the namelist (cf readnl.F90)
!
MODULE case_module

USE TYPES, ONLY : JPIM,JPRB

IMPLICIT NONE
!
!   UNIT NUMBERS AND FILE NAMES   ---------
INTEGER(KIND=JPIM) , SAVE :: IU_NL = 11     ! namelist file  asis.lst

INTEGER(KIND=JPIM) , SAVE :: IU_INI = 12    ! Initial value file
CHARACTER(LEN=15) , SAVE :: initfile

INTEGER(KIND=JPIM) , SAVE :: IU_EMI = 13    ! Emission rate file
CHARACTER(LEN=15) , SAVE :: emisfile

INTEGER(KIND=JPIM) , SAVE :: IU_TUV = 14    ! TUV photodissociation rate file
CHARACTER(LEN=15) , SAVE :: TUVfile

INTEGER(KIND=JPIM) , SAVE :: IU_OUT = 19    ! output files, concentrations
!
!   TEST CASE PARAMETERS   ---------
CHARACTER(LEN=15) , SAVE :: case_name

REAL(KIND=JPRB) , SAVE :: PT                ! Temperature	                        % K
REAL(KIND=JPRB) , SAVE :: PZ                ! Pressure 	                                % hPa
REAL(KIND=JPRB) , SAVE :: WV                ! Water vapor 	                        % kg/kg
REAL(KIND=JPRB) , SAVE :: zhcl              ! Boundary layer thickness  	        % m
REAL(KIND=JPRB) , SAVE :: ef                ! Emission factor
LOGICAL , SAVE :: EMISTEP                   ! Logical, if true emissions are applied at each timestep 
                                            !           (otherwise at each chemical substep)
REAL(KIND=JPRB) , SAVE :: tauless           ! Rain scavenging parameter


!   PHYSICAL PARAMETERS   ---------
REAL(KIND=JPRB) , SAVE :: PHNM              ! Air density                               % molecules/cm3
REAL(KIND=JPRB) , SAVE :: PH2O              ! Water vapor                               % molecules/cm3 


!   NUMERICAL PARAMETERS   ---------
REAL(KIND=JPRB) , SAVE :: dtt               ! time step 	                        % s
INTEGER(KIND=JPIM) , SAVE :: ndays          ! Number of days simulated
INTEGER(KIND=JPIM) , SAVE :: ndt_day        ! number of time steps per day

INTEGER(KIND=JPIM) , SAVE :: nitmax         ! Number of iterations to define the time step
REAL(KIND=JPRB) , SAVE :: dtmin             ! Minimal sub-timestep
REAL(KIND=JPRB) , SAVE :: RTOL              ! ASIS Relative Tolerance parameter
REAL(KIND=JPRB) , SAVE :: ATOL              ! ASIS Absolute Tolerance paremeter
REAL(KIND=JPRB) , SAVE :: coefmin,coefmax
LOGICAL , SAVE :: SEMI                      ! True: use the SIS approximation to determine the sub step

CHARACTER(LEN=5) , SAVE :: SOLMET           ! Solution method DGESV, GMRES or GSEID
INTEGER(KIND=JPIM) , SAVE :: itersol        ! Max number of iterations, iterative method
REAL(KIND=JPRB) , SAVE :: cvsol             ! Convergence criterion, iterative method
INTEGER(KIND=JPIM) , SAVE :: ipl_gmres,ios_gmres,irt_gmres,irl_gmres  ! GMRES method parameters

END MODULE case_module
