!  ASIS method 0D Fortran code demonstrator
!
!  Reference:
!    ASIS v1.0: an adaptative solver for the simulation of atmospheric chemistry
!    Daniel Cariolle, Philippe Moinat, Hubert Teyssèdre, Luc Giraud, Béatrice Josse, and Franck Lefèvre
!    Geosci. Model Dev. Discuss., doi:10.5194/gmd-2016-281, 2016
!    Manuscript accepted for publication in the journal Geosci. Model Dev.
!    Published: 21 November 2016
!
!  JACOBIAN MATRIX FILLING
!    refer to section 2.2 of the paper and Eq. 6 and 9
!
MODULE M_FILL_ASIS

  USE TYPES, ONLY : JPIM,JPRB
  USE chemistry_module, ONLY : NB_SP, &
   &                   NB_PHOT,NB_REACTION_3,NB_REACTION_4, &
   &                   Zindice_phot,Zindice_3,Zindice_4, &
   &                   v_phot,v_3,v_4

CONTAINS


SUBROUTINE fill_matrix(C,M, &
                       SEMI,TL,TP )

! The matrix M filled by this routine is  -M  in Eq 7 of the ASIS paper
IMPLICIT NONE

REAL(KIND=JPRB), DIMENSION(NB_SP), INTENT(IN) :: C
REAL(KIND=JPRB), DIMENSION(NB_SP,NB_SP), INTENT(OUT) :: M

LOGICAL :: SEMI
REAL(KIND=JPRB), DIMENSION(*), INTENT(OUT) :: TL,TP

! local variables
INTEGER(KIND=JPIM):: iphot,iphot_2,iphot_4,iphot_6
INTEGER(KIND=JPIM):: i3,i3_2,i3_4,i3_6
INTEGER(KIND=JPIM):: i4,i4_2,i4_4,i4_6,i4_8
REAL(KIND=JPRB) :: eps,Delta
INTEGER(KIND=JPIM):: n

   M=0.0_JPRB

                  if ( .not. SEMI ) then
!
! PHOTOLYSIS REACTIONS
!----------------------
! Zindice_phot(J_PHOTO)=(Alpha,JA,Gamma,JG,Zeta,JZ) 
!                          1    2   3   4   5   6
!            for PHOTO reaction Alpha A --> Gamma G + Zeta Z
! coefficients applied to A, appear in the equations for A, G and Z

   do iphot=1,NB_PHOT      
      iphot_2=Zindice_phot(iphot)%z2    ! species index
      iphot_4=Zindice_phot(iphot)%z4
      iphot_6=Zindice_phot(iphot)%z6
      M(iphot_6,iphot_2)=M(iphot_6,iphot_2)-Zindice_phot(iphot)%z5*v_phot(iphot)
      M(iphot_2,iphot_2)=M(iphot_2,iphot_2)+Zindice_phot(iphot)%z1*v_phot(iphot)
      M(iphot_4,iphot_2)=M(iphot_4,iphot_2)-Zindice_phot(iphot)%z3*v_phot(iphot)
   enddo
!   

! CHEMICAL REACTIONS OF TYPE  Alpha A --> Gamma G + Zeta Z
!---------------------------------------------------------
   do i3=1,NB_REACTION_3
      i3_2=Zindice_3(i3)%z2             ! species index
      i3_4=Zindice_3(i3)%z4
      i3_6=Zindice_3(i3)%z6
      M(i3_2,i3_2)=M(i3_2,i3_2)+Zindice_3(i3)%z1*v_3(i3)*C(i3_2)
      M(i3_4,i3_2)=M(i3_4,i3_2)-Zindice_3(i3)%z3*v_3(i3)*C(i3_2)
      M(i3_6,i3_2)=M(i3_6,i3_2)-Zindice_3(i3)%z5*v_3(i3)*C(i3_2)
   enddo
!

! CHEMICAL REACTIONS OF TYPE  Alpha A + Beta B --> Gamma G + Zeta Z
!-------------------------------------------------------------------
! Zindice_4(J_REACTION)=(Alpha,JA,Beta,JB,gamma,JG,Zeta,JZ) 
!                          1    2   3   4   5   6    7    8
   eps=1.0D-10

   do i4=1,NB_REACTION_4
      i4_2=Zindice_4(i4)%z2             ! species index
      i4_4=Zindice_4(i4)%z4
      i4_6=Zindice_4(i4)%z6
      i4_8=Zindice_4(i4)%z8

      Delta=abs(C(i4_2))/(abs(C(i4_2))+abs(C(i4_4))+eps)              ! cf Eq 9 of paper with exponent = 1
      Delta=min(Delta,1.0_JPRB)

      M(i4_2,i4_2)=M(i4_2,i4_2)+Zindice_4(i4)%z1*v_4(i4)*(1.0_JPRB - Delta)*C(i4_4)
      M(i4_2,i4_4)=M(i4_2,i4_4)+Zindice_4(i4)%z1*v_4(i4)*       Delta      *C(i4_2)

      M(i4_4,i4_2)=M(i4_4,i4_2)+Zindice_4(i4)%z3*v_4(i4)*(1.0_JPRB - Delta)*C(i4_4)
      M(i4_4,i4_4)=M(i4_4,i4_4)+Zindice_4(i4)%z3*v_4(i4)*       Delta      *C(i4_2)

      M(i4_6,i4_2)=M(i4_6,i4_2)-Zindice_4(i4)%z5*v_4(i4)*(1.0_JPRB - Delta)*C(i4_4)
      M(i4_6,i4_4)=M(i4_6,i4_4)-Zindice_4(i4)%z5*v_4(i4)*       Delta      *C(i4_2)

      M(i4_8,i4_2)=M(i4_8,i4_2)-Zindice_4(i4)%z7*v_4(i4)*(1.0_JPRB - Delta)*C(i4_4)
      M(i4_8,i4_4)=M(i4_8,i4_4)-Zindice_4(i4)%z7*v_4(i4)*       Delta      *C(i4_2)
   enddo


                  else    ! SEMI case

   do n=1,NB_SP
      TP(n)=0.0_JPRB
      TL(n)=0.0_JPRB
   enddo
!
! PHOTOLYSIS REACTIONS
!----------------------
! Zindice_phot(J_PHOTO)=(Alpha,JA,Gamma,JG,Zeta,JZ) 
!                          1    2   3   4   5   6
!            for PHOTO reaction Alpha A --> Gamma G + Zeta Z
! coefficients applied to A, appear in the equations for A, G and Z

   do iphot=1,NB_PHOT      
      iphot_2=Zindice_phot(iphot)%z2		! species index
      iphot_4=Zindice_phot(iphot)%z4
      iphot_6=Zindice_phot(iphot)%z6
      M(iphot_6,iphot_2)=M(iphot_6,iphot_2)-Zindice_phot(iphot)%z5*v_phot(iphot)
      M(iphot_2,iphot_2)=M(iphot_2,iphot_2)+Zindice_phot(iphot)%z1*v_phot(iphot)
      M(iphot_4,iphot_2)=M(iphot_4,iphot_2)-Zindice_phot(iphot)%z3*v_phot(iphot)

      TP(iphot_6)=TP(iphot_6)+Zindice_phot(iphot)%z5*v_phot(iphot)*C(iphot_2)
      TL(iphot_2)=TL(iphot_2)+Zindice_phot(iphot)%z1*v_phot(iphot)
      TP(iphot_4)=TP(iphot_4)+Zindice_phot(iphot)%z3*v_phot(iphot)*C(iphot_2)
   enddo
!   

! CHEMICAL REACTIONS OF TYPE  Alpha A --> Gamma G + Zeta Z
!---------------------------------------------------------
   do i3=1,NB_REACTION_3
      i3_2=Zindice_3(i3)%z2		! species index
      i3_4=Zindice_3(i3)%z4
      i3_6=Zindice_3(i3)%z6
      M(i3_2,i3_2)=M(i3_2,i3_2)+Zindice_3(i3)%z1*v_3(i3)*C(i3_2)
      M(i3_4,i3_2)=M(i3_4,i3_2)-Zindice_3(i3)%z3*v_3(i3)*C(i3_2)
      M(i3_6,i3_2)=M(i3_6,i3_2)-Zindice_3(i3)%z5*v_3(i3)*C(i3_2)

      TL(i3_2)=TL(i3_2)+Zindice_3(i3)%z1*v_3(i3)*C(i3_2)
      TP(i3_4)=TP(i3_4)+Zindice_3(i3)%z3*v_3(i3)*C(i3_2)*C(i3_2)
      TP(i3_6)=TP(i3_6)+Zindice_3(i3)%z5*v_3(i3)*C(i3_2)*C(i3_2)
   enddo
!

! CHEMICAL REACTIONS OF TYPE  Alpha A + Beta B --> Gamma G + Zeta Z
!-------------------------------------------------------------------
! Zindice_4(J_REACTION)=(Alpha,JA,Beta,JB,gamma,JG,Zeta,JZ) 
!                          1    2   3   4   5   6    7    8
   eps=1.0D-10

   do i4=1,NB_REACTION_4
      i4_2=Zindice_4(i4)%z2		! species index
      i4_4=Zindice_4(i4)%z4
      i4_6=Zindice_4(i4)%z6
      i4_8=Zindice_4(i4)%z8

      Delta=abs(C(i4_2))/(abs(C(i4_2))+abs(C(i4_4))+eps)
      Delta=min(Delta,1.0_JPRB)

      M(i4_2,i4_2)=M(i4_2,i4_2)+Zindice_4(i4)%z1*v_4(i4)*(1.0_JPRB - Delta)*C(i4_4)
      M(i4_2,i4_4)=M(i4_2,i4_4)+Zindice_4(i4)%z1*v_4(i4)*       Delta      *C(i4_2)

      M(i4_4,i4_2)=M(i4_4,i4_2)+Zindice_4(i4)%z3*v_4(i4)*(1.0_JPRB - Delta)*C(i4_4)
      M(i4_4,i4_4)=M(i4_4,i4_4)+Zindice_4(i4)%z3*v_4(i4)*       Delta      *C(i4_2)

      M(i4_6,i4_2)=M(i4_6,i4_2)-Zindice_4(i4)%z5*v_4(i4)*(1.0_JPRB - Delta)*C(i4_4)
      M(i4_6,i4_4)=M(i4_6,i4_4)-Zindice_4(i4)%z5*v_4(i4)*       Delta      *C(i4_2)

      M(i4_8,i4_2)=M(i4_8,i4_2)-Zindice_4(i4)%z7*v_4(i4)*(1.0_JPRB - Delta)*C(i4_4)
      M(i4_8,i4_4)=M(i4_8,i4_4)-Zindice_4(i4)%z7*v_4(i4)*       Delta      *C(i4_2)

      TL(i4_2)=TL(i4_2)+Zindice_4(i4)%z1*v_4(i4) *C(i4_4)
      TL(i4_4)=TL(i4_4)+Zindice_4(i4)%z3*v_4(i4) *C(i4_2)
      TP(i4_6)=TP(i4_6)+Zindice_4(i4)%z5*v_4(i4) *C(i4_2) *C(i4_4)
      TP(i4_8)=TP(i4_8)+Zindice_4(i4)%z7*v_4(i4) *C(i4_2) *C(i4_4)
   enddo

                  endif

END SUBROUTINE fill_matrix


SUBROUTINE fill_PL(C, TL,TP )

IMPLICIT NONE

REAL(KIND=JPRB), DIMENSION(NB_SP), INTENT(IN) :: C

REAL(KIND=JPRB), DIMENSION(NB_SP), INTENT(OUT) :: TL,TP

! local variables
INTEGER(KIND=JPIM):: iphot,iphot_2,iphot_4,iphot_6
INTEGER(KIND=JPIM):: i3,i3_2,i3_4,i3_6
INTEGER(KIND=JPIM):: i4,i4_2,i4_4,i4_6,i4_8


   TP=0.0_JPRB
   TL=0.0_JPRB
!
! PHOTOLYSIS REACTIONS
!----------------------
   do iphot=1,NB_PHOT      
      iphot_2=Zindice_phot(iphot)%z2		! species index
      iphot_4=Zindice_phot(iphot)%z4
      iphot_6=Zindice_phot(iphot)%z6
      TP(iphot_6)=TP(iphot_6)+Zindice_phot(iphot)%z5*v_phot(iphot)*C(iphot_2)
      TL(iphot_2)=TL(iphot_2)+Zindice_phot(iphot)%z1*v_phot(iphot)
      TP(iphot_4)=TP(iphot_4)+Zindice_phot(iphot)%z3*v_phot(iphot)*C(iphot_2)
   enddo
!   

! CHEMICAL REACTIONS OF TYPE  Alpha A --> Gamma G + Zeta Z
!---------------------------------------------------------
   do i3=1,NB_REACTION_3
      i3_2=Zindice_3(i3)%z2		! species index
      i3_4=Zindice_3(i3)%z4
      i3_6=Zindice_3(i3)%z6
      TL(i3_2)=TL(i3_2)+Zindice_3(i3)%z1*v_3(i3)*C(i3_2)
      TP(i3_4)=TP(i3_4)+Zindice_3(i3)%z3*v_3(i3)*C(i3_2)*C(i3_2)
      TP(i3_6)=TP(i3_6)+Zindice_3(i3)%z5*v_3(i3)*C(i3_2)*C(i3_2)
   enddo
!

! CHEMICAL REACTIONS OF TYPE  Alpha A + Beta B --> Gamma G + Zeta Z
!-------------------------------------------------------------------

   do i4=1,NB_REACTION_4
      i4_2=Zindice_4(i4)%z2		! species index
      i4_4=Zindice_4(i4)%z4
      i4_6=Zindice_4(i4)%z6
      i4_8=Zindice_4(i4)%z8

      TL(i4_2)=TL(i4_2)+Zindice_4(i4)%z1*v_4(i4) *C(i4_4)
      TL(i4_4)=TL(i4_4)+Zindice_4(i4)%z3*v_4(i4) *C(i4_2)
      TP(i4_6)=TP(i4_6)+Zindice_4(i4)%z5*v_4(i4) *C(i4_2) *C(i4_4)
      TP(i4_8)=TP(i4_8)+Zindice_4(i4)%z7*v_4(i4) *C(i4_2) *C(i4_4)
   enddo

END SUBROUTINE fill_PL

END MODULE M_FILL_ASIS
