!  ASIS method 0D Fortran code demonstrator
!
!  Reference:
!    ASIS v1.0: an adaptative solver for the simulation of atmospheric chemistry
!    Daniel Cariolle, Philippe Moinat, Hubert Teyssèdre, Luc Giraud, Béatrice Josse, and Franck Lefèvre
!    Geosci. Model Dev. Discuss., doi:10.5194/gmd-2016-281, 2016
!    Manuscript accepted for publication in the journal Geosci. Model Dev.
!    Published: 21 November 2016
!
!  INITIALIZATION
!
!    read parameters in namelist file
!    initialize the chemical scheme
!    read initial conditions from a file
!    read emission fluxes from a file
!    initialize solution method and allocate arrays


SUBROUTINE INIT

USE chemistry_module, ONLY : alloc_c, alloc_v, alloc_out
USE num_species_module, ONLY : num_species
USE photodis_module, ONLY : alloc_J, read_J
USE indices_reactions_module, ONLY : indices_reactions
USE solution_module, ONLY : alloc_asis, alloc_meth, set_solve, ini_counter

USE M_readnl, ONLY : READNL
USE M_set_conditions, ONLY : set_inicon, set_emis

IMPLICIT NONE

! Local varialbles


! READ NAMELIST
!----------------------------
call READNL


! INITIALIZE CHEMISTRY
!----------------------------
! species index and number (NB_SP)  - allocate concentration array
CALL num_species
CALL alloc_c

! photodissociation rates
!  (see subroutine read_J, module photodis_module for input file format description)
CALL alloc_J
CALL read_J

! reaction indices  - allocate reaction rates arrays
CALL indices_reactions
CALL alloc_v

! CALL INIHETERO   - introduce here initialization of heterogeneous chemistry


!
! SET INITIAL CONCENTRATIONS (molecules/cm3)
!---------------------------
call set_inicon

!
! SET EMISSION INJECTION (molecules/cm3/s)
!----------------------------
call set_emis

!
! RAIN SCAVENGING RATE
!----------------------------
! only applied to HNO3, parameter tauless read from the namelist



! ALLOCATE ARRAYS AND INITIALIZE THE SOLUTION METHOD
!----------------------------
! basic arrays for ASIS
call alloc_asis

! Additional arrays specific to each solution method
call alloc_meth

! Set the solution method for linear systems
call set_solve

! Initialize counters
call ini_counter



! ALLOCATE STORAGE FOR OUTPUT
!----------------------------
call alloc_out


END SUBROUTINE INIT
