!  ASIS method 0D Fortran code demonstrator
!
!  Reference:
!    ASIS v1.0: an adaptative solver for the simulation of atmospheric chemistry
!    Daniel Cariolle, Philippe Moinat, Hubert Teyssèdre, Luc Giraud, Béatrice Josse, and Franck Lefèvre
!    Geosci. Model Dev. Discuss., doi:10.5194/gmd-2016-281, 2016
!    Manuscript accepted for publication in the journal Geosci. Model Dev.
!    Published: 21 November 2016
!
!  READ NAMELIST FILE   asis.lst
!
MODULE M_readnl

USE TYPES, ONLY : JPIM,JPRB

CONTAINS


SUBROUTINE READNL

USE CASE_MODULE

IMPLICIT NONE

! Argument variables

! Local varialbles
INTEGER(KIND=JPIM)  :: ndt_hour

open(IU_NL,file='asis.lst',form='formatted')
! TEST CASE PHYSICAL PARAMETER DEFINITION
read(IU_NL,*)
read(IU_NL,*)
read(IU_NL,*)
read(IU_NL,*)
read(IU_NL,'(A15)') case_name
read(IU_NL,*) PT                ! Temperature	                        % K
read(IU_NL,*) PZ                ! Pressure 	                        % hPa
read(IU_NL,*) WV                ! Water vapor 	                        % kg/kg
read(IU_NL,*) zhcl              ! Boundary layer thickness  	        % m
read(IU_NL,'(A15)') emisfile    ! Emission rate file
read(IU_NL,*) ef                ! Emission factor
read(IU_NL,*) EMISTEP           ! Logical, if true emissions are applied at each timestep 
                                !             (otherwise at each chemical substep)
read(IU_NL,*) tauless           ! Rain scavenging parameter
read(IU_NL,'(A15)') initfile    ! Initial value file
read(IU_NL,'(A15)') TUVfile     ! TUV photodissociation rate file
read(IU_NL,*) dtt               ! time step 	% s
read(IU_NL,*) ndays             ! Number of days simulated
read(IU_NL,*)

PZ = PZ *100._JPRB              ! convert pressure hPa --> Pa
zhcl = zhcl *100._JPRB          ! convert zhcl m --> cm
PHNM = calc_const(PT,PZ)        ! Air molecules/cm3
PH2O=PHNM*WV*30.56/18.          ! Water vapor molecules/cm3 (rmv*hnm)

tauless = tauless*24.*3600.     !  convert in seconds 


ndt_hour = int(3600._JPRB / dtt)       ! number of time steps per hour
dtt = 3600._JPRB / float(ndt_hour)
ndt_day = ndt_hour * 24                ! number of time steps per day


! ASIS METHOD PARAMETERS
read(IU_NL,*)
read(IU_NL,*)
read(IU_NL,*) nitmax            ! Number of iterations to define the time step
read(IU_NL,*) dtmin             ! Minimal sub-timestep
read(IU_NL,*) RTOL
read(IU_NL,*) ATOL
read(IU_NL,*) coefmax
read(IU_NL,*) coefmin
read(IU_NL,*) SEMI              ! Logical, use the SIS approximation to determine the sub step
read(IU_NL,*)



! LINEAR SYSTEM SOLUTION METHOD
read(IU_NL,*)
read(IU_NL,*)
read(IU_NL,'(a5)') SOLMET       ! Solution method
read(IU_NL,*) itersol           ! Max number of iterations, iterative method
read(IU_NL,*) cvsol             ! Convergence criterion, iterative method

END SUBROUTINE READNL


FUNCTION calc_const(Tx,pm)
!  Calculate number of molecules (n2 + o2) in the atmosphere.
!     xmolec=(r0*ts/xp0)*p/t	 where r0, ts, xp0 are
!  reference values for the number, temperature and pressure

USE cstphy_module

IMPLICIT NONE

REAL(KIND=JPRB) :: calc_const
REAL(KIND=JPRB) :: Tx
REAL(KIND=JPRB) :: pm

INTEGER(KIND=JPIM) :: i

 calc_const=(r0*ts/xp0)*pm/Tx

END FUNCTION calc_const


END MODULE M_readnl
