!  ASIS method 0D Fortran code demonstrator
!
!  Reference:
!    ASIS v1.0: an adaptative solver for the simulation of atmospheric chemistry
!    Daniel Cariolle, Philippe Moinat, Hubert Teyssèdre, Luc Giraud, Béatrice Josse, and Franck Lefèvre
!    Geosci. Model Dev. Discuss., doi:10.5194/gmd-2016-281, 2016
!    Manuscript accepted for publication in the journal Geosci. Model Dev.
!    Published: 21 November 2016
!
!  LINEAR SYSTEM SOLUTION
!    available method are  DGESV (direct) or GMRES (iterative GMRES package) or GSEID (iterative Gauss Seidel)
!
SUBROUTINE SOLVESYS(M,RHSOL,SOLG)

!  M        :  Matrix of the system
!  RHSOL    :  RHS at input, SOLution at output
!  SOLG     :  Solution guess

USE TYPES, ONLY : JPIM,JPRB
USE case_module , ONLY : SOLMET,itersol,cvsol
USE solution_module, ONLY : IPIV &              ! SOLMET=DGESV
    &                      ,mrestart,lwork,icntl,cntl,infog,rinfog &  ! SOLMET=GMRES
    &                      ,nsolve,niterm       ! counters
USE chemistry_module, ONLY : NB_SP

IMPLICIT NONE

! Routine arguments
REAL(KIND=JPRB), DIMENSION(NB_SP,NB_SP) :: M
REAL(KIND=JPRB), DIMENSION(NB_SP) :: RHSOL,SOLG

! Local variables
INTEGER(KIND=JPIM) :: INFO
REAL(KIND=JPRB) :: cvmes
INTEGER(KIND=JPIM) :: n

EXTERNAL DGESV

! ------ Direct Method using DGESV
       if ( SOLMET == 'DGESV' ) &
       call DGESV(NB_SP,1,M,NB_SP,IPIV,RHSOL,NB_SP,INFO)


! ------ Iterative Method, Gauss-Seidel
       if ( SOLMET == 'GSEID' ) then
          call GAUSSEIDEL(M,RHSOL,SOLG,NB_SP,itersol,cvsol,cvmes,n,INFO)
          if (INFO.ne.0) then
             write(*,*) '**** WARNING / Gauss-Seidel method - insufficient convergence'
             write(*,*) 'convergence =',cvmes,' after ',n,' iterations'
          endif
          niterm=niterm+n	! counter nb of iterations
          RHSOL=SOLG
       endif

! ------ Iterative Method, GMRES
       if ( SOLMET == 'GMRES' ) then
          call GMRESSOL(M,RHSOL,SOLG,NB_SP,mrestart,lwork,icntl,cntl,infog,rinfog)
          if (infog(1).ne.0) then
             write(*,*) '**** WARNING / GMRES method - exit abnormally'
             write(*,*) '       status : ',infog(1)
          endif
          niterm=niterm+infog(2)	! counter nb of iterations
          RHSOL=SOLG
       endif
        
       nsolve=nsolve+1		! counter of calls to solvesys

END SUBROUTINE
