!  ASIS method 0D Fortran code demonstrator
!
!  Reference:
!    ASIS v1.0: an adaptive solver for the simulation of atmospheric chemistry
!    Daniel Cariolle, Philippe Moinat, Hubert Teyssèdre, Luc Giraud, Béatrice Josse, and Franck Lefèvre
!    Geosci. Model Dev. Discuss., doi:10.5194/gmd-2016-281, 2016
!    Manuscript accepted for publication in the journal Geosci. Model Dev.
!    Published: 21 November 2016
!
!  CHEMICAL SCHEME VARIABLES
!    number of species and number of reactions
!    description of the reactions
!    reaction rates
!    emission injection rate
!    species concentration
!
MODULE chemistry_module
!
! The following reactions are considered:
!    PHOTO- Photolysis reactions with index JPO_J*
!    TYPE3- Reactions of the form a*A --> b*B+c*C with index JP3_J*
!    TYPE4- Reactions of the form a*A+d*D --> b*B+c*C with index JP4_J*
!
!    NB_PHOT       : number of reactions of type PHOTO
!    NB_REACTION_3 : number of reactions of TYPE3
!    NB_REACTION_4 : number of reactions of TYPE4

USE TYPES , ONLY : JPIM,JPRB, &
                 & Z3SPEC,Z4SPEC
USE case_module, ONLY : ndays,ndt_day

IMPLICIT NONE

INTEGER(KIND=JPIM), SAVE ::  NB_SP

INTEGER(KIND=JPIM), SAVE :: NB_PHOT,NB_REACTION_3,NB_REACTION_4

TYPE(Z3SPEC), DIMENSION(:), ALLOCATABLE, SAVE :: Zindice_phot,Zindice_3
TYPE(Z4SPEC), DIMENSION(:), ALLOCATABLE, SAVE :: Zindice_4

REAL(KIND=JPRB), DIMENSION(:), ALLOCATABLE, SAVE :: v_phot  ! Reaction rates
REAL(KIND=JPRB), DIMENSION(:), ALLOCATABLE, SAVE :: v_3
REAL(KIND=JPRB), DIMENSION(:), ALLOCATABLE, SAVE :: v_4


! Emission rates
REAL(KIND=JPRB), DIMENSION(:), ALLOCATABLE, SAVE :: EMIRATE ! Emission injection rate  molec/cm3/s

! Concentrations
REAL(KIND=JPRB), DIMENSION(:), ALLOCATABLE, SAVE :: CONC    ! Concentrations


! Output arrays
REAL(KIND=JPRB), DIMENSION(:,:), ALLOCATABLE :: C_ST        ! stored concentrations


CONTAINS

SUBROUTINE alloc_c
ALLOCATE(CONC(NB_SP))
CONC=0.0_JPRB
ALLOCATE(EMIRATE(NB_SP))
EMIRATE=0.0_JPRB
END SUBROUTINE alloc_c

SUBROUTINE dealloc_c
DEALLOCATE(CONC)
DEALLOCATE(EMIRATE)
END SUBROUTINE dealloc_c


SUBROUTINE alloc_v
ALLOCATE(v_phot(NB_PHOT))
ALLOCATE(v_3(NB_REACTION_3))
ALLOCATE(v_4(NB_REACTION_4))
v_phot=0._JPRB
v_3=0._JPRB
v_4=0._JPRB
END SUBROUTINE alloc_v

SUBROUTINE dealloc_v
DEALLOCATE(v_phot)
DEALLOCATE(v_3)
DEALLOCATE(v_4)
END SUBROUTINE dealloc_v


SUBROUTINE dealloc_z
DEALLOCATE(Zindice_phot)
DEALLOCATE(Zindice_3)
DEALLOCATE(Zindice_4)
END SUBROUTINE dealloc_z


SUBROUTINE alloc_out
ALLOCATE(C_ST(ndays*ndt_day,NB_SP))
C_ST=0._JPRB
END SUBROUTINE alloc_out

SUBROUTINE dealloc_out
DEALLOCATE(C_ST)
END SUBROUTINE dealloc_out


END MODULE chemistry_module
