!  ASIS method 0D Fortran code demonstrator
!
!  Reference:
!    ASIS v1.0: an adaptative solver for the simulation of atmospheric chemistry
!    Daniel Cariolle, Philippe Moinat, Hubert Teyssèdre, Luc Giraud, Béatrice Josse, and Franck Lefèvre
!    Geosci. Model Dev. Discuss., doi:10.5194/gmd-2016-281, 2016
!    Manuscript accepted for publication in the journal Geosci. Model Dev.
!    Published: 21 November 2016
!
!  SPECIES NUMBERING
!
MODULE num_species_module
!
! This module declares and sets the species indices for the RACMOBUS scheme
!
USE TYPES, ONLY : JPIM,JPRB

USE chemistry_module, ONLY : NB_SP

IMPLICIT NONE


! RACMOBUS SCHEME

INTEGER(KIND=JPIM), SAVE ::  JPO1D      
INTEGER(KIND=JPIM), SAVE ::  JPO3P      
INTEGER(KIND=JPIM), SAVE ::  JPO3       
INTEGER(KIND=JPIM), SAVE ::  JPO2       
INTEGER(KIND=JPIM), SAVE ::  JPH        
INTEGER(KIND=JPIM), SAVE ::  JPOH       
INTEGER(KIND=JPIM), SAVE ::  JPHO2      
INTEGER(KIND=JPIM), SAVE ::  JPH2O2     
INTEGER(KIND=JPIM), SAVE ::  JPN        
INTEGER(KIND=JPIM), SAVE ::  JPNO       
INTEGER(KIND=JPIM), SAVE ::  JPNO2      
INTEGER(KIND=JPIM), SAVE ::  JPNO3      
INTEGER(KIND=JPIM), SAVE ::  JPN2O5     
INTEGER(KIND=JPIM), SAVE ::  JPHNO3     
INTEGER(KIND=JPIM), SAVE ::  JPHNO3s    
INTEGER(KIND=JPIM), SAVE ::  JPHNO4     
INTEGER(KIND=JPIM), SAVE ::  JPN2O      
INTEGER(KIND=JPIM), SAVE ::  JPCH4      
INTEGER(KIND=JPIM), SAVE ::  JPH2       
INTEGER(KIND=JPIM), SAVE ::  JPH2O      
INTEGER(KIND=JPIM), SAVE ::  JPH2Os     
INTEGER(KIND=JPIM), SAVE ::  JPPSC     
INTEGER(KIND=JPIM), SAVE ::  JPHCHO     
INTEGER(KIND=JPIM), SAVE ::  JPMO2      
INTEGER(KIND=JPIM), SAVE ::  JPOP1      
INTEGER(KIND=JPIM), SAVE ::  JPCH3O     
INTEGER(KIND=JPIM), SAVE ::  JPCH3      
INTEGER(KIND=JPIM), SAVE ::  JPCO       
INTEGER(KIND=JPIM), SAVE ::  JPCO2      
INTEGER(KIND=JPIM), SAVE ::  JPCl       
INTEGER(KIND=JPIM), SAVE ::  JPCl2      
INTEGER(KIND=JPIM), SAVE ::  JPClO      
INTEGER(KIND=JPIM), SAVE ::  JPOClO     
INTEGER(KIND=JPIM), SAVE ::  JPCl2O2    
INTEGER(KIND=JPIM), SAVE ::  JPHOCl     
INTEGER(KIND=JPIM), SAVE ::  JPHCl      
INTEGER(KIND=JPIM), SAVE ::  JPHCls     
INTEGER(KIND=JPIM), SAVE ::  JPClONO2   
INTEGER(KIND=JPIM), SAVE ::  JPClNO2    
INTEGER(KIND=JPIM), SAVE ::  JPCFC11    
INTEGER(KIND=JPIM), SAVE ::  JPCFC12     
INTEGER(KIND=JPIM), SAVE ::  JPCFC113   
INTEGER(KIND=JPIM), SAVE ::  JPCCl4     
INTEGER(KIND=JPIM), SAVE ::  JPCH3CCl3  
INTEGER(KIND=JPIM), SAVE ::  JPCH3Cl    
INTEGER(KIND=JPIM), SAVE ::  JPHCFC22   
INTEGER(KIND=JPIM), SAVE ::  JPBr       
INTEGER(KIND=JPIM), SAVE ::  JPBrO      
INTEGER(KIND=JPIM), SAVE ::  JPHOBr     
INTEGER(KIND=JPIM), SAVE ::  JPHBr      
INTEGER(KIND=JPIM), SAVE ::  JPHBrs     
INTEGER(KIND=JPIM), SAVE ::  JPBrONO2   
INTEGER(KIND=JPIM), SAVE ::  JPBrCl     
INTEGER(KIND=JPIM), SAVE ::  JPCH3Br    
INTEGER(KIND=JPIM), SAVE ::  JPH1211    
INTEGER(KIND=JPIM), SAVE ::  JPH1301    
INTEGER(KIND=JPIM), SAVE ::  JPHONO     
 INTEGER(KIND=JPIM), SAVE ::  JPDMS      
INTEGER(KIND=JPIM), SAVE ::  JPSO2      
INTEGER(KIND=JPIM), SAVE ::  JPETH      
 INTEGER(KIND=JPIM), SAVE ::  JPHC3      
 INTEGER(KIND=JPIM), SAVE ::  JPHC5      
 INTEGER(KIND=JPIM), SAVE ::  JPHC8      
 INTEGER(KIND=JPIM), SAVE ::  JPETE      
 INTEGER(KIND=JPIM), SAVE ::  JPOLT      
 INTEGER(KIND=JPIM), SAVE ::  JPOLTP      
 INTEGER(KIND=JPIM), SAVE ::  JPOLI      
 INTEGER(KIND=JPIM), SAVE ::  JPDIEN     
 INTEGER(KIND=JPIM), SAVE ::  JPISO      
 INTEGER(KIND=JPIM), SAVE ::  JPAPI      
 INTEGER(KIND=JPIM), SAVE ::  JPLIM      
 INTEGER(KIND=JPIM), SAVE ::  JPTOL      
  INTEGER(KIND=JPIM), SAVE ::  JPADDT     
 INTEGER(KIND=JPIM), SAVE ::  JPXYL      
 INTEGER(KIND=JPIM), SAVE ::  JPADDX      
 INTEGER(KIND=JPIM), SAVE ::  JPCSL      
 INTEGER(KIND=JPIM), SAVE ::  JPADDC      
!INTEGER(KIND=JPIM), SAVE ::  JPAROMATIC 
INTEGER(KIND=JPIM), SAVE ::  JPALD      
INTEGER(KIND=JPIM), SAVE ::  JPKET      
 INTEGER(KIND=JPIM), SAVE ::  JPGLY      
 INTEGER(KIND=JPIM), SAVE ::  JPMGLY     
 INTEGER(KIND=JPIM), SAVE ::  JPDCB      
INTEGER(KIND=JPIM), SAVE ::  JPMACR     
 INTEGER(KIND=JPIM), SAVE ::  JPUDD      
 INTEGER(KIND=JPIM), SAVE ::  JPHKET     
INTEGER(KIND=JPIM), SAVE ::  JPONIT     
INTEGER(KIND=JPIM), SAVE ::  JPPAN      
 INTEGER(KIND=JPIM), SAVE ::  JPTPAN     
INTEGER(KIND=JPIM), SAVE ::  JPOP2      
 INTEGER(KIND=JPIM), SAVE ::  JPPAA      
 INTEGER(KIND=JPIM), SAVE ::  JPETHP     
 INTEGER(KIND=JPIM), SAVE ::  JPHC3P     
 INTEGER(KIND=JPIM), SAVE ::  JPHC5P     
INTEGER(KIND=JPIM), SAVE ::  JPLINO3    
INTEGER(KIND=JPIM), SAVE ::  JPHC8P     
 INTEGER(KIND=JPIM), SAVE ::  JPETEP     
INTEGER(KIND=JPIM), SAVE ::  JPOLIP     
INTEGER(KIND=JPIM), SAVE ::  JPISOP     
 INTEGER(KIND=JPIM), SAVE ::  JPAPIP     
 INTEGER(KIND=JPIM), SAVE ::  JPLIMP     
INTEGER(KIND=JPIM), SAVE ::  JPPHO      
INTEGER(KIND=JPIM), SAVE ::  JPTOLP     
 INTEGER(KIND=JPIM), SAVE ::  JPXYLP     
 INTEGER(KIND=JPIM), SAVE ::  JPCSLP     
INTEGER(KIND=JPIM), SAVE ::  JPACO3     
 INTEGER(KIND=JPIM), SAVE ::  JPTCO3     
 INTEGER(KIND=JPIM), SAVE ::  JPKETP     
INTEGER(KIND=JPIM), SAVE ::  JPOLNN     
 INTEGER(KIND=JPIM), SAVE ::  JPOLND     
INTEGER(KIND=JPIM), SAVE ::  JPXO2      
INTEGER(KIND=JPIM), SAVE ::  JPSULF     
 INTEGER(KIND=JPIM), SAVE ::  JPH2SO4    
!INTEGER(KIND=JPIM), SAVE ::  JPOx       
!INTEGER(KIND=JPIM), SAVE ::  JPNOx      
!INTEGER(KIND=JPIM), SAVE ::  JPClOx     
!INTEGER(KIND=JPIM), SAVE ::  JPBrOx     
INTEGER(KIND=JPIM), SAVE ::  JPDUMMY    



CONTAINS

SUBROUTINE num_species
!
! This routine defines the species indices for the RACMOBUS scheme
!
IMPLICIT NONE

INTEGER(KIND=JPIM) :: JS

! RACMOBUS MODEL

JS=0
JS=JS+1 ; JPO1D =JS
JS=JS+1 ; JPO3P =JS
JS=JS+1 ; JPO3 =JS
JS=JS+1 ; JPO2 =JS
JS=JS+1 ; JPH =JS
JS=JS+1 ; JPOH =JS
JS=JS+1 ; JPHO2 =JS
JS=JS+1 ; JPH2O2 =JS
JS=JS+1 ; JPN =JS
JS=JS+1 ; JPNO =JS
JS=JS+1 ; JPNO2 =JS
JS=JS+1 ; JPNO3 =JS
JS=JS+1 ; JPN2O5 =JS
JS=JS+1 ; JPHNO3 =JS
!JS=JS+1 ; JPHNO3s =JS
JS=JS+1 ; JPHNO4 =JS
JS=JS+1 ; JPN2O =JS
JS=JS+1 ; JPCH4 =JS
JS=JS+1 ; JPH2 =JS
JS=JS+1 ; JPH2O =JS
!JS=JS+1 ; JPH2Os =JS
! JPPSC=JPH2Os
JS=JS+1 ; JPPSC = JS
JS=JS+1 ; JPHCHO =JS
JS=JS+1 ; JPMO2 =JS
JS=JS+1 ; JPOP1 =JS
JS=JS+1 ; JPCH3O =JS
JS=JS+1 ; JPCH3 =JS
JS=JS+1 ; JPCO =JS
JS=JS+1 ; JPCO2 =JS
JS=JS+1 ; JPCl =JS
JS=JS+1 ; JPCl2 =JS
JS=JS+1 ; JPClO =JS
JS=JS+1 ; JPOClO =JS
JS=JS+1 ; JPCl2O2 =JS
JS=JS+1 ; JPHOCl =JS
JS=JS+1 ; JPHCl =JS
!JS=JS+1 ; JPHCls =JS
JS=JS+1 ; JPClONO2 =JS
JS=JS+1 ; JPClNO2 =JS
JS=JS+1 ; JPCFC11 =JS
JS=JS+1 ; JPCFC12 =JS 
JS=JS+1 ; JPCFC113 =JS
JS=JS+1 ; JPCCl4 =JS
JS=JS+1 ; JPCH3CCl3 =JS
JS=JS+1 ; JPCH3Cl =JS
JS=JS+1 ; JPHCFC22 =JS
JS=JS+1 ; JPBr =JS
JS=JS+1 ; JPBrO =JS
JS=JS+1 ; JPHOBr =JS
JS=JS+1 ; JPHBr =JS
!JS=JS+1 ; JPHBrs =JS
JS=JS+1 ; JPBrONO2 =JS
JS=JS+1 ; JPBrCl =JS
JS=JS+1 ; JPCH3Br =JS
JS=JS+1 ; JPH1211 =JS
JS=JS+1 ; JPH1301 =JS
JS=JS+1 ; JPHONO =JS
 JS=JS+1 ; JPDMS =JS
JS=JS+1 ; JPSO2 =JS
JS=JS+1 ; JPETH =JS
 JS=JS+1 ; JPHC3 =JS
 JS=JS+1 ; JPHC5 =JS
 JS=JS+1 ; JPHC8 =JS
 JS=JS+1 ; JPETE =JS
 JS=JS+1 ; JPOLT =JS
 JS=JS+1 ; JPOLTP =JS
 JS=JS+1 ; JPOLI =JS
 JS=JS+1 ; JPDIEN =JS
 JS=JS+1 ; JPISO =JS
 JS=JS+1 ; JPAPI =JS
 JS=JS+1 ; JPLIM =JS
 JS=JS+1 ; JPTOL =JS
 JS=JS+1 ; JPADDT =JS
 JS=JS+1 ; JPXYL =JS
 JS=JS+1 ; JPADDX =JS
 JS=JS+1 ; JPCSL =JS
 JS=JS+1 ; JPADDC =JS
!JS=JS+1 ; JPAROMATIC =JS
JS=JS+1 ; JPALD =JS
JS=JS+1 ; JPKET =JS
 JS=JS+1 ; JPGLY =JS
 JS=JS+1 ; JPMGLY =JS
 JS=JS+1 ; JPDCB =JS
JS=JS+1 ; JPMACR =JS
 JS=JS+1 ; JPUDD =JS
 JS=JS+1 ; JPHKET =JS
JS=JS+1 ; JPONIT =JS
JS=JS+1 ; JPPAN =JS
 JS=JS+1 ; JPTPAN =JS
JS=JS+1 ; JPOP2 =JS
 JS=JS+1 ; JPPAA =JS
 JS=JS+1 ; JPETHP =JS
 JS=JS+1 ; JPHC3P =JS
 JS=JS+1 ; JPHC5P =JS
JS=JS+1 ; JPLINO3 =JS
JS=JS+1 ; JPHC8P =JS
 JS=JS+1 ; JPETEP =JS
JS=JS+1 ; JPOLIP =JS
JS=JS+1 ; JPISOP =JS
 JS=JS+1 ; JPAPIP =JS
 JS=JS+1 ; JPLIMP =JS
JS=JS+1 ; JPPHO =JS
JS=JS+1 ; JPTOLP =JS
 JS=JS+1 ; JPXYLP =JS
 JS=JS+1 ; JPCSLP =JS
JS=JS+1 ; JPACO3 =JS
 JS=JS+1 ; JPTCO3 =JS
 JS=JS+1 ; JPKETP =JS
JS=JS+1 ; JPOLNN =JS
 JS=JS+1 ; JPOLND =JS
JS=JS+1 ; JPXO2 =JS
JS=JS+1 ; JPSULF =JS
 JS=JS+1 ; JPH2SO4 =JS
!JS=JS+1 ; JPOx =JS
!JS=JS+1 ; JPNOx =JS
!JS=JS+1 ; JPClOx =JS
!JS=JS+1 ; JPBrOx =JS
JS=JS+1 ; JPDUMMY =JS
NB_SP=JS

write(*,*) 'Number of species  NB_SP=',NB_SP

END SUBROUTINE num_species

END MODULE num_species_module
