!  ASIS method 0D Fortran code demonstrator
!
!  Reference:
!    ASIS v1.0: an adaptative solver for the simulation of atmospheric chemistry
!    Daniel Cariolle, Philippe Moinat, Hubert Teyssèdre, Luc Giraud, Béatrice Josse, and Franck Lefèvre
!    Geosci. Model Dev. Discuss., doi:10.5194/gmd-2016-281, 2016
!    Manuscript accepted for publication in the journal Geosci. Model Dev.
!    Published: 21 November 2016
!
!  INITIAL CONDITIONS AND SURFACE EMISSION FLUXES
!    initial conditions are read from the file initfile specified by the namelist
!    emission fluxes are read from the file emisfile specified by the namelist
!
MODULE M_set_conditions
!
USE TYPES, ONLY : JPIM,JPRB
USE case_module, ONLY : IU_INI,initfile, &
   &                    IU_EMI,emisfile, &
   &                    ef,zhcl, &
   &                    PHNM,PH2O

USE chemistry_module, ONLY : NB_SP,EMIRATE,CONC
USE num_species_module

IMPLICIT NONE


CONTAINS

SUBROUTINE set_inicon
IMPLICIT NONE
 ! read mol/mol_air mixing ratios
 call read_condition_file(initfile,IU_INI,CONC,NB_SP)
 ! convert to  molec/cm3
 CONC = CONC * PHNM
 CONC(JPH2O) = PH2O
END SUBROUTINE set_inicon

SUBROUTINE set_emis
IMPLICIT NONE
 ! read  molecules/cm2/s emission fluxes
 call read_condition_file(emisfile,IU_EMI,EMIRATE,NB_SP)
 ! apply emission factor ef
 ! spread over boundary layer to get injection rate   molecules/cm3/s  
 EMIRATE = EMIRATE *ef/zhcl
END SUBROUTINE set_emis


SUBROUTINE read_condition_file(infile,iun,cond,idim)
!
IMPLICIT NONE

CHARACTER(LEN=*) :: infile
INTEGER(KIND=JPIM) :: iun
REAL(KIND=JPRB), DIMENSION(idim) :: cond
INTEGER(KIND=JPIM) :: idim

REAL(KIND=JPRB) :: val
CHARACTER(LEN=72) :: keyw
INTEGER(KIND=JPIM) :: l,ierr

! RACMOBUS MODEL

open(iun,file=infile,form="formatted",status="old",action="read")
do l=1,2
   read(iun,*)  ! 2 header lines
enddo
do l=1,idim
   read(iun,*,iostat=ierr) val,keyw
   if(ierr/=0) then
      cycle
   end if

   select case (TRIM(keyw))
      case ('O1D')  ;  cond(JPO1D) = val 
      case ('O3P')  ;  cond(JPO3P) = val 
      case ('O3')  ;  cond(JPO3) = val 
      case ('O2')  ;  cond(JPO2) = val 
      case ('H')  ;  cond(JPH) = val 
      case ('OH')  ;  cond(JPOH) = val 
      case ('HO2')  ;  cond(JPHO2) = val 
      case ('H2O2')  ;  cond(JPH2O2) = val 
      case ('N')  ;  cond(JPN) = val 
      case ('NO')  ;  cond(JPNO) = val 
      case ('NO2')  ;  cond(JPNO2) = val 
      case ('NO3')  ;  cond(JPNO3) = val 
      case ('N2O5')  ;  cond(JPN2O5) = val 
      case ('HNO3')  ;  cond(JPHNO3) = val 
!      case ('HNO3s')  ;  cond(JPHNO3s) = val 
      case ('HNO4')  ;  cond(JPHNO4) = val 
      case ('N2O')  ;  cond(JPN2O) = val 
      case ('CH4')  ;  cond(JPCH4) = val 
      case ('H2')  ;  cond(JPH2) = val 
      case ('H2O')  ;  cond(JPH2O) = val 
!      case ('H2Os')  ;  cond(JPH2Os) = val 
      case ('PSC')  ;  cond(JPPSC) = val
      case ('HCHO')  ;  cond(JPHCHO) = val 
      case ('MO2')  ;  cond(JPMO2) = val 
      case ('OP1')  ;  cond(JPOP1) = val 
      case ('CH3O')  ;  cond(JPCH3O) = val 
      case ('CH3')  ;  cond(JPCH3) = val 
      case ('CO')  ;  cond(JPCO) = val 
      case ('CO2')  ;  cond(JPCO2) = val 
      case ('Cl')  ;  cond(JPCl) = val 
      case ('Cl2')  ;  cond(JPCl2) = val 
      case ('ClO')  ;  cond(JPClO) = val 
      case ('OClO')  ;  cond(JPOClO) = val 
      case ('Cl2O2')  ;  cond(JPCl2O2) = val 
      case ('HOCl')  ;  cond(JPHOCl) = val 
      case ('HCl')  ;  cond(JPHCl) = val 
!      case ('HCls')  ;  cond(JPHCls) = val 
      case ('ClONO2')  ;  cond(JPClONO2) = val 
      case ('ClNO2')  ;  cond(JPClNO2) = val 
      case ('CFC11')  ;  cond(JPCFC11) = val 
      case ('CFC12')  ;  cond(JPCFC12) = val 
      case ('CFC113')  ;  cond(JPCFC113) = val 
      case ('CCl4')  ;  cond(JPCCl4) = val 
      case ('CH3CCl3')  ;  cond(JPCH3CCl3) = val 
      case ('CH3Cl')  ;  cond(JPCH3Cl) = val 
      case ('HCFC22')  ;  cond(JPHCFC22) = val 
      case ('Br')  ;  cond(JPBr) = val 
      case ('BrO')  ;  cond(JPBrO) = val 
      case ('HOBr')  ;  cond(JPHOBr) = val 
      case ('HBr')  ;  cond(JPHBr) = val 
!      case ('HBrs')  ;  cond(JPHBrs) = val 
      case ('BrONO2')  ;  cond(JPBrONO2) = val 
      case ('BrCl')  ;  cond(JPBrCl) = val 
      case ('CH3Br')  ;  cond(JPCH3Br) = val 
      case ('H1211')  ;  cond(JPH1211) = val 
      case ('H1301')  ;  cond(JPH1301) = val 
      case ('HONO')  ;  cond(JPHONO) = val 
      case ('DMS')  ;  cond(JPDMS) = val 
      case ('SO2')  ;  cond(JPSO2) = val 
      case ('ETH')  ;  cond(JPETH) = val 
      case ('HC3')  ;  cond(JPHC3) = val 
      case ('HC5')  ;  cond(JPHC5) = val 
      case ('HC8')  ;  cond(JPHC8) = val 
      case ('ETE')  ;  cond(JPETE) = val 
      case ('OLT')  ;  cond(JPOLT) = val 
      case ('OLTP')  ;  cond(JPOLTP) = val 
      case ('OLI')  ;  cond(JPOLI) = val 
      case ('DIEN')  ;  cond(JPDIEN) = val 
      case ('ISO')  ;  cond(JPISO) = val 
      case ('API')  ;  cond(JPAPI) = val 
      case ('LIM')  ;  cond(JPLIM) = val 
      case ('TOL')  ;  cond(JPTOL) = val 
      case ('ADDT')  ;  cond(JPADDT) = val 
      case ('XYL')  ;  cond(JPXYL) = val 
      case ('ADDX')  ;  cond(JPADDX) = val 
      case ('CSL')  ;  cond(JPCSL) = val 
      case ('ADDC')  ;  cond(JPADDC) = val 
!      case ('AROMATIC')  ;  cond(JPAROMATIC) = val 
      case ('ALD')  ;  cond(JPALD) = val 
      case ('KET')  ;  cond(JPKET) = val 
      case ('GLY')  ;  cond(JPGLY) = val 
      case ('MGLY')  ;  cond(JPMGLY) = val 
      case ('DCB')  ;  cond(JPDCB) = val 
      case ('MACR')  ;  cond(JPMACR) = val 
      case ('UDD')  ;  cond(JPUDD) = val 
      case ('HKET')  ;  cond(JPHKET) = val 
      case ('ONIT')  ;  cond(JPONIT) = val 
      case ('PAN')  ;  cond(JPPAN) = val 
      case ('TPAN')  ;  cond(JPTPAN) = val 
      case ('OP2')  ;  cond(JPOP2) = val 
      case ('PAA')  ;  cond(JPPAA) = val 
      case ('ETHP')  ;  cond(JPETHP) = val 
      case ('HC3P')  ;  cond(JPHC3P) = val 
      case ('HC5P')  ;  cond(JPHC5P) = val 
      case ('LINO3')  ;  cond(JPLINO3) = val 
      case ('HC8P')  ;  cond(JPHC8P) = val 
      case ('ETEP')  ;  cond(JPETEP) = val 
      case ('OLIP')  ;  cond(JPOLIP) = val 
      case ('ISOP')  ;  cond(JPISOP) = val 
      case ('APIP')  ;  cond(JPAPIP) = val 
      case ('LIMP')  ;  cond(JPLIMP) = val 
      case ('PHO')  ;  cond(JPPHO) = val 
      case ('TOLP')  ;  cond(JPTOLP) = val 
      case ('XYLP')  ;  cond(JPXYLP) = val 
      case ('CSLP')  ;  cond(JPCSLP) = val 
      case ('ACO3')  ;  cond(JPACO3) = val 
      case ('TCO3')  ;  cond(JPTCO3) = val 
      case ('KETP')  ;  cond(JPKETP) = val 
      case ('OLNN')  ;  cond(JPOLNN) = val 
      case ('OLND')  ;  cond(JPOLND) = val 
      case ('XO2')  ;  cond(JPXO2) = val 
      case ('SULF')  ;  cond(JPSULF) = val 
      case ('H2SO4')  ;  cond(JPH2SO4) = val 
!      case ('Ox')  ;  cond(JPOx) = val 
!      case ('NOx')  ;  cond(JPNOx) = val 
!      case ('ClOx')  ;  cond(JPClOx) = val 
!      case ('BrOx')  ;  cond(JPBrOx) = val 
      case default
        write(*,*) infile,'  :  unknown entry   ',TRIM(keyw) 
  end select

enddo
close(iun)


END SUBROUTINE read_condition_file

END MODULE M_set_conditions
