!  ASIS method 0D Fortran code demonstrator
!
!  Reference:
!    ASIS v1.0: an adaptative solver for the simulation of atmospheric chemistry
!    Daniel Cariolle, Philippe Moinat, Hubert Teyssèdre, Luc Giraud, Béatrice Josse, and Franck Lefèvre
!    Geosci. Model Dev. Discuss., doi:10.5194/gmd-2016-281, 2016
!    Manuscript accepted for publication in the journal Geosci. Model Dev.
!    Published: 21 November 2016
!
!  STORE OUTPUT IN FILES
!
MODULE M_storeC


USE TYPES , ONLY : JPIM,JPRB

CONTAINS

SUBROUTINE storec(C_ST,PHNM,kimax,dt,IU_OUT,case_name)

USE num_species_module

IMPLICIT NONE

INTEGER(KIND=JPIM) :: kimax
REAL(KIND=JPRB), DIMENSION(kimax,NB_SP) :: C_ST
REAL(KIND=JPRB) :: PHNM
REAL(KIND=JPRB) :: dt		! time step in seconds
INTEGER(KIND=JPIM) :: IU_OUT
CHARACTER(LEN=15) case_name

INTEGER(KIND=JPIM) :: ki,n

CHARACTER(LEN=30) filename
CHARACTER(LEN=19) outdir

outdir='OUT_'//TRIM(case_name)

call system('mkdir '//outdir)

call ctofile('OH',JPOH,C_ST(:,JPOH),kimax,dt,IU_OUT,outdir)
call ctofile('HO2',JPHO2,C_ST(:,JPHO2),kimax,dt,IU_OUT,outdir)
call ctofile('H2O2',JPH2O2,C_ST(:,JPH2O2),kimax,dt,IU_OUT,outdir)
call ctofile('O1D',JPO1D,C_ST(:,JPO1D),kimax,dt,IU_OUT,outdir)
call ctofile('HCHO',JPHCHO,C_ST(:,JPHCHO),kimax,dt,IU_OUT,outdir)
call ctofile('ClO',JPClO,C_ST(:,JPClO),kimax,dt,IU_OUT,outdir)
call ctofile('BrO',JPBrO,C_ST(:,JPBrO),kimax,dt,IU_OUT,outdir)
!
call ctofile('ACO3',JPACO3,C_ST(:,JPACO3),kimax,dt,IU_OUT,outdir)
call ctofile('ADDC',JPADDC,C_ST(:,JPADDC),kimax,dt,IU_OUT,outdir)
call ctofile('ADDT',JPADDT,C_ST(:,JPADDT),kimax,dt,IU_OUT,outdir)
call ctofile('ADDX',JPADDX,C_ST(:,JPADDX),kimax,dt,IU_OUT,outdir)
call ctofile('ALD',JPALD,C_ST(:,JPALD),kimax,dt,IU_OUT,outdir)
call ctofile('API',JPAPI,C_ST(:,JPAPI),kimax,dt,IU_OUT,outdir)
call ctofile('APIP',JPAPIP,C_ST(:,JPAPIP),kimax,dt,IU_OUT,outdir)
call ctofile('CSL',JPCSL,C_ST(:,JPCSL),kimax,dt,IU_OUT,outdir)
call ctofile('CSLP',JPCSLP,C_ST(:,JPCSLP),kimax,dt,IU_OUT,outdir)
call ctofile('DCB',JPDCB,C_ST(:,JPDCB),kimax,dt,IU_OUT,outdir)
call ctofile('DIEN',JPDIEN,C_ST(:,JPDIEN),kimax,dt,IU_OUT,outdir)
call ctofile('ETE',JPETE,C_ST(:,JPETE),kimax,dt,IU_OUT,outdir)
call ctofile('ETEP',JPETEP,C_ST(:,JPETEP),kimax,dt,IU_OUT,outdir)
call ctofile('ETH',JPETH,C_ST(:,JPETH),kimax,dt,IU_OUT,outdir)
call ctofile('ETHP',JPETHP,C_ST(:,JPETHP),kimax,dt,IU_OUT,outdir)
call ctofile('GLY',JPGLY,C_ST(:,JPGLY),kimax,dt,IU_OUT,outdir)
call ctofile('HC3',JPHC3,C_ST(:,JPHC3),kimax,dt,IU_OUT,outdir)
call ctofile('HC3P',JPHC3P,C_ST(:,JPHC3P),kimax,dt,IU_OUT,outdir)
call ctofile('HC5',JPHC5,C_ST(:,JPHC5),kimax,dt,IU_OUT,outdir)
call ctofile('HC5P',JPHC5P,C_ST(:,JPHC5P),kimax,dt,IU_OUT,outdir)
call ctofile('HC8',JPHC8,C_ST(:,JPHC8),kimax,dt,IU_OUT,outdir)
call ctofile('HC8P',JPHC8P,C_ST(:,JPHC8P),kimax,dt,IU_OUT,outdir)
call ctofile('HKET',JPHKET,C_ST(:,JPHKET),kimax,dt,IU_OUT,outdir)
call ctofile('ISO',JPISO,C_ST(:,JPISO),kimax,dt,IU_OUT,outdir)
call ctofile('ISOP',JPISOP,C_ST(:,JPISOP),kimax,dt,IU_OUT,outdir)
call ctofile('KET',JPKET,C_ST(:,JPKET),kimax,dt,IU_OUT,outdir)
call ctofile('KETP',JPKETP,C_ST(:,JPKETP),kimax,dt,IU_OUT,outdir)
call ctofile('LIM',JPLIM,C_ST(:,JPLIM),kimax,dt,IU_OUT,outdir)
call ctofile('LIMP',JPLIMP,C_ST(:,JPLIMP),kimax,dt,IU_OUT,outdir)
call ctofile('LINO3',JPLINO3,C_ST(:,JPLINO3),kimax,dt,IU_OUT,outdir)
call ctofile('MACR',JPMACR,C_ST(:,JPMACR),kimax,dt,IU_OUT,outdir)
call ctofile('MGLY',JPMGLY,C_ST(:,JPMGLY),kimax,dt,IU_OUT,outdir)
call ctofile('MO2',JPMO2,C_ST(:,JPMO2),kimax,dt,IU_OUT,outdir)
call ctofile('OP1',JPOP1,C_ST(:,JPOP1),kimax,dt,IU_OUT,outdir)
call ctofile('OP2',JPOP2,C_ST(:,JPOP2),kimax,dt,IU_OUT,outdir)
call ctofile('OLI',JPOLI,C_ST(:,JPOLI),kimax,dt,IU_OUT,outdir)
call ctofile('OLIP',JPOLIP,C_ST(:,JPOLIP),kimax,dt,IU_OUT,outdir)
call ctofile('OLND',JPOLND,C_ST(:,JPOLND),kimax,dt,IU_OUT,outdir)
call ctofile('OLNN',JPOLNN,C_ST(:,JPOLNN),kimax,dt,IU_OUT,outdir)
call ctofile('OLT',JPOLT,C_ST(:,JPOLT),kimax,dt,IU_OUT,outdir)
call ctofile('OLTP',JPOLTP,C_ST(:,JPOLTP),kimax,dt,IU_OUT,outdir)
call ctofile('ONIT',JPONIT,C_ST(:,JPONIT),kimax,dt,IU_OUT,outdir)
call ctofile('PAA',JPPAA,C_ST(:,JPPAA),kimax,dt,IU_OUT,outdir)
call ctofile('PAN',JPPAN,C_ST(:,JPPAN),kimax,dt,IU_OUT,outdir)
call ctofile('PHO',JPPHO,C_ST(:,JPPHO),kimax,dt,IU_OUT,outdir)
call ctofile('TCO3',JPTCO3,C_ST(:,JPTCO3),kimax,dt,IU_OUT,outdir)
call ctofile('TOL',JPTOL,C_ST(:,JPTOL),kimax,dt,IU_OUT,outdir)
call ctofile('TOLP',JPTOLP,C_ST(:,JPTOLP),kimax,dt,IU_OUT,outdir)
call ctofile('PAN',JPPAN,C_ST(:,JPPAN),kimax,dt,IU_OUT,outdir)
call ctofile('UDD',JPUDD,C_ST(:,JPUDD),kimax,dt,IU_OUT,outdir)
call ctofile('XO2',JPXO2,C_ST(:,JPXO2),kimax,dt,IU_OUT,outdir)
call ctofile('XYL',JPXYL,C_ST(:,JPXYL),kimax,dt,IU_OUT,outdir)
call ctofile('XYLP',JPXYLP,C_ST(:,JPXYLP),kimax,dt,IU_OUT,outdir)



! write concentrations (mol/mol)
C_ST = C_ST/PHNM

   filename=TRIM(outdir)//'/ZO.dat'
   open(IU_OUT,file=filename)
   do ki=1,kimax
      write(IU_OUT,111) dt/3600.*ki,C_ST(ki,JPO3), &
                     C_ST(ki,JPO3P),C_ST(ki,JPO1D)
   enddo
   close(IU_OUT)

   filename=TRIM(outdir)//'/ZN.dat'
   open(IU_OUT,file=filename)
   do ki=1,kimax
      write(IU_OUT,111) dt/3600.*ki,C_ST(ki,JPNO), &
                     C_ST(ki,JPNO2),C_ST(ki,JPNO3), &
                     C_ST(ki,JPN2O5),C_ST(ki,JPHNO3), &
                     C_ST(ki,JPNO)+C_ST(ki,JPNO2)
   enddo
   close(IU_OUT)

   filename=TRIM(outdir)//'/ZH.dat'
   open(IU_OUT,file=filename)
   do ki=1,kimax
      write(IU_OUT,111) dt/3600.*ki,C_ST(ki,JPH), &
                     C_ST(ki,JPOH),C_ST(ki,JPHO2), &
                     C_ST(ki,JPH2O2)
   enddo
   close(IU_OUT)

   filename=TRIM(outdir)//'/ZCl.dat'
   open(IU_OUT,file=filename)
   do ki=1,kimax
      write(IU_OUT,111) dt/3600.*ki,C_ST(ki,JPCl2), &
                     C_ST(ki,JPCl),C_ST(ki,JPClO), &
                     C_ST(ki,JPCl2O2)
   enddo
   close(IU_OUT)

   filename=TRIM(outdir)//'/ZBr.dat'
   open(IU_OUT,file=filename)
   do ki=1,kimax
      write(IU_OUT,111) dt/3600.*ki,C_ST(ki,JPBr), &
                     C_ST(ki,JPBrO),C_ST(ki,JPBrCl), &
                     C_ST(ki,JPHBr),C_ST(ki,JPHOBr),&
                     C_ST(ki,JPBrONO2)
   enddo
   close(IU_OUT)

   filename=TRIM(outdir)//'/ZC.dat'
   open(IU_OUT,file=filename)
   do ki=1,kimax
      write(IU_OUT,111) dt/3600.*ki,C_ST(ki,JPCO), &
                     C_ST(ki,JPHCHO),C_ST(ki,JPCH3),&
                     C_ST(ki,JPCH3O)
   enddo
   close(IU_OUT)

111 format(f7.2,8(1x,E12.4E3))

END SUBROUTINE storeC


SUBROUTINE ctofile(SPEC,JSPEC,C_ST,kimax,dt,IU_OUT,outdir)
IMPLICIT NONE
CHARACTER(LEN=*) SPEC
INTEGER(KIND=JPIM) :: JSPEC,kimax,IU_OUT
REAL(KIND=JPRB), DIMENSION(kimax) :: C_ST
REAL(KIND=JPRB) :: dt
CHARACTER(LEN=19) outdir

INTEGER(KIND=JPIM) :: ki
CHARACTER(LEN=30) filename

! write concentrations (molec/cm3)
   filename=TRIM(outdir)//'/Z_'//TRIM(SPEC)//'.dat'
   open(IU_OUT,file=filename)
   do ki=1,kimax
      write(IU_OUT,*) dt/3600.*ki,C_ST(ki)
   enddo
   close(IU_OUT)

END SUBROUTINE ctofile


END module M_storeC

