!  ASIS method 0D Fortran code demonstrator
!
!  Reference:
!    ASIS v1.0: an adaptative solver for the simulation of atmospheric chemistry
!    Daniel Cariolle, Philippe Moinat, Hubert Teyssèdre, Luc Giraud, Béatrice Josse, and Franck Lefèvre
!    Geosci. Model Dev. Discuss., doi:10.5194/gmd-2016-281, 2016
!    Manuscript accepted for publication in the journal Geosci. Model Dev.
!    Published: 21 November 2016
!
!  DESCRIPTION OF THE REACTIONS
!    reaction numbering
!    reaction description using the Zindice_phot, Zindice_3 and Zindice_4 structures
!	specifying the index and coefficient of the involved species
!
MODULE indices_reactions_module
!
! The following reactions are considered:
!    PHOTO- Photolysis reactions with index JPO_J*
!    TYPE3- Reactions of the form a*A --> b*B+c*C with index JP3_J*
!    TYPE4- Reactions of the form a*A+d*D --> b*B+c*C with index JP4_J*
!
!    NB_PHOT       : number of reactions of type PHOTO
!    NB_REACTION_3 : number of reactions of TYPE3
!    NB_REACTION_4 : number of reactions of TYPE4


USE TYPES , ONLY : JPIM,JPRB,Z3SPEC,Z4SPEC

USE chemistry_module, ONLY : NB_PHOT,NB_REACTION_3,NB_REACTION_4, &
   &                         Zindice_phot,Zindice_3,Zindice_4

USE num_species_module

IMPLICIT NONE


! RACMOBUS SCHEME

INTEGER(KIND=JPIM), SAVE ::   JPO_JNO2      !(JPO_JNO2)    ="NO2 --> O3P + NO"
INTEGER(KIND=JPIM), SAVE ::   JPO_JO3a      !(JPO_JO3a)    ="O3 --> O1D + O2"
INTEGER(KIND=JPIM), SAVE ::   JPO_JO3b      !(JPO_JO3b)    ="O3 --> O3P + O2"
INTEGER(KIND=JPIM), SAVE ::   JPO_JHONO     !(JPO_JHONO)   ="HONO --> OH + NO"
INTEGER(KIND=JPIM), SAVE ::   JPO_JHNO3     !(JPO_JHNO3)   ="HNO3 --> OH + NO2"
INTEGER(KIND=JPIM), SAVE ::   JPO_JHNO4a    !(JPO_JHNO4)   ="HNO4 --> x*HO2 + x*NO2 + (1-x)*OH + (1-x)*NO3"
INTEGER(KIND=JPIM), SAVE ::   JPO_JHNO4b   
INTEGER(KIND=JPIM), SAVE ::   JPO_JNO3a     !(JPO_JNO3a)   ="NO3 --> NO + O2"
INTEGER(KIND=JPIM), SAVE ::   JPO_JNO3b     !(JPO_JNO3b)   ="NO3 --> NO2 + O3P"
INTEGER(KIND=JPIM), SAVE ::   JPO_JH2O2     !(JPO_JH2O2)   ="H2O2 --> OH + OH"
INTEGER(KIND=JPIM), SAVE ::   JPO_JHCHOa    !(JPO_JHCHOa)  ="HCHO --> H2 + CO"
INTEGER(KIND=JPIM), SAVE ::   JPO_JHCHOb_i   !(JPO_JHCHOb)  ="HCHO --> H + CO + HO2"
INTEGER(KIND=JPIM), SAVE ::   JPO_JHCHOb_ii
INTEGER(KIND=JPIM), SAVE ::   JPO_JALD_i      !(JPO_JALD)    ="ALD --> CH3 + HO2 + CO"
INTEGER(KIND=JPIM), SAVE ::   JPO_JALD_ii    
INTEGER(KIND=JPIM), SAVE ::   JPO_JOP1      !(JPO_JOP1)    ="OP1 -->  CH3O + OH"
INTEGER(KIND=JPIM), SAVE ::   JPO_JOP2_i      !(JPO_JOP2)    ="OP2 --> ALD + HO2 + OH"
INTEGER(KIND=JPIM), SAVE ::   JPO_JOP2_ii    
INTEGER(KIND=JPIM), SAVE ::   JPO_JPAA_i      !(JPO_JPAA)    ="PAA --> CH3 + OH + CO2"
INTEGER(KIND=JPIM), SAVE ::   JPO_JPAA_ii    
INTEGER(KIND=JPIM), SAVE ::   JPO_JKET      !(JPO_JKET)    ="KET --> ACO3 + ETHP"
INTEGER(KIND=JPIM), SAVE ::   JPO_JGLYa     !(JPO_JGLYa)   ="GLY --> 0.13*HCHO + 1.87*CO + 0.87*H2"
INTEGER(KIND=JPIM), SAVE ::   JPO_JGLYb_i     !(JPO_JGLYb)   ="GLY --> 0.45*HCHO + 1.55*CO + 0.80*HO2 + 0.15*H2"
INTEGER(KIND=JPIM), SAVE ::   JPO_JGLYb_ii   
INTEGER(KIND=JPIM), SAVE ::   JPO_JMGLY_i     !(JPO_JMGLY)   ="MGLY --> ACO3 + HO2 + CO"
INTEGER(KIND=JPIM), SAVE ::   JPO_JMGLY_ii   
INTEGER(KIND=JPIM), SAVE ::   JPO_JDCB      !(JPO_JDCB)    ="DCB --> TCO3 + HO2"
INTEGER(KIND=JPIM), SAVE ::   JPO_JONITa_i    !(JPO_JONITa)  ="ONIT --> HO2 + NO2 + 0.20*ALD + 0.80*KET"
INTEGER(KIND=JPIM), SAVE ::   JPO_JONITa_ii  
INTEGER(KIND=JPIM), SAVE ::   JPO_JONITb_i    !(JPO_JONITb)  ="ONIT --> NO2 + 0.341*HO2 + 0.564*RO2R + 0.095*RO2N + 0.152*R2O2 + &
INTEGER(KIND=JPIM), SAVE ::   JPO_JONITb_ii   
INTEGER(KIND=JPIM), SAVE ::   JPO_JMACR_i     !(JPO_JMACR)   ="MACR --> HCHO + ACO3 + CO + HO2"
INTEGER(KIND=JPIM), SAVE ::   JPO_JMACR_ii   
INTEGER(KIND=JPIM), SAVE ::   JPO_JHKET_i     !(JPO_JHKET)   ="HKET --> ACO3 + HCHO + HO2"
INTEGER(KIND=JPIM), SAVE ::   JPO_JHKET_ii   
INTEGER(KIND=JPIM), SAVE ::   JPO_JO2       !(JPO_JO2)     ="O2 --> O3P + O3P"
INTEGER(KIND=JPIM), SAVE ::   JPO_JN2O5     !(JPO_JN2O5)   ="N2O5 --> NOx + NOx + XX"
INTEGER(KIND=JPIM), SAVE ::   JPO_JH2O      !(JPO_JH2O)    ="H2O --> H + OH"
INTEGER(KIND=JPIM), SAVE ::   JPO_JN2O      !(JPO_JN2O)    ="N2O --> N2 + O1D"
INTEGER(KIND=JPIM), SAVE ::   JPO_JHOCl     !(JPO_JHOCl)   ="HOCl --> Cl + OH"
INTEGER(KIND=JPIM), SAVE ::   JPO_JClONO2a  !(JPO_JClONO2a)="ClONO2 --> Cl + NO3"
INTEGER(KIND=JPIM), SAVE ::   JPO_JClONO2b  !(JPO_JClONO2b)="ClONO2 --> ClO + NO2"
INTEGER(KIND=JPIM), SAVE ::   JPO_JCl2O2_i   !(JPO_JCl2O2)  ="Cl2O2 --> Cl + Cl + O2"
INTEGER(KIND=JPIM), SAVE ::   JPO_JCl2O2_ii
INTEGER(KIND=JPIM), SAVE ::   JPO_JHCl      !(JPO_JHCl)    ="HCl --> H + Cl"
INTEGER(KIND=JPIM), SAVE ::   JPO_JCl2      !(JPO_JCl2)    ="Cl2 --> Cl + Cl"
INTEGER(KIND=JPIM), SAVE ::   JPO_JClNO2    !(JPO_JClNO2)  ="ClNO2 --> Cl + NO2"
INTEGER(KIND=JPIM), SAVE ::   JPO_JBrO      !(JPO_JBrO)    ="BrO --> Br + O3P"
INTEGER(KIND=JPIM), SAVE ::   JPO_JBrONO2   !(JPO_JBrONO2) ="BrONO2 --> Br + NO3"
INTEGER(KIND=JPIM), SAVE ::   JPO_JHOBr     !(JPO_JHOBr)   ="HOBr --> Br + OH"
INTEGER(KIND=JPIM), SAVE ::   JPO_JOClO     !(JPO_JOClO)   ="OClO --> O3P + ClO"
INTEGER(KIND=JPIM), SAVE ::   JPO_JBrCl     !(JPO_JBrCl)   ="BrCl --> Br + Cl"
INTEGER(KIND=JPIM), SAVE ::   JPO_JCFC11_i   !(JPO_JCFC11)  ="CFC11 --> ClOx + ClOx + ClOx + XX"
INTEGER(KIND=JPIM), SAVE ::   JPO_JCFC11_ii
INTEGER(KIND=JPIM), SAVE ::   JPO_JCFC12    !(JPO_JCFC12)  ="CFC12 --> ClOx + ClOx + XX"
INTEGER(KIND=JPIM), SAVE ::   JPO_JCFC113_i   !(JPO_JCFC113) ="CFC113 --> ClOx + ClOx + ClOx + XX"
INTEGER(KIND=JPIM), SAVE ::   JPO_JCFC113_ii
INTEGER(KIND=JPIM), SAVE ::   JPO_JCCl4_i    !(JPO_JCCl4)   ="CCl4 --> ClOx + ClOx + ClOx + ClOx + XX"
INTEGER(KIND=JPIM), SAVE ::   JPO_JCCl4_ii
INTEGER(KIND=JPIM), SAVE ::   JPO_JCH3CCl3_i  !(JPO_JCH3CCl3)="CH3CCl3 --> ClOx + ClOx + ClOx + XX"
INTEGER(KIND=JPIM), SAVE ::   JPO_JCH3CCl3_ii
INTEGER(KIND=JPIM), SAVE ::   JPO_JCH3Cl    !(JPO_JCH3Cl)  ="CH3Cl --> ClOx + XX"
INTEGER(KIND=JPIM), SAVE ::   JPO_JHCFC22   !(JPO_JHCFC22) ="HCFC22 --> ClOx + XX"
INTEGER(KIND=JPIM), SAVE ::   JPO_JCH3Br    !(JPO_JCH3Br)  ="CH3Br --> BrOx + XX"
INTEGER(KIND=JPIM), SAVE ::   JPO_JH1211    !(JPO_JH1211)  ="H1211 --> BrOx + XX"
INTEGER(KIND=JPIM), SAVE ::   JPO_JH1301    !(JPO_JH1301)  ="H1301 --> BrOx + XX"
INTEGER(KIND=JPIM), SAVE ::   JPO_JNO       !(JPO_JNO)     ="NO --> N + O3P"
INTEGER(KIND=JPIM), SAVE ::   JPO_JCO2      !(JPO_JCO2)    ="CO_2 + hv -->  CO + O(3P)"
INTEGER(KIND=JPIM), SAVE ::   JPO_O1D_N2a       ! --> O3P + N2
INTEGER(KIND=JPIM), SAVE ::   JPO_OH_H2        
INTEGER(KIND=JPIM), SAVE ::   JPO_H2_O3P       
INTEGER(KIND=JPIM), SAVE ::   JPO_Cl_H2        
INTEGER(KIND=JPIM), SAVE ::   JPO_HNO4         
INTEGER(KIND=JPIM), SAVE ::   JPO_N2O5         
INTEGER(KIND=JPIM), SAVE ::   JPO_PAN          
INTEGER(KIND=JPIM), SAVE ::   JPO_TPAN         
INTEGER(KIND=JPIM), SAVE ::   JPO_O1D_H2       
INTEGER(KIND=JPIM), SAVE ::   JPO_Cl2O2        
!
   INTEGER(KIND=JPIM), SAVE ::   JP3_HO2_HO2a      ! -->  H2O2 + O2
   INTEGER(KIND=JPIM), SAVE ::   JP3_HO2_HO2b      ! + H2O -->  H2O2 + H2O + O2
   INTEGER(KIND=JPIM), SAVE ::   JP3_NO_NO        
   INTEGER(KIND=JPIM), SAVE ::   JP3_NO3_NO3_i      
   INTEGER(KIND=JPIM), SAVE ::   JP3_NO3_NO3_ii     
   INTEGER(KIND=JPIM), SAVE ::   JP3_MO2_MO2a      ! MO2 + MO2  -->  CH3O + CH3O + O2
   INTEGER(KIND=JPIM), SAVE ::   JP3_MO2_MO2b      ! MO2 + MO2  -->  HCHO  + O2 + XX
   INTEGER(KIND=JPIM), SAVE ::   JP3_ACO3_ACO3     ! ACO3 + ACO3  -->  MO2 + MO2
   INTEGER(KIND=JPIM), SAVE ::   JP3_OLNN_OLNN     ! OLNN + OLNN  --> ONIT + ONIT + HO2
   INTEGER(KIND=JPIM), SAVE ::   JP3_OLND_OLND_i    ! 0.504*HCHO + 1.21*ALD + 0.285*KET + ONIT + NO2
   INTEGER(KIND=JPIM), SAVE ::   JP3_OLND_OLND_ii  
   INTEGER(KIND=JPIM), SAVE ::   JP3_OLND_OLND_iii 
   INTEGER(KIND=JPIM), SAVE ::   JP3_XO2_XO2      
   INTEGER(KIND=JPIM), SAVE ::   JP3_OH_OHa        ! -->  H2O + O3P
   INTEGER(KIND=JPIM), SAVE ::   JP3_OH_OHb        ! -->   H2O2
   INTEGER(KIND=JPIM), SAVE ::   JP3_ClO_ClOa      ! -->  Cl2O2
   INTEGER(KIND=JPIM), SAVE ::   JP3_ClO_ClOb      ! -->  Cl2 + O2
   INTEGER(KIND=JPIM), SAVE ::   JP3_ClO_ClOc      ! -->  OClO + Cl
   INTEGER(KIND=JPIM), SAVE ::   JP3_BrO_BrO      
   INTEGER(KIND=JPIM), SAVE ::   JP3_O3P_O3P      
   INTEGER(KIND=JPIM), SAVE ::   JP4_O3P_O2       
   INTEGER(KIND=JPIM), SAVE ::   JP4_O3P_O3       
   INTEGER(KIND=JPIM), SAVE ::   JP4_O1D_O2       
   INTEGER(KIND=JPIM), SAVE ::   JP4_O1D_H2O      
   INTEGER(KIND=JPIM), SAVE ::   JP4_O3_OH        
   INTEGER(KIND=JPIM), SAVE ::   JP4_O3_HO2       
   INTEGER(KIND=JPIM), SAVE ::   JP4_OH_HO2        
   INTEGER(KIND=JPIM), SAVE ::   JP4_H2O2_OH       
   INTEGER(KIND=JPIM), SAVE ::   JP4_O3P_NO       
   INTEGER(KIND=JPIM), SAVE ::   JP4_O3P_NO2a      ! -->  NO + O2
   INTEGER(KIND=JPIM), SAVE ::   JP4_O3P_NO2b      ! -->  NO3
   INTEGER(KIND=JPIM), SAVE ::   JP4_OH_NO        
   INTEGER(KIND=JPIM), SAVE ::   JP4_OH_NO2       
   INTEGER(KIND=JPIM), SAVE ::   JP4_OH_NO3       
   INTEGER(KIND=JPIM), SAVE ::   JP4_HO2_NOa      
   INTEGER(KIND=JPIM), SAVE ::   JP4_HO2_NO2      
   INTEGER(KIND=JPIM), SAVE ::   JP4_HO2_NO3a      ! --> HNO3
   INTEGER(KIND=JPIM), SAVE ::   JP4_HO2_NO3b      ! --> NO2 + OH
   INTEGER(KIND=JPIM), SAVE ::   JP4_OH_HONO      
   INTEGER(KIND=JPIM), SAVE ::   JP4_OH_HNO3      
   INTEGER(KIND=JPIM), SAVE ::   JP4_OH_HNO4      
   INTEGER(KIND=JPIM), SAVE ::   JP4_O3_NO        
   INTEGER(KIND=JPIM), SAVE ::   JP4_O3_NO2       
   INTEGER(KIND=JPIM), SAVE ::   JP4_NO3_NO       
   INTEGER(KIND=JPIM), SAVE ::   JP4_NO3_NO2a      ! -->  NO + NO2 + O2
   INTEGER(KIND=JPIM), SAVE ::   JP4_NO3_NO2b      ! -->  N2O5
   INTEGER(KIND=JPIM), SAVE ::   JP4_OH_SO2       
   INTEGER(KIND=JPIM), SAVE ::   JP4_CO_OHa        ! -->  H + CO2
   INTEGER(KIND=JPIM), SAVE ::   JP4_ISO_O3P_i     
   INTEGER(KIND=JPIM), SAVE ::   JP4_ISO_O3P_ii    
   INTEGER(KIND=JPIM), SAVE ::   JP4_ISO_O3P_iii   
   INTEGER(KIND=JPIM), SAVE ::   JP4_ISO_O3P_iv    
   INTEGER(KIND=JPIM), SAVE ::   JP4_MACR_O3P        
   INTEGER(KIND=JPIM), SAVE ::   JP4_CH4_OH       
   INTEGER(KIND=JPIM), SAVE ::   JP4_ETH_OH       
   INTEGER(KIND=JPIM), SAVE ::   JP4_HC3_OH_i      
   INTEGER(KIND=JPIM), SAVE ::   JP4_HC3_OH_ii     
   INTEGER(KIND=JPIM), SAVE ::   JP4_HC3_OH_iii    
   INTEGER(KIND=JPIM), SAVE ::   JP4_HC3_OH_iv        
   INTEGER(KIND=JPIM), SAVE ::   JP4_HC5_OHa      
   INTEGER(KIND=JPIM), SAVE ::   JP4_HC5_OHb      
   INTEGER(KIND=JPIM), SAVE ::   JP4_HC8_OH_i      
   INTEGER(KIND=JPIM), SAVE ::   JP4_HC8_OH_ii        
   INTEGER(KIND=JPIM), SAVE ::   JP4_ETE_OH       
   INTEGER(KIND=JPIM), SAVE ::   JP4_OLT_OH       
   INTEGER(KIND=JPIM), SAVE ::   JP4_OLI_OH       
   INTEGER(KIND=JPIM), SAVE ::   JP4_DIEN_OH      
   INTEGER(KIND=JPIM), SAVE ::   JP4_ISO_OH       
   INTEGER(KIND=JPIM), SAVE ::   JP4_API_OH       
   INTEGER(KIND=JPIM), SAVE ::   JP4_LIM_OH       
   INTEGER(KIND=JPIM), SAVE ::   JP4_TOL_OHa      
   INTEGER(KIND=JPIM), SAVE ::   JP4_TOL_OHb      
   INTEGER(KIND=JPIM), SAVE ::   JP4_XYL_OHa      
   INTEGER(KIND=JPIM), SAVE ::   JP4_XYL_OHb      
   INTEGER(KIND=JPIM), SAVE ::   JP4_CSL_OHa      
   INTEGER(KIND=JPIM), SAVE ::   JP4_CSL_OHb      
   INTEGER(KIND=JPIM), SAVE ::   JP4_CSL_OHc      
   INTEGER(KIND=JPIM), SAVE ::   JP4_HCHO_OH_i     
   INTEGER(KIND=JPIM), SAVE ::   JP4_HCHO_OH_ii    
   INTEGER(KIND=JPIM), SAVE ::   JP4_ALD_OH       
   INTEGER(KIND=JPIM), SAVE ::   JP4_KET_OH       
   INTEGER(KIND=JPIM), SAVE ::   JP4_HKET_OH      
   INTEGER(KIND=JPIM), SAVE ::   JP4_GLY_OH       
   INTEGER(KIND=JPIM), SAVE ::   JP4_MGLY_OH      
   INTEGER(KIND=JPIM), SAVE ::   JP4_MACR_OH_i     
   INTEGER(KIND=JPIM), SAVE ::   JP4_MACR_OH_ii    
   INTEGER(KIND=JPIM), SAVE ::   JP4_MACR_OH_iii   
   INTEGER(KIND=JPIM), SAVE ::   JP4_MACR_OH_iv    
   INTEGER(KIND=JPIM), SAVE ::   JP4_OH_DCB_i      
   INTEGER(KIND=JPIM), SAVE ::   JP4_OH_DCB_ii     
   INTEGER(KIND=JPIM), SAVE ::   JP4_OH_DCB_iii    
   INTEGER(KIND=JPIM), SAVE ::   JP4_OH_UDDa      
   INTEGER(KIND=JPIM), SAVE ::   JP4_OH_UDDb      
   INTEGER(KIND=JPIM), SAVE ::   JP4_OP1_OHa       ! --> MO2 + H2O
   INTEGER(KIND=JPIM), SAVE ::   JP4_OP1_OHb_i      ! --> HCHO + OH + H2O
   INTEGER(KIND=JPIM), SAVE ::   JP4_OP1_OHb_ii    
   INTEGER(KIND=JPIM), SAVE ::   JP4_OP2_OH_i      
   INTEGER(KIND=JPIM), SAVE ::   JP4_OP2_OH_ii     
   INTEGER(KIND=JPIM), SAVE ::   JP4_OP2_OH_iii       
   INTEGER(KIND=JPIM), SAVE ::   JP4_PAA_OHa      
   INTEGER(KIND=JPIM), SAVE ::   JP4_PAA_OHb_i     
   INTEGER(KIND=JPIM), SAVE ::   JP4_PAA_OHb_ii    
   INTEGER(KIND=JPIM), SAVE ::   JP4_PAN_OH_i      
   INTEGER(KIND=JPIM), SAVE ::   JP4_PAN_OH_ii     
   INTEGER(KIND=JPIM), SAVE ::   JP4_TPAN_OH_i     
   INTEGER(KIND=JPIM), SAVE ::   JP4_TPAN_OH_ii    
   INTEGER(KIND=JPIM), SAVE ::   JP4_TPAN_OH_iii   
   INTEGER(KIND=JPIM), SAVE ::   JP4_ONIT_OH_i     
   INTEGER(KIND=JPIM), SAVE ::   JP4_ONIT_OH_ii    
   INTEGER(KIND=JPIM), SAVE ::   JP4_HCHO_NO3_i    
   INTEGER(KIND=JPIM), SAVE ::   JP4_HCHO_NO3_ii   
   INTEGER(KIND=JPIM), SAVE ::   JP4_ALD_NO3      
   INTEGER(KIND=JPIM), SAVE ::   JP4_GLY_NO3_i     
   INTEGER(KIND=JPIM), SAVE ::   JP4_GLY_NO3_ii      
   INTEGER(KIND=JPIM), SAVE ::   JP4_MGLY_NO3_i    
   INTEGER(KIND=JPIM), SAVE ::   JP4_MGLY_NO3_ii   
   INTEGER(KIND=JPIM), SAVE ::   JP4_MACR_NO3_i    
   INTEGER(KIND=JPIM), SAVE ::   JP4_MACR_NO3_ii   
   INTEGER(KIND=JPIM), SAVE ::   JP4_DCB_NO3_i     
   INTEGER(KIND=JPIM), SAVE ::   JP4_DCB_NO3_ii    
   INTEGER(KIND=JPIM), SAVE ::   JP4_DCB_NO3_iii   
   INTEGER(KIND=JPIM), SAVE ::   JP4_DCB_NO3_iv    
   INTEGER(KIND=JPIM), SAVE ::   JP4_DCB_NO3_v     
   INTEGER(KIND=JPIM), SAVE ::   JP4_CSL_NO3      
   INTEGER(KIND=JPIM), SAVE ::   JP4_ETE_NO3a     
   INTEGER(KIND=JPIM), SAVE ::   JP4_ETE_NO3b        
   INTEGER(KIND=JPIM), SAVE ::   JP4_OLT_NO3a     
   INTEGER(KIND=JPIM), SAVE ::   JP4_OLT_NO3b     
   INTEGER(KIND=JPIM), SAVE ::   JP4_OLI_NO3a     
   INTEGER(KIND=JPIM), SAVE ::   JP4_OLI_NO3b        
   INTEGER(KIND=JPIM), SAVE ::   JP4_DIEN_NO3a    
   INTEGER(KIND=JPIM), SAVE ::   JP4_DIEN_NO3b    
   INTEGER(KIND=JPIM), SAVE ::   JP4_ISO_NO3a     
   INTEGER(KIND=JPIM), SAVE ::   JP4_ISO_NO3b     
   INTEGER(KIND=JPIM), SAVE ::   JP4_API_NO3a     
   INTEGER(KIND=JPIM), SAVE ::   JP4_API_NO3b        
   INTEGER(KIND=JPIM), SAVE ::   JP4_LIM_NO3a     
   INTEGER(KIND=JPIM), SAVE ::   JP4_LIM_NO3b     
   INTEGER(KIND=JPIM), SAVE ::   JP4_TPAN_NO3a_i   
   INTEGER(KIND=JPIM), SAVE ::   JP4_TPAN_NO3a_ii  
   INTEGER(KIND=JPIM), SAVE ::   JP4_TPAN_NO3b_i   
   INTEGER(KIND=JPIM), SAVE ::   JP4_TPAN_NO3b_ii  
   INTEGER(KIND=JPIM), SAVE ::   JP4_ETE_O3_i      
   INTEGER(KIND=JPIM), SAVE ::   JP4_ETE_O3_ii     
   INTEGER(KIND=JPIM), SAVE ::   JP4_ETE_O3_iii    
   INTEGER(KIND=JPIM), SAVE ::   JP4_OLT_O3_i      
   INTEGER(KIND=JPIM), SAVE ::   JP4_OLT_O3_ii     
   INTEGER(KIND=JPIM), SAVE ::   JP4_OLT_O3_iii    
   INTEGER(KIND=JPIM), SAVE ::   JP4_OLT_O3_iv     
   INTEGER(KIND=JPIM), SAVE ::   JP4_OLT_O3_v      
   INTEGER(KIND=JPIM), SAVE ::   JP4_OLT_O3_vi     
   INTEGER(KIND=JPIM), SAVE ::   JP4_OLT_O3_vii    
   INTEGER(KIND=JPIM), SAVE ::   JP4_OLI_O3_i      
   INTEGER(KIND=JPIM), SAVE ::   JP4_OLI_O3_ii     
   INTEGER(KIND=JPIM), SAVE ::   JP4_OLI_O3_iii    
   INTEGER(KIND=JPIM), SAVE ::   JP4_OLI_O3_iv     
   INTEGER(KIND=JPIM), SAVE ::   JP4_OLI_O3_v      
   INTEGER(KIND=JPIM), SAVE ::   JP4_OLI_O3_vi     
   INTEGER(KIND=JPIM), SAVE ::   JP4_DIEN_O3_i     
   INTEGER(KIND=JPIM), SAVE ::   JP4_DIEN_O3_ii    
   INTEGER(KIND=JPIM), SAVE ::   JP4_DIEN_O3_iii   
   INTEGER(KIND=JPIM), SAVE ::   JP4_DIEN_O3_iv    
   INTEGER(KIND=JPIM), SAVE ::   JP4_DIEN_O3_v     
   INTEGER(KIND=JPIM), SAVE ::   JP4_DIEN_O3_vi       
   INTEGER(KIND=JPIM), SAVE ::   JP4_ISO_O3_i      
   INTEGER(KIND=JPIM), SAVE ::   JP4_ISO_O3_ii     
   INTEGER(KIND=JPIM), SAVE ::   JP4_ISO_O3_iii    
   INTEGER(KIND=JPIM), SAVE ::   JP4_ISO_O3_iv     
   INTEGER(KIND=JPIM), SAVE ::   JP4_ISO_O3_v      
   INTEGER(KIND=JPIM), SAVE ::   JP4_ISO_O3_vi        
   INTEGER(KIND=JPIM), SAVE ::   JP4_API_O3_i      
   INTEGER(KIND=JPIM), SAVE ::   JP4_API_O3_ii     
   INTEGER(KIND=JPIM), SAVE ::   JP4_API_O3_iii    
   INTEGER(KIND=JPIM), SAVE ::   JP4_API_O3_iv     
   INTEGER(KIND=JPIM), SAVE ::   JP4_LIM_O3_i      
   INTEGER(KIND=JPIM), SAVE ::   JP4_LIM_O3_ii     
   INTEGER(KIND=JPIM), SAVE ::   JP4_LIM_O3_iii    
   INTEGER(KIND=JPIM), SAVE ::   JP4_LIM_O3_iv     
   INTEGER(KIND=JPIM), SAVE ::   JP4_LIM_O3_v         
   INTEGER(KIND=JPIM), SAVE ::   JP4_MACR_O3_i     
   INTEGER(KIND=JPIM), SAVE ::   JP4_MACR_O3_ii    
   INTEGER(KIND=JPIM), SAVE ::   JP4_MACR_O3_iii   
   INTEGER(KIND=JPIM), SAVE ::   JP4_MACR_O3_iv    
   INTEGER(KIND=JPIM), SAVE ::   JP4_MACR_O3_v     
   INTEGER(KIND=JPIM), SAVE ::   JP4_DCB_O3_i       
   INTEGER(KIND=JPIM), SAVE ::   JP4_DCB_O3_ii    
   INTEGER(KIND=JPIM), SAVE ::   JP4_DCB_O3_iii   
   INTEGER(KIND=JPIM), SAVE ::   JP4_DCB_O3_iv       
   INTEGER(KIND=JPIM), SAVE ::   JP4_TPAN_O3_i     
   INTEGER(KIND=JPIM), SAVE ::   JP4_TPAN_O3_ii    
   INTEGER(KIND=JPIM), SAVE ::   JP4_TPAN_O3_iii   
   INTEGER(KIND=JPIM), SAVE ::   JP4_TPAN_O3_iv    
   INTEGER(KIND=JPIM), SAVE ::   JP4_PHO_NO2      
   INTEGER(KIND=JPIM), SAVE ::   JP4_PHO_HO2      
   INTEGER(KIND=JPIM), SAVE ::   JP4_ADDT_NO2     
   INTEGER(KIND=JPIM), SAVE ::   JP4_ADDT_O2a      ! --> TOLP
   INTEGER(KIND=JPIM), SAVE ::   JP4_ADDT_O2b      ! --> CSL + HO2
   INTEGER(KIND=JPIM), SAVE ::   JP4_ADDT_O3      
   INTEGER(KIND=JPIM), SAVE ::   JP4_ADDX_NO2        
   INTEGER(KIND=JPIM), SAVE ::   JP4_ADDX_O2a     
   INTEGER(KIND=JPIM), SAVE ::   JP4_ADDX_O2b        
   INTEGER(KIND=JPIM), SAVE ::   JP4_ADDX_O3         
   INTEGER(KIND=JPIM), SAVE ::   JP4_ADDC_NO2     
   INTEGER(KIND=JPIM), SAVE ::   JP4_ADDC_O2a     
   INTEGER(KIND=JPIM), SAVE ::   JP4_ADDC_O2b        
   INTEGER(KIND=JPIM), SAVE ::   JP4_ADDC_O3         
   INTEGER(KIND=JPIM), SAVE ::   JP4_ACO3_NO2     
   INTEGER(KIND=JPIM), SAVE ::   JP4_TCO3_NO2        
   INTEGER(KIND=JPIM), SAVE ::   JP4_MO2_NO       
   INTEGER(KIND=JPIM), SAVE ::   JP4_ETHP_NO_i     
   INTEGER(KIND=JPIM), SAVE ::   JP4_ETHP_NO_ii    
   INTEGER(KIND=JPIM), SAVE ::   JP4_HC3P_NO_i     
   INTEGER(KIND=JPIM), SAVE ::   JP4_HC3P_NO_ii    
   INTEGER(KIND=JPIM), SAVE ::   JP4_HC3P_NO_iii   
   INTEGER(KIND=JPIM), SAVE ::   JP4_HC3P_NO_iv    
   INTEGER(KIND=JPIM), SAVE ::   JP4_HC3P_NO_v     
   INTEGER(KIND=JPIM), SAVE ::   JP4_HC5P_NO_i     
   INTEGER(KIND=JPIM), SAVE ::   JP4_HC5P_NO_ii    
   INTEGER(KIND=JPIM), SAVE ::   JP4_HC5P_NO_iii   
   INTEGER(KIND=JPIM), SAVE ::   JP4_HC5P_NO_iv    
   INTEGER(KIND=JPIM), SAVE ::   JP4_HC5P_NO_v      
   INTEGER(KIND=JPIM), SAVE ::   JP4_HC8P_NO_i     
   INTEGER(KIND=JPIM), SAVE ::   JP4_HC8P_NO_ii    
   INTEGER(KIND=JPIM), SAVE ::   JP4_HC8P_NO_iii   
   INTEGER(KIND=JPIM), SAVE ::   JP4_HC8P_NO_iv    
   INTEGER(KIND=JPIM), SAVE ::   JP4_ETEP_NO_i     
   INTEGER(KIND=JPIM), SAVE ::   JP4_ETEP_NO_ii       
   INTEGER(KIND=JPIM), SAVE ::   JP4_OLTP_NO_i     
   INTEGER(KIND=JPIM), SAVE ::   JP4_OLTP_NO_ii    
   INTEGER(KIND=JPIM), SAVE ::   JP4_OLTP_NO_iii      
   INTEGER(KIND=JPIM), SAVE ::   JP4_OLIP_NO_i     
   INTEGER(KIND=JPIM), SAVE ::   JP4_OLIP_NO_ii    
   INTEGER(KIND=JPIM), SAVE ::   JP4_ISOP_NO_i     
   INTEGER(KIND=JPIM), SAVE ::   JP4_ISOP_NO_ii    
   INTEGER(KIND=JPIM), SAVE ::   JP4_ISOP_NO_iii   
   INTEGER(KIND=JPIM), SAVE ::   JP4_APIP_NO_i     
   INTEGER(KIND=JPIM), SAVE ::   JP4_APIP_NO_ii    
   INTEGER(KIND=JPIM), SAVE ::   JP4_APIP_NO_iii   
   INTEGER(KIND=JPIM), SAVE ::   JP4_LIMP_NO_i     
   INTEGER(KIND=JPIM), SAVE ::   JP4_LIMP_NO_ii    
   INTEGER(KIND=JPIM), SAVE ::   JP4_LIMP_NO_iii   
   INTEGER(KIND=JPIM), SAVE ::   JP4_TOLP_NO_i     
   INTEGER(KIND=JPIM), SAVE ::   JP4_TOLP_NO_ii    
   INTEGER(KIND=JPIM), SAVE ::   JP4_TOLP_NO_iii   
   INTEGER(KIND=JPIM), SAVE ::   JP4_XYLP_NO_i     
   INTEGER(KIND=JPIM), SAVE ::   JP4_XYLP_NO_ii    
   INTEGER(KIND=JPIM), SAVE ::   JP4_XYLP_NO_iii      
   INTEGER(KIND=JPIM), SAVE ::   JP4_CSLP_NO_i     
   INTEGER(KIND=JPIM), SAVE ::   JP4_CSLP_NO_ii       
   INTEGER(KIND=JPIM), SAVE ::   JP4_ACO3_NO      
   INTEGER(KIND=JPIM), SAVE ::   JP4_TCO3_NO_i     
   INTEGER(KIND=JPIM), SAVE ::   JP4_TCO3_NO_ii       
   INTEGER(KIND=JPIM), SAVE ::   JP4_KETP_NO_i     
   INTEGER(KIND=JPIM), SAVE ::   JP4_KETP_NO_ii    
   INTEGER(KIND=JPIM), SAVE ::   JP4_KETP_NO_iii      
   INTEGER(KIND=JPIM), SAVE ::   JP4_OLNN_NO_i     
   INTEGER(KIND=JPIM), SAVE ::   JP4_OLNN_NO_ii    
   INTEGER(KIND=JPIM), SAVE ::   JP4_OLND_NO_i     
   INTEGER(KIND=JPIM), SAVE ::   JP4_OLND_NO_ii    
   INTEGER(KIND=JPIM), SAVE ::   JP4_MO2_HO2      
   INTEGER(KIND=JPIM), SAVE ::   JP4_ETHP_HO2     
   INTEGER(KIND=JPIM), SAVE ::   JP4_HC3P_HO2     
   INTEGER(KIND=JPIM), SAVE ::   JP4_HC5P_HO2     
   INTEGER(KIND=JPIM), SAVE ::   JP4_HC8P_HO2     
   INTEGER(KIND=JPIM), SAVE ::   JP4_ETEP_HO2     
   INTEGER(KIND=JPIM), SAVE ::   JP4_OLTP_HO2     
   INTEGER(KIND=JPIM), SAVE ::   JP4_OLIP_HO2     
   INTEGER(KIND=JPIM), SAVE ::   JP4_ISOP_HO2     
   INTEGER(KIND=JPIM), SAVE ::   JP4_APIP_HO2     
   INTEGER(KIND=JPIM), SAVE ::   JP4_LIMP_HO2     
   INTEGER(KIND=JPIM), SAVE ::   JP4_TOLP_HO2     
   INTEGER(KIND=JPIM), SAVE ::   JP4_XYLP_HO2     
   INTEGER(KIND=JPIM), SAVE ::   JP4_CSLP_HO2        
   INTEGER(KIND=JPIM), SAVE ::   JP4_ACO3_HO2a    
   INTEGER(KIND=JPIM), SAVE ::   JP4_ACO3_HO2b    
   INTEGER(KIND=JPIM), SAVE ::   JP4_TCO3_HO2a    
   INTEGER(KIND=JPIM), SAVE ::   JP4_TCO3_HO2b    
   INTEGER(KIND=JPIM), SAVE ::   JP4_KETP_HO2     
   INTEGER(KIND=JPIM), SAVE ::   JP4_OLNN_HO2     
   INTEGER(KIND=JPIM), SAVE ::   JP4_OLND_HO2        
   INTEGER(KIND=JPIM), SAVE ::   JP4_ETHP_MO2_i    
   INTEGER(KIND=JPIM), SAVE ::   JP4_ETHP_MO2_ii   
   INTEGER(KIND=JPIM), SAVE ::   JP4_HC3P_MO2_i    
   INTEGER(KIND=JPIM), SAVE ::   JP4_HC3P_MO2_ii  
   INTEGER(KIND=JPIM), SAVE ::   JP4_HC3P_MO2_iii 
   INTEGER(KIND=JPIM), SAVE ::   JP4_HC3P_MO2_iv  
   INTEGER(KIND=JPIM), SAVE ::   JP4_HC5P_MO2_i    
   INTEGER(KIND=JPIM), SAVE ::   JP4_HC5P_MO2_ii  
   INTEGER(KIND=JPIM), SAVE ::   JP4_HC5P_MO2_iii 
   INTEGER(KIND=JPIM), SAVE ::   JP4_HC5P_MO2_iv  
   INTEGER(KIND=JPIM), SAVE ::   JP4_HC8P_MO2_i    
   INTEGER(KIND=JPIM), SAVE ::   JP4_HC8P_MO2_ii   
   INTEGER(KIND=JPIM), SAVE ::   JP4_HC8P_MO2_iii  
   INTEGER(KIND=JPIM), SAVE ::   JP4_ETEP_MO2_i    
   INTEGER(KIND=JPIM), SAVE ::   JP4_ETEP_MO2_ii   
   INTEGER(KIND=JPIM), SAVE ::   JP4_OLTP_MO2_i    
   INTEGER(KIND=JPIM), SAVE ::   JP4_OLTP_MO2_ii      
   INTEGER(KIND=JPIM), SAVE ::   JP4_OLIP_MO2_i    
   INTEGER(KIND=JPIM), SAVE ::   JP4_OLIP_MO2_ii   
   INTEGER(KIND=JPIM), SAVE ::   JP4_ISOP_MO2_i    
   INTEGER(KIND=JPIM), SAVE ::   JP4_ISOP_MO2_ii   
   INTEGER(KIND=JPIM), SAVE ::   JP4_ISOP_MO2_iii     
   INTEGER(KIND=JPIM), SAVE ::   JP4_APIP_MO2_i    
   INTEGER(KIND=JPIM), SAVE ::   JP4_APIP_MO2_ii   
   INTEGER(KIND=JPIM), SAVE ::   JP4_LIMP_MO2_i    
   INTEGER(KIND=JPIM), SAVE ::   JP4_LIMP_MO2_ii   
   INTEGER(KIND=JPIM), SAVE ::   JP4_TOLP_MO2_i    
   INTEGER(KIND=JPIM), SAVE ::   JP4_TOLP_MO2_ii   
   INTEGER(KIND=JPIM), SAVE ::   JP4_TOLP_MO2_iii  
   INTEGER(KIND=JPIM), SAVE ::   JP4_XYLP_MO2_i    
   INTEGER(KIND=JPIM), SAVE ::   JP4_XYLP_MO2_ii   
   INTEGER(KIND=JPIM), SAVE ::   JP4_XYLP_MO2_iii  
   INTEGER(KIND=JPIM), SAVE ::   JP4_CSLP_MO2_i    
   INTEGER(KIND=JPIM), SAVE ::   JP4_CSLP_MO2_ii      
   INTEGER(KIND=JPIM), SAVE ::   JP4_ACO3_MO2a_i   
   INTEGER(KIND=JPIM), SAVE ::   JP4_ACO3_MO2a_ii  
   INTEGER(KIND=JPIM), SAVE ::   JP4_ACO3_MO2b    
   INTEGER(KIND=JPIM), SAVE ::   JP4_TCO3_MO2a_i   
   INTEGER(KIND=JPIM), SAVE ::   JP4_TCO3_MO2a_ii  
   INTEGER(KIND=JPIM), SAVE ::   JP4_TCO3_MO2b       
   INTEGER(KIND=JPIM), SAVE ::   JP4_KETP_MO2_i    
   INTEGER(KIND=JPIM), SAVE ::   JP4_KETP_MO2_ii   
   INTEGER(KIND=JPIM), SAVE ::   JP4_KETP_MO2_iii   
   INTEGER(KIND=JPIM), SAVE ::   JP4_KETP_MO2_iv   
   INTEGER(KIND=JPIM), SAVE ::   JP4_OLNN_MO2_i    
   INTEGER(KIND=JPIM), SAVE ::   JP4_OLNN_MO2_ii   
   INTEGER(KIND=JPIM), SAVE ::   JP4_OLND_MO2_i    
   INTEGER(KIND=JPIM), SAVE ::   JP4_OLND_MO2_ii   
   INTEGER(KIND=JPIM), SAVE ::   JP4_OLND_MO2_iii  
   INTEGER(KIND=JPIM), SAVE ::   JP4_ETHP_ACO3_i   
   INTEGER(KIND=JPIM), SAVE ::   JP4_ETHP_ACO3_ii  
   INTEGER(KIND=JPIM), SAVE ::   JP4_HC3P_ACO3_i   
   INTEGER(KIND=JPIM), SAVE ::   JP4_HC3P_ACO3_ii  
   INTEGER(KIND=JPIM), SAVE ::   JP4_HC3P_ACO3_iii 
   INTEGER(KIND=JPIM), SAVE ::   JP4_HC3P_ACO3_iv  
   INTEGER(KIND=JPIM), SAVE ::   JP4_HC3P_ACO3_v   
   INTEGER(KIND=JPIM), SAVE ::   JP4_HC5P_ACO3_i   
   INTEGER(KIND=JPIM), SAVE ::   JP4_HC5P_ACO3_ii  
   INTEGER(KIND=JPIM), SAVE ::   JP4_HC5P_ACO3_iii 
   INTEGER(KIND=JPIM), SAVE ::   JP4_HC5P_ACO3_iv  
   INTEGER(KIND=JPIM), SAVE ::   JP4_HC8P_ACO3_i   
   INTEGER(KIND=JPIM), SAVE ::   JP4_HC8P_ACO3_ii  
   INTEGER(KIND=JPIM), SAVE ::   JP4_HC8P_ACO3_iii 
   INTEGER(KIND=JPIM), SAVE ::   JP4_HC8P_ACO3_iv  
   INTEGER(KIND=JPIM), SAVE ::   JP4_ETEP_ACO3_i   
   INTEGER(KIND=JPIM), SAVE ::   JP4_ETEP_ACO3_ii  
   INTEGER(KIND=JPIM), SAVE ::   JP4_OLTP_ACO3_i   
   INTEGER(KIND=JPIM), SAVE ::   JP4_OLTP_ACO3_ii  
   INTEGER(KIND=JPIM), SAVE ::   JP4_OLTP_ACO3_iii    
   INTEGER(KIND=JPIM), SAVE ::   JP4_OLIP_ACO3_i   
   INTEGER(KIND=JPIM), SAVE ::   JP4_OLIP_ACO3_ii  
   INTEGER(KIND=JPIM), SAVE ::   JP4_ISOP_ACO3_i   
   INTEGER(KIND=JPIM), SAVE ::   JP4_ISOP_ACO3_ii  
   INTEGER(KIND=JPIM), SAVE ::   JP4_ISOP_ACO3_iii 
   INTEGER(KIND=JPIM), SAVE ::   JP4_APIP_ACO3_i   
   INTEGER(KIND=JPIM), SAVE ::   JP4_APIP_ACO3_ii  
   INTEGER(KIND=JPIM), SAVE ::   JP4_LIMP_ACO3_i   
   INTEGER(KIND=JPIM), SAVE ::   JP4_LIMP_ACO3_ii  
   INTEGER(KIND=JPIM), SAVE ::   JP4_LIMP_ACO3_iii    
   INTEGER(KIND=JPIM), SAVE ::   JP4_TOLP_ACO3_i   
   INTEGER(KIND=JPIM), SAVE ::   JP4_TOLP_ACO3_ii  
   INTEGER(KIND=JPIM), SAVE ::   JP4_TOLP_ACO3_iii 
   INTEGER(KIND=JPIM), SAVE ::   JP4_XYLP_ACO3_i   
   INTEGER(KIND=JPIM), SAVE ::   JP4_XYLP_ACO3_ii  
   INTEGER(KIND=JPIM), SAVE ::   JP4_XYLP_ACO3_iii    
   INTEGER(KIND=JPIM), SAVE ::   JP4_CSLP_ACO3_i   
   INTEGER(KIND=JPIM), SAVE ::   JP4_CSLP_ACO3_ii  
   INTEGER(KIND=JPIM), SAVE ::   JP4_TCO3_ACO3_i   
   INTEGER(KIND=JPIM), SAVE ::   JP4_TCO3_ACO3_ii  
   INTEGER(KIND=JPIM), SAVE ::   JP4_KETP_ACO3_i   
   INTEGER(KIND=JPIM), SAVE ::   JP4_KETP_ACO3_ii  
   INTEGER(KIND=JPIM), SAVE ::   JP4_KETP_ACO3_iii 
   INTEGER(KIND=JPIM), SAVE ::   JP4_KETP_ACO3_iv  
   INTEGER(KIND=JPIM), SAVE ::   JP4_OLNN_ACO3_i   
   INTEGER(KIND=JPIM), SAVE ::   JP4_OLNN_ACO3_ii  
   INTEGER(KIND=JPIM), SAVE ::   JP4_OLND_ACO3_i   
   INTEGER(KIND=JPIM), SAVE ::   JP4_OLND_ACO3_ii  
   INTEGER(KIND=JPIM), SAVE ::   JP4_OLND_ACO3_iii 
   INTEGER(KIND=JPIM), SAVE ::   JP4_OLNN_OLND_i    ! 0.202*HCHO + 0.640*ALD + 0.149*KET +0.500*HO2 + 0.500*NO2 + 1.50*ONIT
   INTEGER(KIND=JPIM), SAVE ::   JP4_OLNN_OLND_ii  
   INTEGER(KIND=JPIM), SAVE ::   JP4_OLNN_OLND_iii 
   INTEGER(KIND=JPIM), SAVE ::   JP4_MO2_NO3_i     
   INTEGER(KIND=JPIM), SAVE ::   JP4_MO2_NO3_ii    
   INTEGER(KIND=JPIM), SAVE ::   JP4_ETHP_NO3_i    
   INTEGER(KIND=JPIM), SAVE ::   JP4_ETHP_NO3_ii   
   INTEGER(KIND=JPIM), SAVE ::   JP4_HC3P_NO3_i    
   INTEGER(KIND=JPIM), SAVE ::   JP4_HC3P_NO3_ii   
   INTEGER(KIND=JPIM), SAVE ::   JP4_HC3P_NO3_iii  
   INTEGER(KIND=JPIM), SAVE ::   JP4_HC3P_NO3_iv   
   INTEGER(KIND=JPIM), SAVE ::   JP4_HC3P_NO3_v       
   INTEGER(KIND=JPIM), SAVE ::   JP4_HC5P_NO3_i    
   INTEGER(KIND=JPIM), SAVE ::   JP4_HC5P_NO3_ii   
   INTEGER(KIND=JPIM), SAVE ::   JP4_HC5P_NO3_iii  
   INTEGER(KIND=JPIM), SAVE ::   JP4_HC5P_NO3_iv      
   INTEGER(KIND=JPIM), SAVE ::   JP4_HC8P_NO3_i    
   INTEGER(KIND=JPIM), SAVE ::   JP4_HC8P_NO3_ii   
   INTEGER(KIND=JPIM), SAVE ::   JP4_HC8P_NO3_iii  
   INTEGER(KIND=JPIM), SAVE ::   JP4_ETEP_NO3_i    
   INTEGER(KIND=JPIM), SAVE ::   JP4_ETEP_NO3_ii      
   INTEGER(KIND=JPIM), SAVE ::   JP4_OLTP_NO3_i    
   INTEGER(KIND=JPIM), SAVE ::   JP4_OLTP_NO3_ii   
   INTEGER(KIND=JPIM), SAVE ::   JP4_OLTP_NO3_iii     
   INTEGER(KIND=JPIM), SAVE ::   JP4_OLIP_NO3_i    
   INTEGER(KIND=JPIM), SAVE ::   JP4_OLIP_NO3_ii   
   INTEGER(KIND=JPIM), SAVE ::   JP4_ISOP_NO3_i    
   INTEGER(KIND=JPIM), SAVE ::   JP4_ISOP_NO3_ii   
   INTEGER(KIND=JPIM), SAVE ::   JP4_ISOP_NO3_iii     
   INTEGER(KIND=JPIM), SAVE ::   JP4_APIP_NO3_i    
   INTEGER(KIND=JPIM), SAVE ::   JP4_APIP_NO3_ii   
   INTEGER(KIND=JPIM), SAVE ::   JP4_LIMP_NO3_i    
   INTEGER(KIND=JPIM), SAVE ::   JP4_LIMP_NO3_ii   
   INTEGER(KIND=JPIM), SAVE ::   JP4_LIMP_NO3_iii  
   INTEGER(KIND=JPIM), SAVE ::   JP4_TOLP_NO3_i    
   INTEGER(KIND=JPIM), SAVE ::   JP4_TOLP_NO3_ii   
   INTEGER(KIND=JPIM), SAVE ::   JP4_TOLP_NO3_iii     
   INTEGER(KIND=JPIM), SAVE ::   JP4_XYLP_NO3_i    
   INTEGER(KIND=JPIM), SAVE ::   JP4_XYLP_NO3_ii   
   INTEGER(KIND=JPIM), SAVE ::   JP4_XYLP_NO3_iii     
   INTEGER(KIND=JPIM), SAVE ::   JP4_CSLP_NO3_i    
   INTEGER(KIND=JPIM), SAVE ::   JP4_CSLP_NO3_ii   
   INTEGER(KIND=JPIM), SAVE ::   JP4_ACO3_NO3     
   INTEGER(KIND=JPIM), SAVE ::   JP4_TCO3_NO3_i    
   INTEGER(KIND=JPIM), SAVE ::   JP4_TCO3_NO3_ii      
   INTEGER(KIND=JPIM), SAVE ::   JP4_KETP_NO3_i    
   INTEGER(KIND=JPIM), SAVE ::   JP4_KETP_NO3_ii   
   INTEGER(KIND=JPIM), SAVE ::   JP4_KETP_NO3_iii     
   INTEGER(KIND=JPIM), SAVE ::   JP4_OLNN_NO3_i    
   INTEGER(KIND=JPIM), SAVE ::   JP4_OLNN_NO3_ii      
   INTEGER(KIND=JPIM), SAVE ::   JP4_OLND_NO3_i    
   INTEGER(KIND=JPIM), SAVE ::   JP4_OLND_NO3_ii      
   INTEGER(KIND=JPIM), SAVE ::   JP4_XO2_HO2
   INTEGER(KIND=JPIM), SAVE ::   JP4_XO2_MO2      
   INTEGER(KIND=JPIM), SAVE ::   JP4_XO2_ACO3     
   INTEGER(KIND=JPIM), SAVE ::   JP4_XO2_NO       
   INTEGER(KIND=JPIM), SAVE ::   JP4_XO2_NO3      
   INTEGER(KIND=JPIM), SAVE ::   JP4_DMS_NO3      
   INTEGER(KIND=JPIM), SAVE ::   JP4_DMS_O3P         
   INTEGER(KIND=JPIM), SAVE ::   JP4_DMS_OH       
   INTEGER(KIND=JPIM), SAVE ::   JP4_H_O2         
   INTEGER(KIND=JPIM), SAVE ::   JP4_O3_H         
   INTEGER(KIND=JPIM), SAVE ::   JP4_O3P_OH       
   INTEGER(KIND=JPIM), SAVE ::   JP4_O3P_HO2      
   INTEGER(KIND=JPIM), SAVE ::   JP4_H_HO2a        ! -->  OH + OH
   INTEGER(KIND=JPIM), SAVE ::   JP4_H_HO2b        ! -->  H2 + O2
   INTEGER(KIND=JPIM), SAVE ::   JP4_H_HO2c        ! -->  H2O + O3P
!   INTEGER(KIND=JPIM), SAVE ::   JP4_H2_O3P       
   INTEGER(KIND=JPIM), SAVE ::   JP4_N_NO         
   INTEGER(KIND=JPIM), SAVE ::   JP4_N_O2         
   INTEGER(KIND=JPIM), SAVE ::   JP4_O1D_N2Oa      ! -->  N2 + O2
   INTEGER(KIND=JPIM), SAVE ::   JP4_O1D_N2Ob      ! -->  NO + NO
   INTEGER(KIND=JPIM), SAVE ::   JP4_O1D_CH4a      ! -->  OH + CH3
   INTEGER(KIND=JPIM), SAVE ::   JP4_O1D_CH4b      ! -->  CH3O + H
   INTEGER(KIND=JPIM), SAVE ::   JP4_O1D_CH4c      ! -->  HCHO + H2
   INTEGER(KIND=JPIM), SAVE ::   JP4_CH3_O3P      
   INTEGER(KIND=JPIM), SAVE ::   JP4_CH3_O2       
   INTEGER(KIND=JPIM), SAVE ::   JP4_HCHO_O3P_i     
   INTEGER(KIND=JPIM), SAVE ::   JP4_HCHO_O3P_ii    
   INTEGER(KIND=JPIM), SAVE ::   JP4_HCHO_Cl_i      
   INTEGER(KIND=JPIM), SAVE ::   JP4_HCHO_Cl_ii     
   INTEGER(KIND=JPIM), SAVE ::   JP4_CH3O_O2      
   INTEGER(KIND=JPIM), SAVE ::   JP4_MO2_ClO      
   INTEGER(KIND=JPIM), SAVE ::   JP4_HCHO_Br_i     
   INTEGER(KIND=JPIM), SAVE ::   JP4_HCHO_Br_ii    
   INTEGER(KIND=JPIM), SAVE ::   JP4_Cl_O3        
   INTEGER(KIND=JPIM), SAVE ::   JP4_ClO_O3P      
   INTEGER(KIND=JPIM), SAVE ::   JP4_ClO_NO       
   INTEGER(KIND=JPIM), SAVE ::   JP4_Cl_CH4       
!   INTEGER(KIND=JPIM), SAVE ::   JP4_Cl_H2        
   INTEGER(KIND=JPIM), SAVE ::   JP4_Cl_HO2a       ! -->  HCl + O2
   INTEGER(KIND=JPIM), SAVE ::   JP4_Cl_HO2b       ! -->  OH + ClO
   INTEGER(KIND=JPIM), SAVE ::   JP4_ClO_OHa       ! -->  Cl + HO2
   INTEGER(KIND=JPIM), SAVE ::   JP4_ClO_OHb       ! -->  HCl + O2
   INTEGER(KIND=JPIM), SAVE ::   JP4_OH_HCl       
   INTEGER(KIND=JPIM), SAVE ::   JP4_ClO_NO2      
   INTEGER(KIND=JPIM), SAVE ::   JP4_O3P_ClONO2   
   INTEGER(KIND=JPIM), SAVE ::   JP4_ClO_HO2      
   INTEGER(KIND=JPIM), SAVE ::   JP4_OH_HOCl      
   INTEGER(KIND=JPIM), SAVE ::   JP4_O3P_HOCl     
   INTEGER(KIND=JPIM), SAVE ::   JP4_Cl_NO2       
   INTEGER(KIND=JPIM), SAVE ::   JP4_Cl_HOCl      
   INTEGER(KIND=JPIM), SAVE ::   JP4_O1D_CFC11    
   INTEGER(KIND=JPIM), SAVE ::   JP4_O1D_CFC12    
   INTEGER(KIND=JPIM), SAVE ::   JP4_O1D_CFC113   
   INTEGER(KIND=JPIM), SAVE ::   JP4_O1D_CCl4     
   INTEGER(KIND=JPIM), SAVE ::   JP4_OH_CH3CCl3   
   INTEGER(KIND=JPIM), SAVE ::   JP4_OH_CH3Cl     
   INTEGER(KIND=JPIM), SAVE ::   JP4_O1D_HCFC22   
   INTEGER(KIND=JPIM), SAVE ::   JP4_OH_HCFC22    
   INTEGER(KIND=JPIM), SAVE ::   JP4_Br_O3        
   INTEGER(KIND=JPIM), SAVE ::   JP4_BrO_O3P      
   INTEGER(KIND=JPIM), SAVE ::   JP4_BrO_NO       
   INTEGER(KIND=JPIM), SAVE ::   JP4_BrO_ClOa      ! -->  OClO + Br
   INTEGER(KIND=JPIM), SAVE ::   JP4_BrO_ClOb      ! -->  Br + Cl + O2
   INTEGER(KIND=JPIM), SAVE ::   JP4_BrO_ClOc      ! -->  BrCl + O2
   INTEGER(KIND=JPIM), SAVE ::   JP4_Br_HO2       
   INTEGER(KIND=JPIM), SAVE ::   JP4_O1D_HBr      
   INTEGER(KIND=JPIM), SAVE ::   JP4_BrO_OH       
   INTEGER(KIND=JPIM), SAVE ::   JP4_OH_HBr       
   INTEGER(KIND=JPIM), SAVE ::   JP4_BrO_NO2      
   INTEGER(KIND=JPIM), SAVE ::   JP4_BrO_HO2      
   INTEGER(KIND=JPIM), SAVE ::   JP4_HOBr_O3P     
   INTEGER(KIND=JPIM), SAVE ::   JP4_O1D_CH3Br       
   INTEGER(KIND=JPIM), SAVE ::   JP4_OH_CH3Br        
   INTEGER(KIND=JPIM), SAVE ::   JP4_O1D_H1211    
   INTEGER(KIND=JPIM), SAVE ::   JP4_O1D_H1301    
#ifdef HETERO
   INTEGER(KIND=JPIM), SAVE ::   NB_HETERO
   INTEGER(KIND=JPIM), SAVE ::   JP4_ClONO2_H2Os   
   INTEGER(KIND=JPIM), SAVE ::   JP4_ClONO2_HCls   
   INTEGER(KIND=JPIM), SAVE ::   JP4_N2O5_H2Os     
   INTEGER(KIND=JPIM), SAVE ::   JP4_N2O5_HCls     
   INTEGER(KIND=JPIM), SAVE ::   JP4_HOCl_HCls     
   INTEGER(KIND=JPIM), SAVE ::   JP4_BrONO2_H2Os   
   INTEGER(KIND=JPIM), SAVE ::   JP4_HOBr_HCls     
   INTEGER(KIND=JPIM), SAVE ::   JP4_HOBr_HBrs_i     
   INTEGER(KIND=JPIM), SAVE ::   JP4_HOBr_HBrs_ii     
   INTEGER(KIND=JPIM), SAVE ::   JP4_HOCl_HBrs     
#endif
   INTEGER(KIND=JPIM), SAVE ::   JP4_HO2_NOb      


CONTAINS


SUBROUTINE indices_reactions
!
! This routine defines the reaction indices for the RACMOBUS scheme
! and set up the Zindice arrays including species numbers and coefficients 
! The following reactions are considered:
!    - Photolysis reactions with index JPO_J*, 
!    -                                described by Zindice_phot
!    - Reactions of the form a*A --> b*B+c*C with index JP3_J*,
!    -                                described by Zindice_3
!    - Reactions of the form a*A+d*D --> b*B+c*C with index JP4_J*, 
!    -                                described by Zindice_4
!
! INPUT:   JP*          - specie indices
! OUTPUT:  JPO*         - reaction indices
!          Zindice_phot - description of photolysis reactions 
!          Zindice_3    - description of a*A --> b*B+c*C reactions 
!          Zindice_4    - description of a*A+d*D --> b*B+c*C  reactions 


IMPLICIT NONE

TYPE(Z3SPEC) :: zloc,zloc1
TYPE(Z4SPEC) :: zloc2

INTEGER(KIND=JPIM) :: JJ,JK,i


zloc=Z3SPEC(999.,999,999.,999,999.,999)
zloc1=zloc
zloc2=Z4SPEC(999.,999,999.,999,999.,999,999.,999)

! RACMOBUS SCHEME

JJ=0
JJ=JJ+1 ; JPO_JNO2 =JJ  !CHEM_JCNAME(JPO_JNO2) ="NO2 --> O3P + NO"
JJ=JJ+1 ; JPO_JO3a =JJ  !CHEM_JCNAME(JPO_JO3a) ="O3 --> O1D + O2"
JJ=JJ+1 ; JPO_JO3b =JJ  !CHEM_JCNAME(JPO_JO3b) ="O3 --> O3P + O2"
JJ=JJ+1 ; JPO_JHONO =JJ  !CHEM_JCNAME(JPO_JHONO) ="HONO --> OH + NO"
JJ=JJ+1 ; JPO_JHNO3 =JJ  !CHEM_JCNAME(JPO_JHNO3) ="HNO3 --> OH + NO2"
JJ=JJ+1 ; JPO_JHNO4a =JJ  !CHEM_JCNAME(JPO_JHNO4) ="HNO4 --> x*HO2 + x*NO2 + (1-x)*OH + (1-x)*NO3"
JJ=JJ+1 ; JPO_JHNO4b =JJ 
JJ=JJ+1 ; JPO_JNO3a =JJ  !CHEM_JCNAME(JPO_JNO3a) ="NO3 --> NO + O2"
JJ=JJ+1 ; JPO_JNO3b =JJ  !CHEM_JCNAME(JPO_JNO3b) ="NO3 --> NO2 + O3P"
JJ=JJ+1 ; JPO_JH2O2 =JJ  !CHEM_JCNAME(JPO_JH2O2) ="H2O2 --> OH + OH"
JJ=JJ+1 ; JPO_JHCHOa =JJ  !CHEM_JCNAME(JPO_JHCHOa) ="HCHO --> H2 + CO"
JJ=JJ+1 ; JPO_JHCHOb_i =JJ  !CHEM_JCNAME(JPO_JHCHOb) ="HCHO --> H + CO + HO2"
JJ=JJ+1 ; JPO_JHCHOb_ii=JJ
JJ=JJ+1 ; JPO_JALD_i =JJ  !CHEM_JCNAME(JPO_JALD) ="ALD --> CH3 + HO2 + CO"
JJ=JJ+1 ; JPO_JALD_ii =JJ 
JJ=JJ+1 ; JPO_JOP1 =JJ  !CHEM_JCNAME(JPO_JOP1) ="OP1 --> CH3O + OH"
JJ=JJ+1 ; JPO_JOP2_i =JJ  !CHEM_JCNAME(JPO_JOP2) ="OP2 --> ALD + HO2 + OH"
JJ=JJ+1 ; JPO_JOP2_ii =JJ 
JJ=JJ+1 ; JPO_JPAA_i =JJ  !CHEM_JCNAME(JPO_JPAA) ="PAA --> CH3 + OH + CO2"
JJ=JJ+1 ; JPO_JPAA_ii =JJ 
JJ=JJ+1 ; JPO_JKET =JJ  !CHEM_JCNAME(JPO_JKET) ="KET --> ACO3 + ETHP"
JJ=JJ+1 ; JPO_JGLYa =JJ  !CHEM_JCNAME(JPO_JGLYa) ="GLY --> 0.13*HCHO + 1.87*CO + 0.87*H2"
JJ=JJ+1 ; JPO_JGLYb_i =JJ  !CHEM_JCNAME(JPO_JGLYb) ="GLY --> 0.45*HCHO + 1.55*CO + 0.80*HO2 + 0.15*H2"
JJ=JJ+1 ; JPO_JGLYb_ii =JJ 
JJ=JJ+1 ; JPO_JMGLY_i =JJ  !CHEM_JCNAME(JPO_JMGLY) ="MGLY --> ACO3 + HO2 + CO"
JJ=JJ+1 ; JPO_JMGLY_ii =JJ 
JJ=JJ+1 ; JPO_JDCB =JJ  !CHEM_JCNAME(JPO_JDCB) ="DCB --> TCO3 + HO2"
JJ=JJ+1 ; JPO_JONITa_i =JJ  !CHEM_JCNAME(JPO_JONITa) ="ONIT --> HO2 + NO2 + 0.20*ALD + 0.80*KET"
JJ=JJ+1 ; JPO_JONITa_ii =JJ 
JJ=JJ+1 ; JPO_JONITb_i =JJ  !CHEM_JCNAME(JPO_JONITb) ="ONIT --> NO2 + 0.341*HO2 + 0.564*RO2R + 0.095*RO2N + 0.152*R2O2 + &
!&0.134*HCHO + 0.431*CH3CHO + 0.147*RCHO + 0.02*ACET + 0.243*MEK + 0.435*PROD2 + 0.35*XC1"
JJ=JJ+1 ; JPO_JONITb_ii =JJ 
JJ=JJ+1 ; JPO_JMACR_i =JJ  !CHEM_JCNAME(JPO_JMACR) ="MACR --> HCHO + ACO3 + CO + HO2"
JJ=JJ+1 ; JPO_JMACR_ii =JJ 
JJ=JJ+1 ; JPO_JHKET_i =JJ  !CHEM_JCNAME(JPO_JHKET) ="HKET --> ACO3 + HCHO + HO2"
JJ=JJ+1 ; JPO_JHKET_ii =JJ 
JJ=JJ+1 ; JPO_JO2 =JJ  !CHEM_JCNAME(JPO_JO2) ="O2 --> O3P + O3P"
JJ=JJ+1 ; JPO_JN2O5 =JJ  !CHEM_JCNAME(JPO_JN2O5) ="N2O5 --> NOx + NOx + XX"
JJ=JJ+1 ; JPO_JH2O =JJ  !CHEM_JCNAME(JPO_JH2O) ="H2O --> H + OH"
JJ=JJ+1 ; JPO_JN2O =JJ  !CHEM_JCNAME(JPO_JN2O) ="N2O --> N2 + O1D"
JJ=JJ+1 ; JPO_JHOCl =JJ  !CHEM_JCNAME(JPO_JHOCl) ="HOCl --> Cl + OH"
JJ=JJ+1 ; JPO_JClONO2a=JJ  !CHEM_JCNAME(JPO_JClONO2a)="ClONO2 --> Cl + NO3"
JJ=JJ+1 ; JPO_JClONO2b=JJ  !CHEM_JCNAME(JPO_JClONO2b)="ClONO2 --> ClO + NO2"
JJ=JJ+1 ; JPO_JCl2O2_i =JJ  !CHEM_JCNAME(JPO_JCl2O2) ="Cl2O2 --> Cl + Cl + O2"
JJ=JJ+1 ; JPO_JCl2O2_ii=JJ
JJ=JJ+1 ; JPO_JHCl =JJ  !CHEM_JCNAME(JPO_JHCl) ="HCl --> H + Cl"
JJ=JJ+1 ; JPO_JCl2 =JJ  !CHEM_JCNAME(JPO_JCl2) ="Cl2 --> Cl + Cl"
JJ=JJ+1 ; JPO_JClNO2 =JJ  !CHEM_JCNAME(JPO_JClNO2) ="ClNO2 --> Cl + NO2"
JJ=JJ+1 ; JPO_JBrO =JJ  !CHEM_JCNAME(JPO_JBrO) ="BrO --> Br + O3P"
JJ=JJ+1 ; JPO_JBrONO2 =JJ  !CHEM_JCNAME(JPO_JBrONO2) ="BrONO2 --> Br + NO3"
JJ=JJ+1 ; JPO_JHOBr =JJ  !CHEM_JCNAME(JPO_JHOBr) ="HOBr --> Br + OH"
JJ=JJ+1 ; JPO_JOClO =JJ  !CHEM_JCNAME(JPO_JOClO) ="OClO --> O3P + ClO"
JJ=JJ+1 ; JPO_JBrCl =JJ  !CHEM_JCNAME(JPO_JBrCl) ="BrCl --> Br + Cl"
JJ=JJ+1 ; JPO_JCFC11_i =JJ  !CHEM_JCNAME(JPO_JCFC11) ="CFC11 --> ClOx + ClOx + ClOx + XX"
JJ=JJ+1 ; JPO_JCFC11_ii=JJ
JJ=JJ+1 ; JPO_JCFC12 =JJ  !CHEM_JCNAME(JPO_JCFC12) ="CFC12 --> ClOx + ClOx + XX"
JJ=JJ+1 ; JPO_JCFC113_i =JJ  !CHEM_JCNAME(JPO_JCFC113) ="CFC113 --> ClOx + ClOx + ClOx + XX"
JJ=JJ+1 ; JPO_JCFC113_ii=JJ
JJ=JJ+1 ; JPO_JCCl4_i =JJ  !CHEM_JCNAME(JPO_JCCl4) ="CCl4 --> ClOx + ClOx + ClOx + ClOx + XX"
JJ=JJ+1 ; JPO_JCCl4_ii=JJ
JJ=JJ+1 ; JPO_JCH3CCl3_i=JJ  !CHEM_JCNAME(JPO_JCH3CCl3)="CH3CCl3 --> ClOx + ClOx + ClOx + XX"
JJ=JJ+1 ; JPO_JCH3CCl3_ii=JJ
JJ=JJ+1 ; JPO_JCH3Cl =JJ  !CHEM_JCNAME(JPO_JCH3Cl) ="CH3Cl --> ClOx + XX"
JJ=JJ+1 ; JPO_JHCFC22 =JJ  !CHEM_JCNAME(JPO_JHCFC22) ="HCFC22 --> ClOx + XX"
JJ=JJ+1 ; JPO_JCH3Br =JJ  !CHEM_JCNAME(JPO_JCH3Br) ="CH3Br --> BrOx + XX"
JJ=JJ+1 ; JPO_JH1211 =JJ  !CHEM_JCNAME(JPO_JH1211) ="H1211 --> BrOx + XX"
JJ=JJ+1 ; JPO_JH1301 =JJ  !CHEM_JCNAME(JPO_JH1301) ="H1301 --> BrOx + XX"
JJ=JJ+1 ; JPO_JNO =JJ  !CHEM_JCNAME(JPO_JNO) ="NO --> N + O3P"
JJ=JJ+1 ; JPO_JCO2 =JJ  !CHEM_JCNAME(JPO_JCO2) ="CO_2 + hv --> CO + O(3P)"
!
JJ=JJ+1 ; JPO_O1D_N2a =JJ ! --> O3P + N2
JJ=JJ+1 ; JPO_OH_H2 =JJ
JJ=JJ+1 ; JPO_H2_O3P =JJ
JJ=JJ+1 ; JPO_Cl_H2 =JJ
JJ=JJ+1 ; JPO_HNO4 =JJ
JJ=JJ+1 ; JPO_N2O5 =JJ
JJ=JJ+1 ; JPO_PAN =JJ
JJ=JJ+1 ; JPO_TPAN =JJ
JJ=JJ+1 ; JPO_O1D_H2 =JJ
JJ=JJ+1 ; JPO_Cl2O2 =JJ
!
NB_PHOT=JJ ; ALLOCATE(Zindice_phot(NB_PHOT)) ; Zindice_phot=zloc
!
zloc=Z3SPEC(1.0,JPNO2,1.0,JPO3P,1.0,JPNO)
Zindice_phot(JPO_JNO2)=zloc
!
zloc=Z3SPEC(1.0,JPO3,1.0,JPO1D,1.0,JPO2)
Zindice_phot(JPO_JO3a)=zloc
!
zloc=Z3SPEC(1.0,JPO3,1.0,JPO3P,1.0,JPO2)
Zindice_phot(JPO_JO3b)=zloc
!
zloc=Z3SPEC(1.0,JPHNO3,1.0,JPOH,1.0,JPNO2)
Zindice_phot(JPO_JHNO3)=zloc
!
zloc=Z3SPEC(1.0,JPHNO4,1.0,JPHO2,1.0,JPNO2)
Zindice_phot(JPO_JHNO4a)=zloc
!
zloc=Z3SPEC(1.0,JPHNO4,1.0,JPOH,1.0,JPNO3)
Zindice_phot(JPO_JHNO4b)=zloc
!
zloc=Z3SPEC(1.0,JPNO3,1.0,JPNO,1.0,JPO2)
Zindice_phot(JPO_JNO3a)=zloc
!
zloc=Z3SPEC(1.0,JPNO3,1.0,JPNO2,1.0,JPO3P)
Zindice_phot(JPO_JNO3b)=zloc
!
zloc=Z3SPEC(1.0,JPH2O2,1.0,JPOH,1.0,JPOH)
Zindice_phot(JPO_JH2O2)=zloc
!
zloc=Z3SPEC(1.0,JPHCHO,1.0,JPCO,0.0,JPDUMMY)
Zindice_phot(JPO_JHCHOa)=zloc
!
zloc=Z3SPEC(1.0,JPHCHO,1.0,JPH,1.0,JPHO2)
Zindice_phot(JPO_JHCHOb_i)=zloc
zloc=Z3SPEC(0.0,JPHCHO,1.0,JPCO,0.0,JPDUMMY)
Zindice_phot(JPO_JHCHOb_ii)=zloc
!
zloc=Z3SPEC(1.0,JPOP1,1.0,JPCH3O,1.0,JPOH)
Zindice_phot(JPO_JOP1)=zloc
!
zloc=Z3SPEC(1.0,JPO2,1.0,JPO3P,1.0,JPO3P)
Zindice_phot(JPO_JO2)=zloc
!
zloc=Z3SPEC(1.0,JPN2O5,1.0,JPNO2,1.0,JPNO3)
Zindice_phot(JPO_JN2O5)=zloc
!
zloc=Z3SPEC(1.0,JPH2O,1.0,JPH,1.0,JPOH)
Zindice_phot(JPO_JH2O)=zloc
!
zloc=Z3SPEC(1.0,JPN2O,1.0,JPO1D,0.0,JPDUMMY)
Zindice_phot(JPO_JN2O)=zloc
!
zloc=Z3SPEC(1.0,JPHOCl,1.0,JPCl,1.0,JPOH)
Zindice_phot(JPO_JHOCl)=zloc
!
zloc=Z3SPEC(1.0,JPClONO2,1.0,JPCl,1.0,JPNO3)
Zindice_phot(JPO_JClONO2a)=zloc
!
zloc=Z3SPEC(1.0,JPClONO2,1.0,JPClO,1.0,JPNO2)
Zindice_phot(JPO_JClONO2b)=zloc
!
zloc=Z3SPEC(1.0,JPCl2O2,1.0,JPCl,1.0,JPCl)
Zindice_phot(JPO_JCl2O2_i)=zloc
zloc=Z3SPEC(0.0,JPCl2O2,1.0,JPO2,0.0,JPDUMMY)
Zindice_phot(JPO_JCl2O2_ii)=zloc
!
zloc=Z3SPEC(1.0,JPHCl,1.0,JPH,1.0,JPCl)
Zindice_phot(JPO_JHCl)=zloc
!
zloc=Z3SPEC(1.0,JPCl2,1.0,JPCl,1.0,JPCl)
Zindice_phot(JPO_JCl2)=zloc
!
zloc=Z3SPEC(1.0,JPClNO2,1.0,JPCl,1.0,JPNO2)
Zindice_phot(JPO_JClNO2)=zloc
!
zloc=Z3SPEC(1.0,JPBrO,1.0,JPBr,1.0,JPO3P)
Zindice_phot(JPO_JBrO)=zloc
!
zloc=Z3SPEC(1.0,JPBrONO2,1.0,JPBr,1.0,JPNO3)
Zindice_phot(JPO_JBrONO2)=zloc
!
zloc=Z3SPEC(1.0,JPHOBr,1.0,JPBr,1.0,JPOH)
Zindice_phot(JPO_JHOBr)=zloc
!
zloc=Z3SPEC(1.0,JPOClO,1.0,JPO3P,1.0,JPClO)
Zindice_phot(JPO_JOClO)=zloc
!
zloc=Z3SPEC(1.0,JPBrCl,1.0,JPBr,1.0,JPCl)
Zindice_phot(JPO_JBrCl)=zloc
!
zloc=Z3SPEC(1.0,JPCFC11,1.0,JPCl,1.0,JPCl)
Zindice_phot(JPO_JCFC11_i)=zloc
zloc=Z3SPEC(0.0,JPCFC11,1.0,JPCl,0.0,JPDUMMY)
Zindice_phot(JPO_JCFC11_ii)=zloc
!
zloc=Z3SPEC(1.0,JPCFC12,1.0,JPCl,1.0,JPCl)
Zindice_phot(JPO_JCFC12)=zloc
!
zloc=Z3SPEC(1.0,JPCFC113,1.0,JPCl,1.0,JPCl)
Zindice_phot(JPO_JCFC113_i)=zloc
zloc=Z3SPEC(0.0,JPCFC113,1.0,JPCl,0.0,JPDUMMY)
Zindice_phot(JPO_JCFC113_ii)=zloc
!
zloc=Z3SPEC(1.0,JPCCl4,1.0,JPCl,1.0,JPCl)
Zindice_phot(JPO_JCCl4_i)=zloc
zloc=Z3SPEC(0.0,JPCCl4,1.0,JPCl,1.0,JPCl)
Zindice_phot(JPO_JCCl4_ii)=zloc
!
zloc=Z3SPEC(1.0,JPCH3CCl3,1.0,JPCl,1.0,JPCl)
Zindice_phot(JPO_JCH3CCl3_i)=zloc
zloc=Z3SPEC(0.0,JPCH3CCl3,1.0,JPCl,0.0,JPDUMMY)
Zindice_phot(JPO_JCH3CCl3_ii)=zloc
!
zloc=Z3SPEC(1.0,JPCH3Cl,1.0,JPCl,0.0,JPDUMMY)
Zindice_phot(JPO_JCH3Cl)=zloc
!
zloc=Z3SPEC(1.0,JPHCFC22,1.0,JPCl,0.0,JPDUMMY)
Zindice_phot(JPO_JHCFC22)=zloc
!
zloc=Z3SPEC(1.0,JPCH3Br,1.0,JPBr,0.0,JPDUMMY)
Zindice_phot(JPO_JCH3Br)=zloc
!
zloc=Z3SPEC(1.0,JPH1211,1.0,JPBr,0.0,JPDUMMY)
Zindice_phot(JPO_JH1211)=zloc
!
zloc=Z3SPEC(1.0,JPH1301,1.0,JPBr,0.0,JPDUMMY)
Zindice_phot(JPO_JH1301)=zloc
!
zloc=Z3SPEC(1.0,JPNO,1.0,JPN,1.0,JPO3P)
Zindice_phot(JPO_JNO)=zloc
!
zloc=Z3SPEC(1.0,JPCO2,1.0,JPCO,1.0,JPO3P)
Zindice_phot(JPO_JCO2)=zloc
!
zloc=Z3SPEC(1.0,JPO1D,1.0,JPO3P,0.0,JPDUMMY)
Zindice_phot(JPO_O1D_N2a)=zloc
!
zloc=Z3SPEC(1.0,JPHNO4,1.0,JPHO2,1.0,JPNO2)
Zindice_phot(JPO_HNO4)=zloc
!
zloc=Z3SPEC(1.0,JPN2O5,1.0,JPNO2,1.0,JPNO3)
Zindice_phot(JPO_N2O5)=zloc
!
zloc=Z3SPEC(1.0,JPCl2O2,1.0,JPClO,1.0,JPClO)
Zindice_phot(JPO_Cl2O2)=zloc
!
zloc=Z3SPEC(1.0,JPOH,1.0,JPH2O,1.0,JPH)
Zindice_phot(JPO_OH_H2)=zloc
!
zloc=Z3SPEC(1.0,JPO1D,1.0,JPOH,1.0,JPH)
Zindice_phot(JPO_O1D_H2)=zloc
!
zloc=Z3SPEC(1.0,JPO3P,1.0,JPOH,1.0,JPH)
Zindice_phot(JPO_H2_O3P)=zloc
!
zloc=Z3SPEC(1.0,JPCl,1.0,JPHCl,1.0,JPH)
Zindice_phot(JPO_Cl_H2)=zloc
!
zloc=Z3SPEC(1.0,JPHONO,1.0,JPOH,1.0,JPNO)
Zindice_phot(JPO_JHONO)=zloc
!
zloc=Z3SPEC(1.0,JPALD,1.0,JPMO2,1.0,JPHO2)
Zindice_phot(JPO_JALD_i)=zloc
zloc=Z3SPEC(0.0,JPALD,1.0,JPCO,0.0,JPDUMMY)
Zindice_phot(JPO_JALD_ii)=zloc
!
zloc=Z3SPEC(1.0,JPOP2,1.0,JPALD,1.0,JPHO2)
Zindice_phot(JPO_JOP2_i)=zloc
zloc=Z3SPEC(0.0,JPOP2,1.0,JPOH,0.0,JPDUMMY)
Zindice_phot(JPO_JOP2_ii)=zloc
!
zloc=Z3SPEC(1.0,JPPAA,1.0,JPCH3,1.0,JPOH)
Zindice_phot(JPO_JPAA_i)=zloc
zloc=Z3SPEC(0.0,JPPAA,1.0,JPCO2,0.0,JPDUMMY)
Zindice_phot(JPO_JPAA_ii)=zloc
!GLY --> 0.13*HCHO + 1.87*CO + 0.87*H2"
zloc=Z3SPEC(1.0,JPGLY,0.13,JPHCHO,1.87,JPCO)
Zindice_phot(JPO_JGLYa)=zloc
!GLY --> 0.45*HCHO + 1.55*CO + 0.80*HO2 + 0.15*H2
zloc=Z3SPEC(1.0,JPGLY,0.45,JPHCHO,1.55,JPCO)
Zindice_phot(JPO_JGLYb_i)=zloc
zloc=Z3SPEC(0.0,JPGLY,0.80,JPHO2,0.0,JPDUMMY)
Zindice_phot(JPO_JGLYb_ii)=zloc
!MGLY --> ACO3 + HO2 + CO
zloc=Z3SPEC(1.0,JPMGLY,1.0,JPACO3,1.0,JPHO2)
Zindice_phot(JPO_JMGLY_i)=zloc
zloc=Z3SPEC(0.0,JPMGLY,1.0,JPCO,0.0,JPDUMMY)
Zindice_phot(JPO_JMGLY_ii)=zloc
!DCB --> TCO3 + HO2"
zloc=Z3SPEC(1.0,JPDCB,1.0,JPTCO3,1.0,JPHO2)
Zindice_phot(JPO_JDCB)=zloc
!
zloc=Z3SPEC(1.0,JPKET,1.0,JPACO3,1.0,JPETHP)
Zindice_phot(JPO_JKET)=zloc
!
zloc=Z3SPEC(1.0,JPMACR,1.0,JPHCHO,1.0,JPACO3)
Zindice_phot(JPO_JMACR_i)=zloc
zloc=Z3SPEC(0.0,JPMACR,1.0,JPHO2,1.0,JPCO)
Zindice_phot(JPO_JMACR_ii)=zloc
!HKET --> ACO3 + HCHO + HO2
zloc=Z3SPEC(1.0,JPHKET,1.0,JPACO3,1.0,JPHCHO)
Zindice_phot(JPO_JHKET_i)=zloc
zloc=Z3SPEC(0.0,JPHKET,1.0,JPHO2,0.0,JPDUMMY)
Zindice_phot(JPO_JHKET_ii)=zloc
!
zloc=Z3SPEC(1.0,JPONIT,1.0,JPALD,1.0,JPHO2)
Zindice_phot(JPO_JONITa_i)=zloc
zloc=Z3SPEC(0.0,JPONIT,1.0,JPNO2,0.0,JPDUMMY)
Zindice_phot(JPO_JONITa_ii)=zloc
!
zloc=Z3SPEC(1.0,JPONIT,1.0,JPKET,1.0,JPHO2)
Zindice_phot(JPO_JONITb_i)=zloc
zloc=Z3SPEC(0.0,JPONIT,1.0,JPNO2,0.0,JPDUMMY)
Zindice_phot(JPO_JONITb_ii)=zloc
!
zloc=Z3SPEC(1.0,JPPAN,1.0,JPACO3,1.0,JPNO2)
Zindice_phot(JPO_PAN)=zloc
!
zloc=Z3SPEC(1.0,JPTPAN,1.0,JPTCO3,1.0,JPNO2)
Zindice_phot(JPO_TPAN)=zloc
!
!
JK=0
JK=JK+1 ; JP3_HO2_HO2a =JK ! --> H2O2 + O2
JK=JK+1 ; JP3_HO2_HO2b =JK ! + H2O --> H2O2 + H2O + O2
JK=JK+1 ; JP3_NO_NO =JK
JK=JK+1 ; JP3_NO3_NO3_i =JK
JK=JK+1 ; JP3_NO3_NO3_ii =JK
JK=JK+1 ; JP3_MO2_MO2a =JK ! MO2 + MO2 --> CH3O + CH3O + O2
JK=JK+1 ; JP3_MO2_MO2b =JK ! MO2 + MO2 --> HCHO + O2 + XX
JK=JK+1 ; JP3_ACO3_ACO3 =JK ! ACO3 + ACO3 --> MO2 + MO2
JK=JK+1 ; JP3_OLNN_OLNN =JK ! OLNN + OLNN --> ONIT + ONIT + HO2
JK=JK+1 ; JP3_OLND_OLND_i =JK ! 0.504*HCHO + 1.21*ALD + 0.285*KET + ONIT + NO2
JK=JK+1 ; JP3_OLND_OLND_ii =JK
JK=JK+1 ; JP3_OLND_OLND_iii =JK
JK=JK+1 ; JP3_XO2_XO2 =JK
JK=JK+1 ; JP3_OH_OHa =JK ! --> H2O + O3P
JK=JK+1 ; JP3_OH_OHb =JK ! --> H2O2
JK=JK+1 ; JP3_ClO_ClOa =JK ! --> Cl2O2
JK=JK+1 ; JP3_ClO_ClOb =JK ! --> Cl2 + O2
JK=JK+1 ; JP3_ClO_ClOc =JK ! --> OClO + Cl
JK=JK+1 ; JP3_BrO_BrO =JK
JK=JK+1 ; JP3_O3P_O3P =JK
!
NB_REACTION_3=JK ; ALLOCATE(Zindice_3(NB_REACTION_3)) ; Zindice_3=zloc1
!
zloc1=Z3SPEC(2.0,JPHO2,1.0,JPH2O2,1.0,JPO2)
Zindice_3(JP3_HO2_HO2a)=zloc1
!
zloc1=Z3SPEC(2.0,JPHO2,1.0,JPH2O2,1.0,JPO2)
Zindice_3(JP3_HO2_HO2b)=zloc1
!
zloc1=Z3SPEC(2.0,JPNO,1.0,JPNO2,1.0,JPN)
Zindice_3(JP3_NO_NO)=zloc1
!
zloc1=Z3SPEC(2.0,JPNO3,1.0,JPNO2,1.0,JPNO2)
Zindice_3(JP3_NO3_NO3_i)=zloc1
zloc1=Z3SPEC(0.0,JPNO3,1.0,JPO2,0.0,JPDUMMY)
Zindice_3(JP3_NO3_NO3_ii)=zloc1
!
zloc1=Z3SPEC(2.0,JPMO2,2.0,JPCH3O,1.0,JPO2)
Zindice_3(JP3_MO2_MO2a)=zloc1
!
zloc1=Z3SPEC(2.0,JPMO2,1.33,JPHCHO,0.66,JPO2)
Zindice_3(JP3_MO2_MO2b)=zloc1
!
zloc1=Z3SPEC(2.0,JPOH,1.0,JPH2O,1.0,JPO3P)
Zindice_3(JP3_OH_OHa)=zloc1
!
zloc1=Z3SPEC(2.0,JPOH,1.0,JPH2O2,0.0,JPDUMMY)
Zindice_3(JP3_OH_OHb)=zloc1
!
zloc1=Z3SPEC(2.0,JPClO,1.0,JPCl2O2,0.0,JPDUMMY)
Zindice_3(JP3_ClO_ClOa)=zloc1
!
zloc1=Z3SPEC(2.0,JPClO,1.0,JPCl2,1.0,JPO2)
Zindice_3(JP3_ClO_ClOb)=zloc1
!
zloc1=Z3SPEC(2.0,JPClO,1.0,JPOClO,1.0,JPCl)
Zindice_3(JP3_ClO_ClOc)=zloc1
!
zloc1=Z3SPEC(2.0,JPBrO,2.0,JPBr,1.0,JPO2)
Zindice_3(JP3_BrO_BrO)=zloc1
!
zloc1=Z3SPEC(2.0,JPO3P,1.0,JPO2,0.0,JPDUMMY)
Zindice_3(JP3_O3P_O3P)=zloc1
!
zloc1=Z3SPEC(2.0,JPACO3,2.0,JPMO2,0.0,JPDUMMY)
Zindice_3(JP3_ACO3_ACO3)=zloc1
!
zloc1=Z3SPEC(2.0,JPOLNN,2.0,JPONIT,1.0,JPHO2)
Zindice_3(JP3_OLNN_OLNN)=zloc1
!
! OLND +OLND - 0.504*HCHO + 1.21*ALD + 0.285*KET + ONIT + NO2
zloc1=Z3SPEC(2.0,JPOLND,0.504,JPHCHO,1.21,JPALD)
Zindice_3(JP3_OLND_OLND_i)=zloc1
zloc1=Z3SPEC(0.0,JPOLND,0.285,JPKET,1.0,JPONIT)
Zindice_3(JP3_OLND_OLND_ii)=zloc1
zloc1=Z3SPEC(0.0,JPOLND,1.0,JPNO2,0.0,JPDUMMY)
Zindice_3(JP3_OLND_OLND_iii)=zloc1
!
zloc1=Z3SPEC(2.0,JPXO2,2.0,JPO2,0.0,JPDUMMY)
Zindice_3(JP3_XO2_XO2)=zloc1
!
JK=0
JK=JK+1 ; JP4_O3P_O2 =JK
JK=JK+1 ; JP4_O3P_O3 =JK
JK=JK+1 ; JP4_O1D_O2 =JK
JK=JK+1 ; JP4_O1D_H2O =JK
JK=JK+1 ; JP4_O3_OH =JK
JK=JK+1 ; JP4_O3_HO2 =JK
JK=JK+1 ; JP4_OH_HO2 =JK
JK=JK+1 ; JP4_H2O2_OH =JK
JK=JK+1 ; JP4_O3P_NO =JK
JK=JK+1 ; JP4_O3P_NO2a =JK ! --> NO + O2
JK=JK+1 ; JP4_O3P_NO2b =JK ! --> NO3
JK=JK+1 ; JP4_OH_NO =JK
JK=JK+1 ; JP4_OH_NO2 =JK
JK=JK+1 ; JP4_OH_NO3 =JK
JK=JK+1 ; JP4_HO2_NOa =JK
JK=JK+1 ; JP4_HO2_NO2 =JK
JK=JK+1 ; JP4_HO2_NO3a =JK ! --> HNO3
JK=JK+1 ; JP4_HO2_NO3b =JK ! --> NO2 + OH
JK=JK+1 ; JP4_OH_HONO =JK
JK=JK+1 ; JP4_OH_HNO3 =JK
JK=JK+1 ; JP4_OH_HNO4 =JK
JK=JK+1 ; JP4_O3_NO =JK
JK=JK+1 ; JP4_O3_NO2 =JK
JK=JK+1 ; JP4_NO3_NO =JK
JK=JK+1 ; JP4_NO3_NO2a =JK ! --> NO + NO2 + O2
JK=JK+1 ; JP4_NO3_NO2b =JK ! --> N2O5
JK=JK+1 ; JP4_OH_SO2 =JK
JK=JK+1 ; JP4_CO_OHa =JK ! --> H + CO2
JK=JK+1 ; JP4_ISO_O3P_i =JK
JK=JK+1 ; JP4_ISO_O3P_ii =JK
JK=JK+1 ; JP4_ISO_O3P_iii =JK
JK=JK+1 ; JP4_ISO_O3P_iv =JK
JK=JK+1 ; JP4_MACR_O3P =JK
JK=JK+1 ; JP4_CH4_OH =JK
JK=JK+1 ; JP4_ETH_OH =JK
JK=JK+1 ; JP4_HC3_OH_i =JK
JK=JK+1 ; JP4_HC3_OH_ii =JK
JK=JK+1 ; JP4_HC3_OH_iii =JK
JK=JK+1 ; JP4_HC3_OH_iv =JK
JK=JK+1 ; JP4_HC5_OHa =JK
JK=JK+1 ; JP4_HC5_OHb =JK
JK=JK+1 ; JP4_HC8_OH_i =JK
JK=JK+1 ; JP4_HC8_OH_ii =JK
JK=JK+1 ; JP4_ETE_OH =JK
JK=JK+1 ; JP4_OLT_OH =JK
JK=JK+1 ; JP4_OLI_OH =JK
JK=JK+1 ; JP4_DIEN_OH =JK
JK=JK+1 ; JP4_ISO_OH =JK
JK=JK+1 ; JP4_API_OH =JK
JK=JK+1 ; JP4_LIM_OH =JK
JK=JK+1 ; JP4_TOL_OHa =JK
JK=JK+1 ; JP4_TOL_OHb =JK
JK=JK+1 ; JP4_XYL_OHa =JK
JK=JK+1 ; JP4_XYL_OHb =JK
JK=JK+1 ; JP4_CSL_OHa =JK
JK=JK+1 ; JP4_CSL_OHb =JK
JK=JK+1 ; JP4_CSL_OHc =JK
JK=JK+1 ; JP4_HCHO_OH_i =JK
JK=JK+1 ; JP4_HCHO_OH_ii =JK
JK=JK+1 ; JP4_ALD_OH =JK
JK=JK+1 ; JP4_KET_OH =JK
JK=JK+1 ; JP4_HKET_OH =JK
JK=JK+1 ; JP4_GLY_OH =JK
JK=JK+1 ; JP4_MGLY_OH =JK
JK=JK+1 ; JP4_MACR_OH_i =JK
JK=JK+1 ; JP4_MACR_OH_ii =JK
JK=JK+1 ; JP4_MACR_OH_iii =JK
JK=JK+1 ; JP4_MACR_OH_iv =JK
JK=JK+1 ; JP4_OH_DCB_i =JK
JK=JK+1 ; JP4_OH_DCB_ii =JK
JK=JK+1 ; JP4_OH_DCB_iii =JK
JK=JK+1 ; JP4_OH_UDDa =JK
JK=JK+1 ; JP4_OH_UDDb =JK
JK=JK+1 ; JP4_OP1_OHa =JK ! --> MO2 + H2O
JK=JK+1 ; JP4_OP1_OHb_i =JK ! --> HCHO + OH + H2O
JK=JK+1 ; JP4_OP1_OHb_ii =JK
JK=JK+1 ; JP4_OP2_OH_i =JK
JK=JK+1 ; JP4_OP2_OH_ii =JK
JK=JK+1 ; JP4_OP2_OH_iii =JK
JK=JK+1 ; JP4_PAA_OHa =JK
JK=JK+1 ; JP4_PAA_OHb_i =JK
JK=JK+1 ; JP4_PAA_OHb_ii =JK
JK=JK+1 ; JP4_PAN_OH_i =JK
JK=JK+1 ; JP4_PAN_OH_ii =JK
JK=JK+1 ; JP4_TPAN_OH_i =JK
JK=JK+1 ; JP4_TPAN_OH_ii =JK
JK=JK+1 ; JP4_TPAN_OH_iii =JK
JK=JK+1 ; JP4_ONIT_OH_i =JK
JK=JK+1 ; JP4_ONIT_OH_ii =JK
JK=JK+1 ; JP4_HCHO_NO3_i =JK
JK=JK+1 ; JP4_HCHO_NO3_ii =JK
JK=JK+1 ; JP4_ALD_NO3 =JK
JK=JK+1 ; JP4_GLY_NO3_i =JK
JK=JK+1 ; JP4_GLY_NO3_ii =JK
JK=JK+1 ; JP4_MGLY_NO3_i =JK
JK=JK+1 ; JP4_MGLY_NO3_ii =JK
JK=JK+1 ; JP4_MACR_NO3_i =JK
JK=JK+1 ; JP4_MACR_NO3_ii =JK
JK=JK+1 ; JP4_DCB_NO3_i =JK
JK=JK+1 ; JP4_DCB_NO3_ii =JK
JK=JK+1 ; JP4_DCB_NO3_iii =JK
JK=JK+1 ; JP4_DCB_NO3_iv =JK
JK=JK+1 ; JP4_DCB_NO3_v =JK
JK=JK+1 ; JP4_CSL_NO3 =JK
JK=JK+1 ; JP4_ETE_NO3a =JK
JK=JK+1 ; JP4_ETE_NO3b =JK
JK=JK+1 ; JP4_OLT_NO3a =JK
JK=JK+1 ; JP4_OLT_NO3b =JK
JK=JK+1 ; JP4_OLI_NO3a =JK
JK=JK+1 ; JP4_OLI_NO3b =JK
JK=JK+1 ; JP4_DIEN_NO3a =JK
JK=JK+1 ; JP4_DIEN_NO3b =JK
JK=JK+1 ; JP4_ISO_NO3a =JK
JK=JK+1 ; JP4_ISO_NO3b =JK
JK=JK+1 ; JP4_API_NO3a =JK
JK=JK+1 ; JP4_API_NO3b =JK
JK=JK+1 ; JP4_LIM_NO3a =JK
JK=JK+1 ; JP4_LIM_NO3b =JK
JK=JK+1 ; JP4_TPAN_NO3a_i =JK
JK=JK+1 ; JP4_TPAN_NO3a_ii =JK
JK=JK+1 ; JP4_TPAN_NO3b_i =JK
JK=JK+1 ; JP4_TPAN_NO3b_ii =JK
JK=JK+1 ; JP4_ETE_O3_i =JK
JK=JK+1 ; JP4_ETE_O3_ii =JK
JK=JK+1 ; JP4_ETE_O3_iii =JK
JK=JK+1 ; JP4_OLT_O3_i =JK
JK=JK+1 ; JP4_OLT_O3_ii =JK
JK=JK+1 ; JP4_OLT_O3_iii =JK
JK=JK+1 ; JP4_OLT_O3_iv =JK
JK=JK+1 ; JP4_OLT_O3_v =JK
JK=JK+1 ; JP4_OLT_O3_vi =JK
JK=JK+1 ; JP4_OLT_O3_vii =JK
JK=JK+1 ; JP4_OLI_O3_i =JK
JK=JK+1 ; JP4_OLI_O3_ii =JK
JK=JK+1 ; JP4_OLI_O3_iii =JK
JK=JK+1 ; JP4_OLI_O3_iv =JK
JK=JK+1 ; JP4_OLI_O3_v =JK
JK=JK+1 ; JP4_OLI_O3_vi =JK
JK=JK+1 ; JP4_DIEN_O3_i =JK
JK=JK+1 ; JP4_DIEN_O3_ii =JK
JK=JK+1 ; JP4_DIEN_O3_iii =JK
JK=JK+1 ; JP4_DIEN_O3_iv =JK
JK=JK+1 ; JP4_DIEN_O3_v =JK
JK=JK+1 ; JP4_DIEN_O3_vi =JK
JK=JK+1 ; JP4_ISO_O3_i =JK
JK=JK+1 ; JP4_ISO_O3_ii =JK
JK=JK+1 ; JP4_ISO_O3_iii =JK
JK=JK+1 ; JP4_ISO_O3_iv =JK
JK=JK+1 ; JP4_ISO_O3_v =JK
JK=JK+1 ; JP4_ISO_O3_vi =JK
JK=JK+1 ; JP4_API_O3_i =JK
JK=JK+1 ; JP4_API_O3_ii =JK
JK=JK+1 ; JP4_API_O3_iii =JK
JK=JK+1 ; JP4_API_O3_iv =JK
JK=JK+1 ; JP4_LIM_O3_i =JK
JK=JK+1 ; JP4_LIM_O3_ii =JK
JK=JK+1 ; JP4_LIM_O3_iii =JK
JK=JK+1 ; JP4_LIM_O3_iv =JK
JK=JK+1 ; JP4_LIM_O3_v =JK
JK=JK+1 ; JP4_MACR_O3_i =JK
JK=JK+1 ; JP4_MACR_O3_ii =JK
JK=JK+1 ; JP4_MACR_O3_iii =JK
JK=JK+1 ; JP4_MACR_O3_iv =JK
JK=JK+1 ; JP4_MACR_O3_v =JK
JK=JK+1 ; JP4_DCB_O3_i =JK
JK=JK+1 ; JP4_DCB_O3_ii =JK
JK=JK+1 ; JP4_DCB_O3_iii =JK
JK=JK+1 ; JP4_DCB_O3_iv =JK
JK=JK+1 ; JP4_TPAN_O3_i =JK
JK=JK+1 ; JP4_TPAN_O3_ii =JK
JK=JK+1 ; JP4_TPAN_O3_iii =JK
JK=JK+1 ; JP4_TPAN_O3_iv =JK
JK=JK+1 ; JP4_PHO_NO2 =JK
JK=JK+1 ; JP4_PHO_HO2 =JK
JK=JK+1 ; JP4_ADDT_NO2 =JK
JK=JK+1 ; JP4_ADDT_O2a =JK ! --> TOLP
JK=JK+1 ; JP4_ADDT_O2b =JK ! --> CSL + HO2
JK=JK+1 ; JP4_ADDT_O3 =JK
JK=JK+1 ; JP4_ADDX_NO2 =JK
JK=JK+1 ; JP4_ADDX_O2a =JK
JK=JK+1 ; JP4_ADDX_O2b =JK
JK=JK+1 ; JP4_ADDX_O3 =JK
JK=JK+1 ; JP4_ADDC_NO2 =JK
JK=JK+1 ; JP4_ADDC_O2a =JK
JK=JK+1 ; JP4_ADDC_O2b =JK
JK=JK+1 ; JP4_ADDC_O3 =JK
JK=JK+1 ; JP4_ACO3_NO2 =JK
JK=JK+1 ; JP4_TCO3_NO2 =JK
JK=JK+1 ; JP4_MO2_NO =JK
JK=JK+1 ; JP4_ETHP_NO_i =JK
JK=JK+1 ; JP4_ETHP_NO_ii =JK
JK=JK+1 ; JP4_HC3P_NO_i =JK
JK=JK+1 ; JP4_HC3P_NO_ii =JK
JK=JK+1 ; JP4_HC3P_NO_iii =JK
JK=JK+1 ; JP4_HC3P_NO_iv =JK
JK=JK+1 ; JP4_HC3P_NO_v =JK
JK=JK+1 ; JP4_HC5P_NO_i =JK
JK=JK+1 ; JP4_HC5P_NO_ii =JK
JK=JK+1 ; JP4_HC5P_NO_iii =JK
JK=JK+1 ; JP4_HC5P_NO_iv =JK
JK=JK+1 ; JP4_HC5P_NO_v =JK
JK=JK+1 ; JP4_HC8P_NO_i =JK
JK=JK+1 ; JP4_HC8P_NO_ii =JK
JK=JK+1 ; JP4_HC8P_NO_iii =JK
JK=JK+1 ; JP4_HC8P_NO_iv =JK
JK=JK+1 ; JP4_ETEP_NO_i =JK
JK=JK+1 ; JP4_ETEP_NO_ii =JK
JK=JK+1 ; JP4_OLTP_NO_i =JK
JK=JK+1 ; JP4_OLTP_NO_ii =JK
JK=JK+1 ; JP4_OLTP_NO_iii =JK
JK=JK+1 ; JP4_OLIP_NO_i =JK
JK=JK+1 ; JP4_OLIP_NO_ii =JK
JK=JK+1 ; JP4_ISOP_NO_i =JK
JK=JK+1 ; JP4_ISOP_NO_ii =JK
JK=JK+1 ; JP4_ISOP_NO_iii =JK
JK=JK+1 ; JP4_APIP_NO_i =JK
JK=JK+1 ; JP4_APIP_NO_ii =JK
JK=JK+1 ; JP4_APIP_NO_iii =JK
JK=JK+1 ; JP4_LIMP_NO_i =JK
JK=JK+1 ; JP4_LIMP_NO_ii =JK
JK=JK+1 ; JP4_LIMP_NO_iii =JK
JK=JK+1 ; JP4_TOLP_NO_i =JK
JK=JK+1 ; JP4_TOLP_NO_ii =JK
JK=JK+1 ; JP4_TOLP_NO_iii =JK
JK=JK+1 ; JP4_XYLP_NO_i =JK
JK=JK+1 ; JP4_XYLP_NO_ii =JK
JK=JK+1 ; JP4_XYLP_NO_iii =JK
JK=JK+1 ; JP4_CSLP_NO_i =JK
JK=JK+1 ; JP4_CSLP_NO_ii =JK
JK=JK+1 ; JP4_ACO3_NO =JK
JK=JK+1 ; JP4_TCO3_NO_i =JK
JK=JK+1 ; JP4_TCO3_NO_ii =JK
JK=JK+1 ; JP4_KETP_NO_i =JK
JK=JK+1 ; JP4_KETP_NO_ii =JK
JK=JK+1 ; JP4_KETP_NO_iii =JK
JK=JK+1 ; JP4_OLNN_NO_i =JK
JK=JK+1 ; JP4_OLNN_NO_ii =JK
JK=JK+1 ; JP4_OLND_NO_i =JK
JK=JK+1 ; JP4_OLND_NO_ii =JK
JK=JK+1 ; JP4_MO2_HO2 =JK
JK=JK+1 ; JP4_ETHP_HO2 =JK
JK=JK+1 ; JP4_HC3P_HO2 =JK
JK=JK+1 ; JP4_HC5P_HO2 =JK
JK=JK+1 ; JP4_HC8P_HO2 =JK
JK=JK+1 ; JP4_ETEP_HO2 =JK
JK=JK+1 ; JP4_OLTP_HO2 =JK
JK=JK+1 ; JP4_OLIP_HO2 =JK
JK=JK+1 ; JP4_ISOP_HO2 =JK
JK=JK+1 ; JP4_APIP_HO2 =JK
JK=JK+1 ; JP4_LIMP_HO2 =JK
JK=JK+1 ; JP4_TOLP_HO2 =JK
JK=JK+1 ; JP4_XYLP_HO2 =JK
JK=JK+1 ; JP4_CSLP_HO2 =JK
JK=JK+1 ; JP4_ACO3_HO2a =JK
JK=JK+1 ; JP4_ACO3_HO2b =JK
JK=JK+1 ; JP4_TCO3_HO2a =JK
JK=JK+1 ; JP4_TCO3_HO2b =JK
JK=JK+1 ; JP4_KETP_HO2 =JK
JK=JK+1 ; JP4_OLNN_HO2 =JK
JK=JK+1 ; JP4_OLND_HO2 =JK
JK=JK+1 ; JP4_ETHP_MO2_i =JK
JK=JK+1 ; JP4_ETHP_MO2_ii =JK
JK=JK+1 ; JP4_HC3P_MO2_i =JK
JK=JK+1 ; JP4_HC3P_MO2_ii =JK
JK=JK+1 ; JP4_HC3P_MO2_iii =JK
JK=JK+1 ; JP4_HC3P_MO2_iv =JK
JK=JK+1 ; JP4_HC5P_MO2_i =JK
JK=JK+1 ; JP4_HC5P_MO2_ii =JK
JK=JK+1 ; JP4_HC5P_MO2_iii =JK
JK=JK+1 ; JP4_HC5P_MO2_iv =JK
JK=JK+1 ; JP4_HC8P_MO2_i =JK
JK=JK+1 ; JP4_HC8P_MO2_ii =JK
JK=JK+1 ; JP4_HC8P_MO2_iii =JK
JK=JK+1 ; JP4_ETEP_MO2_i =JK
JK=JK+1 ; JP4_ETEP_MO2_ii =JK
JK=JK+1 ; JP4_OLTP_MO2_i =JK
JK=JK+1 ; JP4_OLTP_MO2_ii =JK
JK=JK+1 ; JP4_OLIP_MO2_i =JK
JK=JK+1 ; JP4_OLIP_MO2_ii =JK
JK=JK+1 ; JP4_ISOP_MO2_i =JK
JK=JK+1 ; JP4_ISOP_MO2_ii =JK
JK=JK+1 ; JP4_ISOP_MO2_iii =JK
JK=JK+1 ; JP4_APIP_MO2_i =JK
JK=JK+1 ; JP4_APIP_MO2_ii =JK
JK=JK+1 ; JP4_LIMP_MO2_i =JK
JK=JK+1 ; JP4_LIMP_MO2_ii =JK
JK=JK+1 ; JP4_TOLP_MO2_i =JK
JK=JK+1 ; JP4_TOLP_MO2_ii =JK
JK=JK+1 ; JP4_TOLP_MO2_iii =JK
JK=JK+1 ; JP4_XYLP_MO2_i =JK
JK=JK+1 ; JP4_XYLP_MO2_ii =JK
JK=JK+1 ; JP4_XYLP_MO2_iii =JK
JK=JK+1 ; JP4_CSLP_MO2_i =JK
JK=JK+1 ; JP4_CSLP_MO2_ii =JK
JK=JK+1 ; JP4_ACO3_MO2a_i =JK
JK=JK+1 ; JP4_ACO3_MO2a_ii =JK
JK=JK+1 ; JP4_ACO3_MO2b =JK
JK=JK+1 ; JP4_TCO3_MO2a_i =JK
JK=JK+1 ; JP4_TCO3_MO2a_ii =JK
JK=JK+1 ; JP4_TCO3_MO2b =JK
JK=JK+1 ; JP4_KETP_MO2_i =JK
JK=JK+1 ; JP4_KETP_MO2_ii =JK
JK=JK+1 ; JP4_KETP_MO2_iii =JK
JK=JK+1 ; JP4_KETP_MO2_iv =JK
JK=JK+1 ; JP4_OLNN_MO2_i =JK
JK=JK+1 ; JP4_OLNN_MO2_ii =JK
JK=JK+1 ; JP4_OLND_MO2_i =JK
JK=JK+1 ; JP4_OLND_MO2_ii =JK
JK=JK+1 ; JP4_OLND_MO2_iii =JK
JK=JK+1 ; JP4_ETHP_ACO3_i =JK
JK=JK+1 ; JP4_ETHP_ACO3_ii =JK
JK=JK+1 ; JP4_HC3P_ACO3_i =JK
JK=JK+1 ; JP4_HC3P_ACO3_ii =JK
JK=JK+1 ; JP4_HC3P_ACO3_iii =JK
JK=JK+1 ; JP4_HC3P_ACO3_iv =JK
JK=JK+1 ; JP4_HC3P_ACO3_v =JK
JK=JK+1 ; JP4_HC5P_ACO3_i =JK
JK=JK+1 ; JP4_HC5P_ACO3_ii =JK
JK=JK+1 ; JP4_HC5P_ACO3_iii =JK
JK=JK+1 ; JP4_HC5P_ACO3_iv =JK
JK=JK+1 ; JP4_HC8P_ACO3_i =JK
JK=JK+1 ; JP4_HC8P_ACO3_ii =JK
JK=JK+1 ; JP4_HC8P_ACO3_iii =JK
JK=JK+1 ; JP4_HC8P_ACO3_iv =JK
JK=JK+1 ; JP4_ETEP_ACO3_i =JK
JK=JK+1 ; JP4_ETEP_ACO3_ii =JK
JK=JK+1 ; JP4_OLTP_ACO3_i =JK
JK=JK+1 ; JP4_OLTP_ACO3_ii =JK
JK=JK+1 ; JP4_OLTP_ACO3_iii =JK
JK=JK+1 ; JP4_OLIP_ACO3_i =JK
JK=JK+1 ; JP4_OLIP_ACO3_ii =JK
JK=JK+1 ; JP4_ISOP_ACO3_i =JK
JK=JK+1 ; JP4_ISOP_ACO3_ii =JK
JK=JK+1 ; JP4_ISOP_ACO3_iii =JK
JK=JK+1 ; JP4_APIP_ACO3_i =JK
JK=JK+1 ; JP4_APIP_ACO3_ii =JK
JK=JK+1 ; JP4_LIMP_ACO3_i =JK
JK=JK+1 ; JP4_LIMP_ACO3_ii =JK
JK=JK+1 ; JP4_LIMP_ACO3_iii =JK
JK=JK+1 ; JP4_TOLP_ACO3_i =JK
JK=JK+1 ; JP4_TOLP_ACO3_ii =JK
JK=JK+1 ; JP4_TOLP_ACO3_iii =JK
JK=JK+1 ; JP4_XYLP_ACO3_i =JK
JK=JK+1 ; JP4_XYLP_ACO3_ii =JK
JK=JK+1 ; JP4_XYLP_ACO3_iii =JK
JK=JK+1 ; JP4_CSLP_ACO3_i =JK
JK=JK+1 ; JP4_CSLP_ACO3_ii =JK
JK=JK+1 ; JP4_TCO3_ACO3_i =JK
JK=JK+1 ; JP4_TCO3_ACO3_ii =JK
JK=JK+1 ; JP4_KETP_ACO3_i =JK
JK=JK+1 ; JP4_KETP_ACO3_ii =JK
JK=JK+1 ; JP4_KETP_ACO3_iii =JK
JK=JK+1 ; JP4_KETP_ACO3_iv =JK
JK=JK+1 ; JP4_OLNN_ACO3_i =JK
JK=JK+1 ; JP4_OLNN_ACO3_ii =JK
JK=JK+1 ; JP4_OLND_ACO3_i =JK
JK=JK+1 ; JP4_OLND_ACO3_ii =JK
JK=JK+1 ; JP4_OLND_ACO3_iii =JK
JK=JK+1 ; JP4_OLNN_OLND_i =JK ! 0.202*HCHO + 0.640*ALD + 0.149*KET +0.500*HO2 + 0.500*NO2 + 1.50*ONIT
JK=JK+1 ; JP4_OLNN_OLND_ii =JK
JK=JK+1 ; JP4_OLNN_OLND_iii =JK
JK=JK+1 ; JP4_MO2_NO3_i =JK
JK=JK+1 ; JP4_MO2_NO3_ii =JK
JK=JK+1 ; JP4_ETHP_NO3_i =JK
JK=JK+1 ; JP4_ETHP_NO3_ii =JK
JK=JK+1 ; JP4_HC3P_NO3_i =JK
JK=JK+1 ; JP4_HC3P_NO3_ii =JK
JK=JK+1 ; JP4_HC3P_NO3_iii =JK
JK=JK+1 ; JP4_HC3P_NO3_iv =JK
JK=JK+1 ; JP4_HC3P_NO3_v =JK
JK=JK+1 ; JP4_HC5P_NO3_i =JK
JK=JK+1 ; JP4_HC5P_NO3_ii =JK
JK=JK+1 ; JP4_HC5P_NO3_iii =JK
JK=JK+1 ; JP4_HC5P_NO3_iv =JK
JK=JK+1 ; JP4_HC8P_NO3_i =JK
JK=JK+1 ; JP4_HC8P_NO3_ii =JK
JK=JK+1 ; JP4_HC8P_NO3_iii =JK
JK=JK+1 ; JP4_ETEP_NO3_i =JK
JK=JK+1 ; JP4_ETEP_NO3_ii =JK
JK=JK+1 ; JP4_OLTP_NO3_i =JK
JK=JK+1 ; JP4_OLTP_NO3_ii =JK
JK=JK+1 ; JP4_OLTP_NO3_iii =JK
JK=JK+1 ; JP4_OLIP_NO3_i =JK
JK=JK+1 ; JP4_OLIP_NO3_ii =JK
JK=JK+1 ; JP4_ISOP_NO3_i =JK
JK=JK+1 ; JP4_ISOP_NO3_ii =JK
JK=JK+1 ; JP4_ISOP_NO3_iii =JK
JK=JK+1 ; JP4_APIP_NO3_i =JK
JK=JK+1 ; JP4_APIP_NO3_ii =JK
JK=JK+1 ; JP4_LIMP_NO3_i =JK
JK=JK+1 ; JP4_LIMP_NO3_ii =JK
JK=JK+1 ; JP4_LIMP_NO3_iii =JK
JK=JK+1 ; JP4_TOLP_NO3_i =JK
JK=JK+1 ; JP4_TOLP_NO3_ii =JK
JK=JK+1 ; JP4_TOLP_NO3_iii =JK
JK=JK+1 ; JP4_XYLP_NO3_i =JK
JK=JK+1 ; JP4_XYLP_NO3_ii =JK
JK=JK+1 ; JP4_XYLP_NO3_iii =JK
JK=JK+1 ; JP4_CSLP_NO3_i =JK
JK=JK+1 ; JP4_CSLP_NO3_ii =JK
JK=JK+1 ; JP4_ACO3_NO3 =JK
JK=JK+1 ; JP4_TCO3_NO3_i =JK
JK=JK+1 ; JP4_TCO3_NO3_ii =JK
JK=JK+1 ; JP4_KETP_NO3_i =JK
JK=JK+1 ; JP4_KETP_NO3_ii =JK
JK=JK+1 ; JP4_KETP_NO3_iii =JK
JK=JK+1 ; JP4_OLNN_NO3_i =JK
JK=JK+1 ; JP4_OLNN_NO3_ii =JK
JK=JK+1 ; JP4_OLND_NO3_i =JK
JK=JK+1 ; JP4_OLND_NO3_ii =JK
JK=JK+1 ; JP4_XO2_HO2 =JK
JK=JK+1 ; JP4_XO2_MO2 =JK
JK=JK+1 ; JP4_XO2_ACO3 =JK
JK=JK+1 ; JP4_XO2_NO =JK
JK=JK+1 ; JP4_XO2_NO3 =JK
JK=JK+1 ; JP4_DMS_NO3 =JK
JK=JK+1 ; JP4_DMS_O3P =JK
JK=JK+1 ; JP4_DMS_OH =JK
!
JK=JK+1 ; JP4_H_O2 =JK
JK=JK+1 ; JP4_O3_H =JK
JK=JK+1 ; JP4_O3P_OH =JK
JK=JK+1 ; JP4_O3P_HO2 =JK
JK=JK+1 ; JP4_H_HO2a =JK ! --> OH + OH
JK=JK+1 ; JP4_H_HO2b =JK ! --> H2 + O2
JK=JK+1 ; JP4_H_HO2c =JK ! --> H2O + O3P
! JK=JK+1 ; JP4_H2_O3P =JK
JK=JK+1 ; JP4_N_NO =JK
JK=JK+1 ; JP4_N_O2 =JK
JK=JK+1 ; JP4_O1D_N2Oa =JK ! --> N2 + O2
JK=JK+1 ; JP4_O1D_N2Ob =JK ! --> NO + NO
JK=JK+1 ; JP4_O1D_CH4a =JK ! --> OH + CH3
JK=JK+1 ; JP4_O1D_CH4b =JK ! --> CH3O + H
JK=JK+1 ; JP4_O1D_CH4c =JK ! --> HCHO + H2
JK=JK+1 ; JP4_CH3_O3P =JK
JK=JK+1 ; JP4_CH3_O2 =JK
JK=JK+1 ; JP4_HCHO_O3P_i =JK
JK=JK+1 ; JP4_HCHO_O3P_ii =JK
JK=JK+1 ; JP4_HCHO_Cl_i =JK
JK=JK+1 ; JP4_HCHO_Cl_ii =JK
JK=JK+1 ; JP4_CH3O_O2 =JK
JK=JK+1 ; JP4_MO2_ClO =JK
JK=JK+1 ; JP4_HCHO_Br_i =JK
JK=JK+1 ; JP4_HCHO_Br_ii =JK
JK=JK+1 ; JP4_Cl_O3 =JK
JK=JK+1 ; JP4_ClO_O3P =JK
JK=JK+1 ; JP4_ClO_NO =JK
JK=JK+1 ; JP4_Cl_CH4 =JK
! JK=JK+1 ; JP4_Cl_H2 =JK
JK=JK+1 ; JP4_Cl_HO2a =JK ! --> HCl + O2
JK=JK+1 ; JP4_Cl_HO2b =JK ! --> OH + ClO
JK=JK+1 ; JP4_ClO_OHa =JK ! --> Cl + HO2
JK=JK+1 ; JP4_ClO_OHb =JK ! --> HCl + O2
JK=JK+1 ; JP4_OH_HCl =JK
JK=JK+1 ; JP4_ClO_NO2 =JK
JK=JK+1 ; JP4_O3P_ClONO2 =JK
JK=JK+1 ; JP4_ClO_HO2 =JK
JK=JK+1 ; JP4_OH_HOCl =JK
JK=JK+1 ; JP4_O3P_HOCl =JK
JK=JK+1 ; JP4_Cl_NO2 =JK
JK=JK+1 ; JP4_Cl_HOCl =JK
JK=JK+1 ; JP4_O1D_CFC11 =JK
JK=JK+1 ; JP4_O1D_CFC12 =JK
JK=JK+1 ; JP4_O1D_CFC113 =JK
JK=JK+1 ; JP4_O1D_CCl4 =JK
JK=JK+1 ; JP4_OH_CH3CCl3 =JK
JK=JK+1 ; JP4_OH_CH3Cl =JK
JK=JK+1 ; JP4_O1D_HCFC22 =JK
JK=JK+1 ; JP4_OH_HCFC22 =JK
JK=JK+1 ; JP4_Br_O3 =JK
JK=JK+1 ; JP4_BrO_O3P =JK
JK=JK+1 ; JP4_BrO_NO =JK
JK=JK+1 ; JP4_BrO_ClOa =JK ! --> OClO + Br
JK=JK+1 ; JP4_BrO_ClOb =JK ! --> Br + Cl + O2
JK=JK+1 ; JP4_BrO_ClOc =JK ! --> BrCl + O2
JK=JK+1 ; JP4_Br_HO2 =JK
JK=JK+1 ; JP4_O1D_HBr =JK
JK=JK+1 ; JP4_BrO_OH =JK
JK=JK+1 ; JP4_OH_HBr =JK
JK=JK+1 ; JP4_BrO_NO2 =JK
JK=JK+1 ; JP4_BrO_HO2 =JK
JK=JK+1 ; JP4_HOBr_O3P =JK
JK=JK+1 ; JP4_O1D_CH3Br =JK
JK=JK+1 ; JP4_OH_CH3Br =JK
JK=JK+1 ; JP4_O1D_H1211 =JK
JK=JK+1 ; JP4_O1D_H1301 =JK

#ifdef HETERO
 JK=JK+1 ; JP4_ClONO2_H2Os =JK
 JK=JK+1 ; JP4_ClONO2_HCls =JK
 JK=JK+1 ; JP4_N2O5_H2Os =JK
 JK=JK+1 ; JP4_N2O5_HCls =JK
 JK=JK+1 ; JP4_HOCl_HCls =JK
 JK=JK+1 ; JP4_BrONO2_H2Os =JK
 JK=JK+1 ; JP4_HOBr_HCls =JK
 JK=JK+1 ; JP4_HOBr_HBrs_i =JK
 JK=JK+1 ; JP4_HOBr_HBrs_ii =JK
 JK=JK+1 ; JP4_HOCl_HBrs =JK
#endif

JK=JK+1 ; JP4_HO2_NOb =JK
!
NB_REACTION_4=JK ; ALLOCATE(Zindice_4(NB_REACTION_4)) ; Zindice_4=zloc2
!
zloc2=Z4SPEC(1.0,JPO3P,1.0,JPO2,1.0,JPO3,0.0,JPDUMMY)
Zindice_4(JP4_O3P_O2)=zloc2
!
zloc2=Z4SPEC(1.0,JPO1D,1.0,JPO2,1.0,JPO3P,1.0,JPO2)
Zindice_4(JP4_O1D_O2)=zloc2
!
zloc2=Z4SPEC(1.0,JPH,1.0,JPO2,1.0,JPHO2,0.0,JPDUMMY)
Zindice_4(JP4_H_O2)=zloc2
!
zloc2=Z4SPEC(1.0,JPN,1.0,JPO2,1.0,JPNO,1.0,JPO3P)
Zindice_4(JP4_N_O2)=zloc2
!
zloc2=Z4SPEC(1.0,JPCH3,1.0,JPO2,1.0,JPMO2,0.0,JPDUMMY)
Zindice_4(JP4_CH3_O2)=zloc2
!
zloc2=Z4SPEC(1.0,JPCH3O,1.0,JPO2,1.0,JPHCHO,1.0,JPHO2)
Zindice_4(JP4_CH3O_O2)=zloc2
!
zloc2=Z4SPEC(1.0,JPO3P,1.0,JPO3,1.0,JPO2,1.0,JPO2)
Zindice_4(JP4_O3P_O3)=zloc2
!
zloc2=Z4SPEC(1.0,JPO1D,1.0,JPH2O,1.0,JPOH,1.0,JPOH)
Zindice_4(JP4_O1D_H2O)=zloc2
!
zloc2=Z4SPEC(1.0,JPO3,1.0,JPOH,1.0,JPHO2,1.0,JPO2)
Zindice_4(JP4_O3_OH)=zloc2
!
zloc2=Z4SPEC(1.0,JPO3,1.0,JPHO2,1.0,JPOH,2.0,JPO2)
Zindice_4(JP4_O3_HO2)=zloc2
!
zloc2=Z4SPEC(1.0,JPOH,1.0,JPHO2,1.0,JPH2O,1.0,JPO2)
Zindice_4(JP4_OH_HO2)=zloc2
!
zloc2=Z4SPEC(1.0,JPH2O2,1.0,JPOH,1.0,JPHO2,1.0,JPH2O)
Zindice_4(JP4_H2O2_OH)=zloc2
!
zloc2=Z4SPEC(1.0,JPO3P,1.0,JPNO,1.0,JPNO2,0.0,JPDUMMY)
Zindice_4(JP4_O3P_NO)=zloc2
!
zloc2=Z4SPEC(1.0,JPO3P,1.0,JPNO2,1.0,JPNO,1.0,JPO2)
Zindice_4(JP4_O3P_NO2a)=zloc2
!
zloc2=Z4SPEC(1.0,JPO3P,1.0,JPNO2,1.0,JPNO3,0.0,JPDUMMY)
Zindice_4(JP4_O3P_NO2b)=zloc2
!
zloc2=Z4SPEC(1.0,JPOH,1.0,JPNO2,1.0,JPHNO3,0.0,JPDUMMY)
Zindice_4(JP4_OH_NO2)=zloc2
!
zloc2=Z4SPEC(1.0,JPOH,1.0,JPNO3,1.0,JPNO2,1.0,JPHO2)
Zindice_4(JP4_OH_NO3)=zloc2
!
zloc2=Z4SPEC(1.0,JPHO2,1.0,JPNO,1.0,JPNO2,1.0,JPOH)
Zindice_4(JP4_HO2_NOa)=zloc2
!
zloc2=Z4SPEC(1.0,JPHO2,1.0,JPNO,1.0,JPHNO3,0.0,JPDUMMY)
Zindice_4(JP4_HO2_NOb)=zloc2
!
zloc2=Z4SPEC(1.0,JPHO2,1.0,JPNO2,1.0,JPHNO4,0.0,JPDUMMY)
Zindice_4(JP4_HO2_NO2)=zloc2
!
zloc2=Z4SPEC(1.0,JPHO2,1.0,JPNO3,1.0,JPHNO3,1.0,JPO2)
Zindice_4(JP4_HO2_NO3a)=zloc2
!
zloc2=Z4SPEC(1.0,JPHO2,1.0,JPNO3,1.0,JPNO2,1.0,JPOH)
Zindice_4(JP4_HO2_NO3b)=zloc2
!
zloc2=Z4SPEC(1.0,JPOH,1.0,JPHNO3,1.0,JPNO3,1.0,JPH2O)
Zindice_4(JP4_OH_HNO3)=zloc2
!
zloc2=Z4SPEC(1.0,JPOH,1.0,JPHNO4,1.0,JPNO2,1.0,JPH2O)
Zindice_4(JP4_OH_HNO4)=zloc2
!
zloc2=Z4SPEC(1.0,JPO3,1.0,JPNO,1.0,JPNO2,1.0,JPO2)
Zindice_4(JP4_O3_NO)=zloc2
!
zloc2=Z4SPEC(1.0,JPO3,1.0,JPNO2,1.0,JPNO3,1.0,JPO2)
Zindice_4(JP4_O3_NO2)=zloc2
!
zloc2=Z4SPEC(1.0,JPNO3,1.0,JPNO,1.0,JPNO2,1.0,JPNO2)
Zindice_4(JP4_NO3_NO)=zloc2
!
zloc2=Z4SPEC(1.0,JPNO3,1.0,JPNO2,1.0,JPNO,1.0,JPNO2)
Zindice_4(JP4_NO3_NO2a)=zloc2
!
zloc2=Z4SPEC(1.0,JPNO3,1.0,JPNO2,1.0,JPN2O5,0.0,JPDUMMY)
Zindice_4(JP4_NO3_NO2b)=zloc2
!
zloc2=Z4SPEC(1.0,JPCO,1.0,JPOH,1.0,JPH,1.0,JPCO2)
Zindice_4(JP4_CO_OHa)=zloc2
!
zloc2=Z4SPEC(1.0,JPCH4,1.0,JPOH,1.0,JPCH3,1.0,JPH2O)
Zindice_4(JP4_CH4_OH)=zloc2
!
zloc2=Z4SPEC(1.0,JPHCHO,1.0,JPOH,1.0,JPHO2,1.0,JPCO)
Zindice_4(JP4_HCHO_OH_i)=zloc2
zloc2=Z4SPEC(0.0,JPHCHO,0.0,JPOH,1.0,JPH2O,0.0,JPDUMMY)
Zindice_4(JP4_HCHO_OH_ii)=zloc2
!
zloc2=Z4SPEC(1.0,JPOP1,1.0,JPOH,1.0,JPMO2,1.0,JPH2O)
Zindice_4(JP4_OP1_OHa)=zloc2
!
zloc2=Z4SPEC(1.0,JPOP1,1.0,JPOH,1.0,JPHCHO,1.0,JPOH)
Zindice_4(JP4_OP1_OHb_i)=zloc2
zloc2=Z4SPEC(0.0,JPOP1,0.0,JPOH,1.0,JPH2O,0.0,JPDUMMY)
Zindice_4(JP4_OP1_OHb_ii)=zloc2
!
zloc2=Z4SPEC(1.0,JPHCHO,1.0,JPNO3,1.0,JPHO2,1.0,JPHNO3)
Zindice_4(JP4_HCHO_NO3_i)=zloc2
zloc2=Z4SPEC(0.0,JPHCHO,0.0,JPNO3,1.0,JPCO,0.0,JPDUMMY)
Zindice_4(JP4_HCHO_NO3_ii)=zloc2
!
zloc2=Z4SPEC(1.0,JPMO2,1.0,JPNO,1.0,JPCH3O,1.0,JPNO2)
Zindice_4(JP4_MO2_NO)=zloc2
!
zloc2=Z4SPEC(1.0,JPETHP,1.0,JPNO,1.0,JPALD,1.0,JPHO2)
Zindice_4(JP4_ETHP_NO_i)=zloc2
zloc2=Z4SPEC(0.0,JPETHP,0.0,JPNO,1.0,JPNO2,0.0,JPDUMMY)
Zindice_4(JP4_ETHP_NO_ii)=zloc2
!
zloc2=Z4SPEC(1.0,JPHC3P,1.0,JPNO,0.047,JPHCHO,0.233,JPALD)
Zindice_4(JP4_HC3P_NO_i)=zloc2
zloc2=Z4SPEC(0.0,JPHC3P,0.0,JPNO,0.623,JPKET,0.063,JPGLY)
Zindice_4(JP4_HC3P_NO_ii)=zloc2
zloc2=Z4SPEC(0.0,JPHC3P,0.0,JPNO,0.742,JPHO2,0.150,JPMO2)
Zindice_4(JP4_HC3P_NO_iii)=zloc2
zloc2=Z4SPEC(0.0,JPHC3P,0.0,JPNO,0.048,JPETHP,0.048,JPXO2)
Zindice_4(JP4_HC3P_NO_iv)=zloc2
zloc2=Z4SPEC(0.0,JPHC3P,0.0,JPNO,0.059,JPONIT,0.941,JPNO2)
Zindice_4(JP4_HC3P_NO_v)=zloc2
!
zloc2=Z4SPEC(1.0,JPHC5P,1.0,JPNO,0.021,JPHCHO,0.211,JPALD)
Zindice_4(JP4_HC5P_NO_i)=zloc2
zloc2=Z4SPEC(0.0,JPHC5P,0.0,JPNO,0.722,JPKET,0.599,JPHO2)
Zindice_4(JP4_HC5P_NO_ii)=zloc2
zloc2=Z4SPEC(0.0,JPHC5P,0.0,JPNO,0.031,JPMO2,0.245,JPETHP)
Zindice_4(JP4_HC5P_NO_iii)=zloc2
zloc2=Z4SPEC(0.0,JPHC5P,0.0,JPNO,0.334,JPXO2,0.124,JPONIT)
Zindice_4(JP4_HC5P_NO_iv)=zloc2
zloc2=Z4SPEC(0.0,JPHC5P,0.0,JPNO,0.876,JPNO2,0.0,JPDUMMY)
Zindice_4(JP4_HC5P_NO_v)=zloc2
!
zloc2=Z4SPEC(1.0,JPMO2,1.0,JPHO2,1.0,JPOP1,0.0,JPDUMMY)
Zindice_4(JP4_MO2_HO2)=zloc2
!
zloc2=Z4SPEC(1.0,JPETHP,1.0,JPHO2,1.0,JPOP2,0.0,JPDUMMY)
Zindice_4(JP4_ETHP_HO2)=zloc2
!
zloc2=Z4SPEC(1.0,JPHC3P,1.0,JPHO2,1.0,JPOP2,0.0,JPDUMMY)
Zindice_4(JP4_HC3P_HO2)=zloc2
!
zloc2=Z4SPEC(1.0,JPHC5P,1.0,JPHO2,1.0,JPOP2,0.0,JPDUMMY)
Zindice_4(JP4_HC5P_HO2)=zloc2
!
zloc2=Z4SPEC(1.0,JPHC8P,1.0,JPHO2,1.0,JPOP2,0.0,JPDUMMY)
Zindice_4(JP4_HC8P_HO2)=zloc2
!
zloc2=Z4SPEC(1.0,JPETEP,1.0,JPHO2,1.0,JPOP2,0.0,JPDUMMY)
Zindice_4(JP4_ETEP_HO2)=zloc2
!
zloc2=Z4SPEC(1.0,JPOLTP,1.0,JPHO2,1.0,JPOP2,0.0,JPDUMMY)
Zindice_4(JP4_OLTP_HO2)=zloc2
!
zloc2=Z4SPEC(1.0,JPMO2,1.0,JPNO3,1.0,JPHCHO,1.0,JPHO2)
Zindice_4(JP4_MO2_NO3_i)=zloc2
zloc2=Z4SPEC(0.0,JPMO2,0.0,JPNO3,1.0,JPNO2,0.0,JPDUMMY)
Zindice_4(JP4_MO2_NO3_ii)=zloc2
!
zloc2=Z4SPEC(1.0,JPETHP,1.0,JPNO3,1.0,JPALD,1.0,JPHO2)
Zindice_4(JP4_ETHP_NO3_i)=zloc2
zloc2=Z4SPEC(0.0,JPETHP,0.0,JPNO3,1.0,JPNO2,0.0,JPDUMMY)
Zindice_4(JP4_ETHP_NO3_ii)=zloc2
!
zloc2=Z4SPEC(1.0,JPHC3P,1.0,JPNO3,0.048,JPHCHO,0.243,JPALD)
Zindice_4(JP4_HC3P_NO3_i)=zloc2
zloc2=Z4SPEC(0.0,JPHC3P,0.0,JPNO3,0.670,JPKET,0.063,JPGLY)
Zindice_4(JP4_HC3P_NO3_ii)=zloc2
zloc2=Z4SPEC(0.0,JPHC3P,0.0,JPNO3,0.792,JPHO2,0.155,JPMO2)
Zindice_4(JP4_HC3P_NO3_iii)=zloc2
zloc2=Z4SPEC(0.0,JPHC3P,0.0,JPNO3,0.053,JPETHP,0.051,JPXO2)
Zindice_4(JP4_HC3P_NO3_iv)=zloc2
zloc2=Z4SPEC(0.0,JPHC3P,0.0,JPNO3,1.0,JPNO2,0.0,JPDUMMY)
Zindice_4(JP4_HC3P_NO3_v)=zloc2
! HC5P + NO3 --> 0.021*HCHO + 0.239*ALD + 0.828*KET +
! 0.699*HO2 + 0.040*MO2 + 0.262*ETHP + 0.391*XO2 + NO2
zloc2=Z4SPEC(1.0,JPHC5P,1.0,JPNO3,0.021,JPHCHO,0.239,JPALD)
Zindice_4(JP4_HC5P_NO3_i)=zloc2
zloc2=Z4SPEC(0.0,JPHC5P,0.0,JPNO3,0.828,JPKET,0.699,JPHO2)
Zindice_4(JP4_HC5P_NO3_ii)=zloc2
zloc2=Z4SPEC(0.0,JPHC5P,0.0,JPNO3,0.040,JPMO2,0.262,JPETHP)
Zindice_4(JP4_HC5P_NO3_iii)=zloc2
zloc2=Z4SPEC(0.0,JPHC5P,0.0,JPNO3,0.391,JPXO2,1.0,JPNO2)
Zindice_4(JP4_HC5P_NO3_iv)=zloc2
!
zloc2=Z4SPEC(1.0,JPO3,1.0,JPH,1.0,JPOH,1.0,JPO2)
Zindice_4(JP4_O3_H)=zloc2
!
zloc2=Z4SPEC(1.0,JPO3P,1.0,JPOH,1.0,JPH,1.0,JPO2)
Zindice_4(JP4_O3P_OH)=zloc2
!
zloc2=Z4SPEC(1.0,JPO3P,1.0,JPHO2,1.0,JPOH,1.0,JPO2)
Zindice_4(JP4_O3P_HO2)=zloc2
!
zloc2=Z4SPEC(1.0,JPH,1.0,JPHO2,1.0,JPOH,1.0,JPOH)
Zindice_4(JP4_H_HO2a)=zloc2
!
zloc2=Z4SPEC(1.0,JPH,1.0,JPHO2,1.0,JPO2,0.0,JPDUMMY)
Zindice_4(JP4_H_HO2b)=zloc2
!
zloc2=Z4SPEC(1.0,JPH,1.0,JPHO2,1.0,JPH2O,1.0,JPO3P)
Zindice_4(JP4_H_HO2c)=zloc2
!
!zloc2=Z4SPEC(1.0,JPH2,1.0,JPO3P,1.0,JPOH,1.0,JPH)
!Zindice_4(JP4_H_HO2c)=zloc2
!
zloc2=Z4SPEC(1.0,JPN,1.0,JPNO,1.0,JPO3P,0.0,JPDUMMY)
Zindice_4(JP4_N_NO)=zloc2
!
zloc2=Z4SPEC(1.0,JPO1D,1.0,JPN2O,1.0,JPO2,0.0,JPDUMMY)
Zindice_4(JP4_O1D_N2Oa)=zloc2
!
zloc2=Z4SPEC(1.0,JPO1D,1.0,JPN2O,1.0,JPNO,1.0,JPNO)
Zindice_4(JP4_O1D_N2Ob)=zloc2
!
zloc2=Z4SPEC(1.0,JPO1D,1.0,JPCH4,1.0,JPOH,1.0,JPCH3)
Zindice_4(JP4_O1D_CH4a)=zloc2
!
zloc2=Z4SPEC(1.0,JPO1D,1.0,JPCH4,1.0,JPCH3O,1.0,JPH)
Zindice_4(JP4_O1D_CH4b)=zloc2
!
zloc2=Z4SPEC(1.0,JPO1D,1.0,JPCH4,1.0,JPHCHO,0.0,JPDUMMY)
Zindice_4(JP4_O1D_CH4c)=zloc2
!
zloc2=Z4SPEC(1.0,JPCH3,1.0,JPO3P,1.0,JPHCHO,1.0,JPH)
Zindice_4(JP4_CH3_O3P)=zloc2
!
zloc2=Z4SPEC(1.0,JPHCHO,1.0,JPO3P,1.0,JPHO2,1.0,JPCO)
Zindice_4(JP4_HCHO_O3P_i)=zloc2
zloc2=Z4SPEC(0.0,JPHCHO,0.0,JPO3P,1.0,JPOH,0.0,JPDUMMY)
Zindice_4(JP4_HCHO_O3P_ii)=zloc2
!
zloc2=Z4SPEC(1.0,JPHCHO,1.0,JPCl,1.0,JPHO2,1.0,JPCO)
Zindice_4(JP4_HCHO_Cl_i)=zloc2
zloc2=Z4SPEC(0.0,JPHCHO,0.0,JPCl,1.0,JPHCl,0.0,JPDUMMY)
Zindice_4(JP4_HCHO_Cl_ii)=zloc2
!
zloc2=Z4SPEC(1.0,JPMO2,1.0,JPClO,1.0,JPCH3O,1.0,JPCl)
Zindice_4(JP4_MO2_ClO)=zloc2
!
zloc2=Z4SPEC(1.0,JPHCHO,1.0,JPBr,1.0,JPHO2,1.0,JPCO)
Zindice_4(JP4_HCHO_Br_i)=zloc2
zloc2=Z4SPEC(0.0,JPHCHO,0.0,JPBr,1.0,JPHBr,0.0,JPDUMMY)
Zindice_4(JP4_HCHO_Br_ii)=zloc2
!
zloc2=Z4SPEC(1.0,JPCl,1.0,JPO3,1.0,JPClO,1.0,JPO2)
Zindice_4(JP4_Cl_O3)=zloc2
!
zloc2=Z4SPEC(1.0,JPClO,1.0,JPO3P,1.0,JPCl,1.0,JPO2)
Zindice_4(JP4_ClO_O3P)=zloc2
!
zloc2=Z4SPEC(1.0,JPClO,1.0,JPNO,1.0,JPNO2,1.0,JPCl)
Zindice_4(JP4_ClO_NO)=zloc2
!
zloc2=Z4SPEC(1.0,JPCl,1.0,JPCH4,1.0,JPHCl,1.0,JPCH3)
Zindice_4(JP4_Cl_CH4)=zloc2
!
zloc2=Z4SPEC(1.0,JPCl,1.0,JPHO2,1.0,JPHCl,1.0,JPO2)
Zindice_4(JP4_Cl_HO2a)=zloc2
!
zloc2=Z4SPEC(1.0,JPCl,1.0,JPHO2,1.0,JPOH,1.0,JPClO)
Zindice_4(JP4_Cl_HO2b)=zloc2
!
zloc2=Z4SPEC(1.0,JPClO,1.0,JPOH,1.0,JPCl,1.0,JPHO2)
Zindice_4(JP4_ClO_OHa)=zloc2
!
zloc2=Z4SPEC(1.0,JPClO,1.0,JPOH,1.0,JPHCl,1.0,JPO2)
Zindice_4(JP4_ClO_OHb)=zloc2
!
zloc2=Z4SPEC(1.0,JPOH,1.0,JPHCl,1.0,JPH2O,1.0,JPCl)
Zindice_4(JP4_OH_HCl)=zloc2
!
zloc2=Z4SPEC(1.0,JPClO,1.0,JPNO2,1.0,JPClONO2,0.0,JPDUMMY)
Zindice_4(JP4_ClO_NO2)=zloc2
!
zloc2=Z4SPEC(1.0,JPO3P,1.0,JPClONO2,1.0,JPClO,1.0,JPNO3)
Zindice_4(JP4_O3P_ClONO2)=zloc2
!
zloc2=Z4SPEC(1.0,JPClO,1.0,JPHO2,1.0,JPHOCl,1.0,JPO2)
Zindice_4(JP4_ClO_HO2)=zloc2
!
zloc2=Z4SPEC(1.0,JPOH,1.0,JPHOCl,1.0,JPH2O,1.0,JPClO)
Zindice_4(JP4_OH_HOCl)=zloc2
!
zloc2=Z4SPEC(1.0,JPO3P,1.0,JPHOCl,1.0,JPOH,1.0,JPClO)
Zindice_4(JP4_O3P_HOCl)=zloc2
!
zloc2=Z4SPEC(1.0,JPCl,1.0,JPNO2,1.0,JPClNO2,0.0,JPDUMMY)
Zindice_4(JP4_Cl_NO2)=zloc2
!
zloc2=Z4SPEC(1.0,JPCl,1.0,JPHOCl,1.0,JPOH,1.0,JPCl2)
Zindice_4(JP4_Cl_HOCl)=zloc2
!
zloc2=Z4SPEC(1.0,JPO1D,1.0,JPCFC11,3.0,JPCl,0.0,JPDUMMY)
Zindice_4(JP4_O1D_CFC11)=zloc2
!
zloc2=Z4SPEC(1.0,JPO1D,1.0,JPCFC12,1.0,JPCl,1.0,JPCl)
Zindice_4(JP4_O1D_CFC12)=zloc2
!
zloc2=Z4SPEC(1.0,JPO1D,1.0,JPCFC113,3.0,JPCl,0.0,JPDUMMY)
Zindice_4(JP4_O1D_CFC113)=zloc2
!
zloc2=Z4SPEC(1.0,JPO1D,1.0,JPCCl4,4.0,JPCl,0.0,JPDUMMY)
Zindice_4(JP4_O1D_CCl4)=zloc2
!
zloc2=Z4SPEC(1.0,JPOH,1.0,JPCH3CCl3,3.0,JPCl,1.0,JPH2O)
Zindice_4(JP4_OH_CH3CCl3)=zloc2
!
zloc2=Z4SPEC(1.0,JPOH,1.0,JPCH3Cl,1.0,JPCl,1.0,JPH2O)
Zindice_4(JP4_OH_CH3Cl)=zloc2
!
zloc2=Z4SPEC(1.0,JPO1D,1.0,JPHCFC22,1.0,JPCl,0.0,JPDUMMY)
Zindice_4(JP4_O1D_HCFC22)=zloc2
!
zloc2=Z4SPEC(1.0,JPOH,1.0,JPHCFC22,1.0,JPCl,1.0,JPH2O)
Zindice_4(JP4_OH_HCFC22)=zloc2
!
zloc2=Z4SPEC(1.0,JPBr,1.0,JPO3,1.0,JPBrO,1.0,JPO2)
Zindice_4(JP4_Br_O3)=zloc2
!
zloc2=Z4SPEC(1.0,JPBrO,1.0,JPO3P,1.0,JPBr,1.0,JPO2)
Zindice_4(JP4_BrO_O3P)=zloc2
!
zloc2=Z4SPEC(1.0,JPBrO,1.0,JPNO,1.0,JPNO2,1.0,JPBr)
Zindice_4(JP4_BrO_NO)=zloc2
!
zloc2=Z4SPEC(1.0,JPBrO,1.0,JPClO,1.0,JPOClO,1.0,JPBr)
Zindice_4(JP4_BrO_ClOa)=zloc2
!
zloc2=Z4SPEC(1.0,JPBrO,1.0,JPClO,1.0,JPBr,1.0,JPCl)
Zindice_4(JP4_BrO_ClOb)=zloc2
!
zloc2=Z4SPEC(1.0,JPBrO,1.0,JPClO,1.0,JPBrCl,1.0,JPO2)
Zindice_4(JP4_BrO_ClOc)=zloc2
!
zloc2=Z4SPEC(1.0,JPBr,1.0,JPHO2,1.0,JPHBr,1.0,JPO2)
Zindice_4(JP4_Br_HO2)=zloc2
!
zloc2=Z4SPEC(1.0,JPO1D,1.0,JPHBr,1.0,JPOH,1.0,JPBr)
Zindice_4(JP4_O1D_HBr)=zloc2
!
zloc2=Z4SPEC(1.0,JPBrO,1.0,JPOH,1.0,JPBr,1.0,JPHO2)
Zindice_4(JP4_BrO_OH)=zloc2
!
zloc2=Z4SPEC(1.0,JPOH,1.0,JPHBr,1.0,JPH2O,1.0,JPBr)
Zindice_4(JP4_OH_HBr)=zloc2
!
zloc2=Z4SPEC(1.0,JPBrO,1.0,JPNO2,1.0,JPBrONO2,0.0,JPDUMMY)
Zindice_4(JP4_BrO_NO2)=zloc2
!
zloc2=Z4SPEC(1.0,JPBrO,1.0,JPHO2,1.0,JPHOBr,0.0,JPDUMMY)
Zindice_4(JP4_BrO_HO2)=zloc2
!
zloc2=Z4SPEC(1.0,JPHOBr,1.0,JPO3P,1.0,JPOH,1.0,JPBrO)
Zindice_4(JP4_HOBr_O3P)=zloc2
!
zloc2=Z4SPEC(1.0,JPO1D,1.0,JPCH3Br,1.0,JPBr,0.0,JPDUMMY)
Zindice_4(JP4_O1D_CH3Br)=zloc2
!
zloc2=Z4SPEC(1.0,JPOH,1.0,JPCH3Br,1.0,JPBr,1.0,JPH2O)
Zindice_4(JP4_OH_CH3Br)=zloc2
!
zloc2=Z4SPEC(1.0,JPO1D,1.0,JPH1211,1.0,JPBr,0.0,JPDUMMY)
Zindice_4(JP4_O1D_H1211)=zloc2
!
zloc2=Z4SPEC(1.0,JPO1D,1.0,JPH1301,1.0,JPBr,0.0,JPDUMMY)
Zindice_4(JP4_O1D_H1301)=zloc2

#ifdef HETERO
!
zloc2=Z4SPEC(1.0,JPClONO2,1.0,JPH2O,1.0,JPHOCl,1.0,JPHNO3)
Zindice_4(JP4_ClONO2_H2Os)=zloc2
!
zloc2=Z4SPEC(1.0,JPClONO2,1.0,JPHCl,1.0,JPCl2,1.0,JPHNO3)
Zindice_4(JP4_ClONO2_HCls)=zloc2
!
zloc2=Z4SPEC(1.0,JPN2O5,1.0,JPH2O,1.0,JPHNO3,1.0,JPHNO3)
Zindice_4(JP4_N2O5_H2Os)=zloc2
!
zloc2=Z4SPEC(1.0,JPN2O5,1.0,JPHCl,1.0,JPClNO2,1.0,JPHNO3)
Zindice_4(JP4_N2O5_HCls)=zloc2
!
zloc2=Z4SPEC(1.0,JPHOCl,1.0,JPHCl,1.0,JPCl2,1.0,JPH2O)
Zindice_4(JP4_HOCl_HCls)=zloc2
!
zloc2=Z4SPEC(1.0,JPBrONO2,1.0,JPH2O,1.0,JPHOBr,1.0,JPHNO3)
Zindice_4(JP4_BrONO2_H2Os)=zloc2
!
zloc2=Z4SPEC(1.0,JPHOBr,1.0,JPHCl,1.0,JPBrCl,1.0,JPH2O)
Zindice_4(JP4_HOBr_HCls)=zloc2
!    HOBr + HBr  -->  BrOx + BrOx + H2O  split in HOBr + HBr --> BrOx + BrOx and 0*HOBr + 0*HOBr --> H2O
zloc2=Z4SPEC(1.0,JPHOBr,1.0,JPHBr,1.0,JPBr,1.0,JPBr)
Zindice_4(JP4_HOBr_HBrs_i)=zloc2
zloc2=Z4SPEC(0.0,JPHOBr,0.0,JPHBr,1.0,JPH2O,0.0,JPDUMMY)
Zindice_4(JP4_HOBr_HBrs_ii)=zloc2
!
zloc2=Z4SPEC(1.0,JPHOCl,1.0,JPHBr,1.0,JPBrCl,1.0,JPH2O)
Zindice_4(JP4_HOCl_HBrs)=zloc2
#endif

!zloc2=Z4SPEC(1.0,JPCO,1.0,JPOH,1.0,JPCO2,1.0,JPHO2)
!Zindice_4(JP4_CO_OHb)=zloc2
!
zloc2=Z4SPEC(1.0,JPOH,1.0,JPNO,1.0,JPHONO,0.0,JPDUMMY)
Zindice_4(JP4_OH_NO)=zloc2
!
zloc2=Z4SPEC(1.0,JPOH,1.0,JPHONO,1.0,JPH2O,1.0,JPNO2)
Zindice_4(JP4_OH_HONO)=zloc2
!
zloc2=Z4SPEC(1.0,JPOH,1.0,JPSO2,1.0,JPSULF,1.0,JPHO2)
Zindice_4(JP4_OH_SO2)=zloc2
!
zloc2=Z4SPEC(1.0,JPISO,1.0,JPO3P,0.86,JPOLT,0.05,JPHCHO)
Zindice_4(JP4_ISO_O3P_i)=zloc2
zloc2=Z4SPEC(0.0,JPISO,0.0,JPO3P,0.02,JPOH,0.01,JPCO)
Zindice_4(JP4_ISO_O3P_ii)=zloc2
zloc2=Z4SPEC(0.0,JPISO,0.0,JPO3P,0.13,JPDCB,0.28,JPHO2)
Zindice_4(JP4_ISO_O3P_iii)=zloc2
zloc2=Z4SPEC(0.0,JPISO,0.0,JPO3P,0.15,JPXO2,0.0,JPDUMMY)
Zindice_4(JP4_ISO_O3P_iv)=zloc2
!
zloc2=Z4SPEC(1.0,JPMACR,1.0,JPO3P,1.0,JPALD,0.0,JPDUMMY)
Zindice_4(JP4_MACR_O3P)=zloc2
!
zloc2=Z4SPEC(1.0,JPETH,1.0,JPOH,1.0,JPETHP,1.0,JPH2O)
Zindice_4(JP4_ETH_OH)=zloc2
!
zloc2=Z4SPEC(1.0,JPHC3,1.0,JPOH,0.583,JPHC3P,0.381,JPHO2)
Zindice_4(JP4_HC3_OH_i)=zloc2
zloc2=Z4SPEC(0.0,JPHC3,0.0,JPOH,0.335,JPALD,0.036,JPCO)
Zindice_4(JP4_HC3_OH_ii)=zloc2
zloc2=Z4SPEC(0.0,JPHC3,0.0,JPOH,0.036,JPGLY,0.036,JPOH)
Zindice_4(JP4_HC3_OH_iii)=zloc2
zloc2=Z4SPEC(0.0,JPHC3,0.0,JPOH,0.01,JPHCHO,1.0,JPH2O)
Zindice_4(JP4_HC3_OH_iv)=zloc2
!
zloc2=Z4SPEC(1.0,JPHC5,1.0,JPOH,1.0,JPHC5P,1.0,JPH2O)
Zindice_4(JP4_HC5_OHa)=zloc2
!
zloc2=Z4SPEC(1.0,JPHC5,1.0,JPOH,1.0,JPKET,1.0,JPHO2)
Zindice_4(JP4_HC5_OHb)=zloc2
!
zloc2=Z4SPEC(1.0,JPHC8,1.0,JPOH,0.951,JPHC8P,0.025,JPALD)
Zindice_4(JP4_HC8_OH_i)=zloc2
zloc2=Z4SPEC(0.0,JPHC8,0.0,JPOH,0.024,JPHKET,0.049,JPHO2)
Zindice_4(JP4_HC8_OH_ii)=zloc2
!
zloc2=Z4SPEC(1.0,JPETE,1.0,JPOH,1.0,JPETEP,0.0,JPDUMMY)
Zindice_4(JP4_ETE_OH)=zloc2
!
zloc2=Z4SPEC(1.0,JPOLT,1.0,JPOH,1.0,JPOLTP,0.0,JPDUMMY)
Zindice_4(JP4_OLT_OH)=zloc2
!
zloc2=Z4SPEC(1.0,JPOLI,1.0,JPOH,1.0,JPOLIP,0.0,JPDUMMY)
Zindice_4(JP4_OLI_OH)=zloc2
!
zloc2=Z4SPEC(1.0,JPDIEN,1.0,JPOH,1.0,JPISOP,0.0,JPDUMMY)
Zindice_4(JP4_DIEN_OH)=zloc2
!
zloc2=Z4SPEC(1.0,JPISO,1.0,JPOH,1.0,JPISOP,0.0,JPDUMMY)
Zindice_4(JP4_ISO_OH)=zloc2
!
zloc2=Z4SPEC(1.0,JPAPI,1.0,JPOH,1.0,JPAPIP,0.0,JPDUMMY)
Zindice_4(JP4_API_OH)=zloc2
!
zloc2=Z4SPEC(1.0,JPLIM,1.0,JPOH,1.0,JPLIMP,0.0,JPDUMMY)
Zindice_4(JP4_LIM_OH)=zloc2
!
zloc2=Z4SPEC(1.0,JPTOL,1.0,JPOH,1.0,JPADDT,0.0,JPDUMMY)
Zindice_4(JP4_TOL_OHa)=zloc2
!
zloc2=Z4SPEC(1.0,JPTOL,1.0,JPOH,1.0,JPXO2,1.0,JPHO2)
Zindice_4(JP4_TOL_OHb)=zloc2
!
zloc2=Z4SPEC(1.0,JPXYL,1.0,JPOH,1.0,JPADDX,0.0,JPDUMMY)
Zindice_4(JP4_XYL_OHa)=zloc2
!
zloc2=Z4SPEC(1.0,JPXYL,1.0,JPOH,1.0,JPXO2,1.0,JPHO2)
Zindice_4(JP4_XYL_OHb)=zloc2
!
zloc2=Z4SPEC(1.0,JPCSL,1.0,JPOH,1.0,JPADDC,0.0,JPDUMMY)
Zindice_4(JP4_CSL_OHa)=zloc2
!
zloc2=Z4SPEC(1.0,JPCSL,1.0,JPOH,1.0,JPPHO,0.0,JPDUMMY)
Zindice_4(JP4_CSL_OHb)=zloc2
!
zloc2=Z4SPEC(1.0,JPCSL,1.0,JPOH,1.0,JPHO2,1.0,JPXO2)
Zindice_4(JP4_CSL_OHc)=zloc2
!
zloc2=Z4SPEC(1.0,JPALD,1.0,JPOH,1.0,JPACO3,1.0,JPH2O)
Zindice_4(JP4_ALD_OH)=zloc2
!
zloc2=Z4SPEC(1.0,JPKET,1.0,JPOH,1.0,JPKETP,1.0,JPH2O)
Zindice_4(JP4_KET_OH)=zloc2
!
zloc2=Z4SPEC(1.0,JPHKET,1.0,JPOH,1.0,JPHO2,1.0,JPMGLY)
Zindice_4(JP4_HKET_OH)=zloc2
!
zloc2=Z4SPEC(1.0,JPGLY,1.0,JPOH,1.0,JPHO2,2.0,JPCO)
Zindice_4(JP4_GLY_OH)=zloc2
!
zloc2=Z4SPEC(1.0,JPMGLY,1.0,JPOH,1.0,JPACO3,1.0,JPCO)
Zindice_4(JP4_MGLY_OH)=zloc2
!
zloc2=Z4SPEC(1.0,JPMACR,1.0,JPOH,0.51,JPTCO3,0.41,JPHKET)
Zindice_4(JP4_MACR_OH_i)=zloc2
zloc2=Z4SPEC(0.0,JPMACR,0.0,JPOH,0.08,JPMGLY,0.41,JPCO)
Zindice_4(JP4_MACR_OH_ii)=zloc2
zloc2=Z4SPEC(0.0,JPMACR,0.0,JPOH,0.08,JPHCHO,0.49,JPHO2)
Zindice_4(JP4_MACR_OH_iii)=zloc2
zloc2=Z4SPEC(0.0,JPMACR,0.0,JPOH,0.49,JPXO2,0.0,JPDUMMY)
Zindice_4(JP4_MACR_OH_iv)=zloc2
!
zloc2=Z4SPEC(1.0,JPOH,1.0,JPDCB,0.50,JPTCO3,0.50,JPHO2)
Zindice_4(JP4_OH_DCB_i)=zloc2
zloc2=Z4SPEC(0.0,JPOH,0.0,JPDCB,0.50,JPXO2,0.35,JPUDD)
Zindice_4(JP4_OH_DCB_ii)=zloc2
zloc2=Z4SPEC(0.0,JPOH,0.0,JPDCB,0.15,JPGLY,0.15,JPMGLY)
Zindice_4(JP4_OH_DCB_iii)=zloc2
!
zloc2=Z4SPEC(1.0,JPOH,1.0,JPUDD,1.0,JPALD,1.0,JPHO2)
Zindice_4(JP4_OH_UDDa)=zloc2
!
zloc2=Z4SPEC(1.0,JPOH,1.0,JPUDD,1.0,JPKET,1.0,JPHO2)
Zindice_4(JP4_OH_UDDb)=zloc2
!
zloc2=Z4SPEC(1.0,JPOP2,1.0,JPOH,0.44,JPHC3P,0.08,JPALD)
Zindice_4(JP4_OP2_OH_i)=zloc2
zloc2=Z4SPEC(0.0,JPOP2,0.0,JPOH,0.49,JPOH,0.07,JPXO2)
Zindice_4(JP4_OP2_OH_ii)=zloc2
zloc2=Z4SPEC(0.0,JPOP2,0.0,JPOH,0.41,JPKET,0.0,JPDUMMY)
Zindice_4(JP4_OP2_OH_iii)=zloc2
!
zloc2=Z4SPEC(1.0,JPPAA,1.0,JPOH,1.0,JPACO3,0.0,JPDUMMY)
Zindice_4(JP4_PAA_OHa)=zloc2
!
zloc2=Z4SPEC(1.0,JPPAA,1.0,JPOH,1.0,JPHO2,1.0,JPHCHO)
Zindice_4(JP4_PAA_OHb_i)=zloc2
zloc2=Z4SPEC(0.0,JPPAA,0.0,JPOH,1.0,JPXO2,0.0,JPDUMMY)
Zindice_4(JP4_PAA_OHb_ii)=zloc2
!
zloc2=Z4SPEC(1.0,JPPAN,1.0,JPOH,1.0,JPHCHO,1.0,JPNO3)
Zindice_4(JP4_PAN_OH_i)=zloc2
zloc2=Z4SPEC(0.0,JPPAN,0.0,JPOH,1.0,JPXO2,1.0,JPH2O)
Zindice_4(JP4_PAN_OH_ii)=zloc2
!
zloc2=Z4SPEC(1.0,JPTPAN,1.0,JPOH,0.6,JPHKET,0.6,JPNO3)
Zindice_4(JP4_TPAN_OH_i)=zloc2
zloc2=Z4SPEC(0.0,JPTPAN,0.0,JPOH,0.4,JPPAN,0.4,JPHCHO)
Zindice_4(JP4_TPAN_OH_ii)=zloc2
zloc2=Z4SPEC(0.0,JPTPAN,0.0,JPOH,0.4,JPHO2,1.0,JPXO2)
Zindice_4(JP4_TPAN_OH_iii)=zloc2
!
zloc2=Z4SPEC(1.0,JPONIT,1.0,JPOH,1.0,JPHC3P,1.0,JPNO2)
Zindice_4(JP4_ONIT_OH_i)=zloc2
zloc2=Z4SPEC(0.0,JPONIT,0.0,JPOH,1.0,JPH2O,0.0,JPDUMMY)
Zindice_4(JP4_ONIT_OH_ii)=zloc2
!
zloc2=Z4SPEC(1.0,JPALD,1.0,JPNO3,1.0,JPACO3,1.0,JPHNO3)
Zindice_4(JP4_ALD_NO3)=zloc2
!
zloc2=Z4SPEC(1.0,JPGLY,1.0,JPNO3,1.0,JPHNO3,1.0,JPHO2)
Zindice_4(JP4_GLY_NO3_i)=zloc2
zloc2=Z4SPEC(0.0,JPGLY,0.0,JPNO3,2.0,JPCO,0.0,JPDUMMY)
Zindice_4(JP4_GLY_NO3_ii)=zloc2
!
zloc2=Z4SPEC(1.0,JPMGLY,1.0,JPNO3,1.0,JPHNO3,1.0,JPACO3)
Zindice_4(JP4_MGLY_NO3_i)=zloc2
zloc2=Z4SPEC(0.0,JPMGLY,0.0,JPNO3,1.0,JPCO,0.0,JPDUMMY)
Zindice_4(JP4_MGLY_NO3_ii)=zloc2
!
zloc2=Z4SPEC(1.0,JPMACR,1.0,JPNO3,0.2,JPTCO3,0.2,JPHNO3)
Zindice_4(JP4_MACR_NO3_i)=zloc2
zloc2=Z4SPEC(0.0,JPMACR,0.0,JPNO3,0.8,JPOLNN,0.8,JPCO)
Zindice_4(JP4_MACR_NO3_ii)=zloc2
!
zloc2=Z4SPEC(1.0,JPDCB,1.0,JPNO3,0.5,JPTCO3,0.5,JPHO2)
Zindice_4(JP4_DCB_NO3_i)=zloc2
zloc2=Z4SPEC(0.0,JPDCB,0.0,JPNO3,0.5,JPXO2,0.25,JPGLY)
Zindice_4(JP4_DCB_NO3_ii)=zloc2
zloc2=Z4SPEC(0.0,JPDCB,0.0,JPNO3,0.25,JPALD,0.5,JPNO2)
Zindice_4(JP4_DCB_NO3_iii)=zloc2
zloc2=Z4SPEC(0.0,JPDCB,0.0,JPNO3,0.03,JPKET,0.25,JPMGLY)
Zindice_4(JP4_DCB_NO3_iv)=zloc2
zloc2=Z4SPEC(0.0,JPDCB,0.0,JPNO3,0.5,JPHNO3,0.0,JPDUMMY)
Zindice_4(JP4_DCB_NO3_v)=zloc2
!
zloc2=Z4SPEC(1.0,JPCSL,1.0,JPNO3,1.0,JPHNO3,1.0,JPPHO)
Zindice_4(JP4_CSL_NO3)=zloc2
!
zloc2=Z4SPEC(1.0,JPETE,1.0,JPNO3,1.0,JPOLNN,0.0,JPDUMMY)
Zindice_4(JP4_ETE_NO3a)=zloc2
!
zloc2=Z4SPEC(1.0,JPETE,1.0,JPNO3,1.0,JPOLND,0.0,JPDUMMY)
Zindice_4(JP4_ETE_NO3b)=zloc2
!
zloc2=Z4SPEC(1.0,JPOLT,1.0,JPNO3,1.0,JPOLNN,0.0,JPDUMMY)
Zindice_4(JP4_OLT_NO3a)=zloc2
!
zloc2=Z4SPEC(1.0,JPOLT,1.0,JPNO3,1.0,JPOLND,0.0,JPDUMMY)
Zindice_4(JP4_OLT_NO3b)=zloc2
!
zloc2=Z4SPEC(1.0,JPOLI,1.0,JPNO3,1.0,JPOLNN,0.0,JPDUMMY)
Zindice_4(JP4_OLI_NO3a)=zloc2
!
zloc2=Z4SPEC(1.0,JPOLI,1.0,JPNO3,1.0,JPOLND,0.0,JPDUMMY)
Zindice_4(JP4_OLI_NO3b)=zloc2
!
zloc2=Z4SPEC(1.0,JPDIEN,1.0,JPNO3,1.0,JPOLNN,1.0,JPMACR)
Zindice_4(JP4_DIEN_NO3a)=zloc2
!
zloc2=Z4SPEC(1.0,JPDIEN,1.0,JPNO3,1.0,JPOLND,0.0,JPDUMMY)
Zindice_4(JP4_DIEN_NO3b)=zloc2
!
zloc2=Z4SPEC(1.0,JPISO,1.0,JPNO3,1.0,JPOLNN,1.0,JPMACR)
Zindice_4(JP4_ISO_NO3a)=zloc2
!
zloc2=Z4SPEC(1.0,JPISO,1.0,JPNO3,1.0,JPOLND,0.0,JPDUMMY)
Zindice_4(JP4_ISO_NO3b)=zloc2
!
zloc2=Z4SPEC(1.0,JPAPI,1.0,JPNO3,1.0,JPOLNN,0.0,JPDUMMY)
Zindice_4(JP4_API_NO3a)=zloc2
!
zloc2=Z4SPEC(1.0,JPAPI,1.0,JPNO3,1.0,JPOLND,0.0,JPDUMMY)
Zindice_4(JP4_API_NO3b)=zloc2
!
zloc2=Z4SPEC(1.0,JPLIM,1.0,JPNO3,1.0,JPOLNN,0.0,JPDUMMY)
Zindice_4(JP4_LIM_NO3a)=zloc2
!
zloc2=Z4SPEC(1.0,JPLIM,1.0,JPNO3,1.0,JPOLND,0.0,JPDUMMY)
Zindice_4(JP4_LIM_NO3b)=zloc2
!
zloc2=Z4SPEC(1.0,JPTPAN,1.0,JPNO3,1.0,JPONIT,1.0,JPNO3)
Zindice_4(JP4_TPAN_NO3a_i)=zloc2
zloc2=Z4SPEC(0.0,JPTPAN,0.0,JPNO3,1.0,JPXO2,0.0,JPDUMMY)
Zindice_4(JP4_TPAN_NO3a_ii)=zloc2
!
zloc2=Z4SPEC(1.0,JPTPAN,1.0,JPNO3,1.0,JPPAN,1.0,JPHCHO)
Zindice_4(JP4_TPAN_NO3b_i)=zloc2
zloc2=Z4SPEC(0.0,JPTPAN,0.0,JPNO3,1.0,JPNO2,1.0,JPXO2)
Zindice_4(JP4_TPAN_NO3b_ii)=zloc2
!
zloc2=Z4SPEC(1.0,JPETE,1.0,JPO3,1.0,JPHCHO,0.43,JPCO)
Zindice_4(JP4_ETE_O3_i)=zloc2
zloc2=Z4SPEC(0.0,JPETE,0.0,JPO3,0.0,JPDUMMY,0.26,JPHO2)
Zindice_4(JP4_ETE_O3_ii)=zloc2
zloc2=Z4SPEC(0.0,JPETE,0.0,JPO3,0.13,JPDUMMY,0.12,JPOH)
Zindice_4(JP4_ETE_O3_iii)=zloc2
!
zloc2=Z4SPEC(1.0,JPOLT,1.0,JPO3,0.64,JPHCHO,0.44,JPALD)
Zindice_4(JP4_OLT_O3_i)=zloc2
zloc2=Z4SPEC(0.0,JPOLT,0.0,JPO3,0.37,JPCO,0.0,JPDUMMY)
Zindice_4(JP4_OLT_O3_ii)=zloc2
zloc2=Z4SPEC(0.0,JPOLT,0.0,JPO3,0.0,JPDUMMY,0.25,JPHO2)
Zindice_4(JP4_OLT_O3_iii)=zloc2
zloc2=Z4SPEC(0.0,JPOLT,0.0,JPO3,0.4,JPOH,0.03,JPKET)
Zindice_4(JP4_OLT_O3_iv)=zloc2
zloc2=Z4SPEC(0.0,JPOLT,0.0,JPO3,0.03,JPKETP,0.06,JPCH4)
Zindice_4(JP4_OLT_O3_v)=zloc2
zloc2=Z4SPEC(0.0,JPOLT,0.0,JPO3,0.1,JPETHP,0.03,JPETH)
Zindice_4(JP4_OLT_O3_vi)=zloc2
zloc2=Z4SPEC(0.0,JPOLT,0.0,JPO3,0.006,JPH2O2,0.19,JPMO2)
Zindice_4(JP4_OLT_O3_vii)=zloc2
!
zloc2=Z4SPEC(1.0,JPOLI,1.0,JPO3,0.02,JPHCHO,0.99,JPALD)
Zindice_4(JP4_OLI_O3_i)=zloc2
zloc2=Z4SPEC(0.0,JPOLI,0.0,JPO3,0.16,JPKET,0.3,JPCO)
Zindice_4(JP4_OLI_O3_ii)=zloc2
zloc2=Z4SPEC(0.0,JPOLI,0.0,JPO3,0.011,JPH2O2,0.07,JPCH4)
Zindice_4(JP4_OLI_O3_iii)=zloc2
zloc2=Z4SPEC(0.0,JPOLI,0.0,JPO3,0.22,JPHO2,0.63,JPOH)
Zindice_4(JP4_OLI_O3_iv)=zloc2
zloc2=Z4SPEC(0.0,JPOLI,0.0,JPO3,0.23,JPMO2,0.12,JPKETP)
Zindice_4(JP4_OLI_O3_v)=zloc2
zloc2=Z4SPEC(0.0,JPOLI,0.0,JPO3,0.06,JPETH,0.18,JPETHP)
Zindice_4(JP4_OLI_O3_vi)=zloc2
!
zloc2=Z4SPEC(1.0,JPDIEN,1.0,JPO3,0.9,JPHCHO,0.39,JPMACR)
Zindice_4(JP4_DIEN_O3_i)=zloc2
zloc2=Z4SPEC(0.0,JPDIEN,0.0,JPO3,0.36,JPCO,0.09,JPO3P)
Zindice_4(JP4_DIEN_O3_ii)=zloc2
zloc2=Z4SPEC(0.0,JPDIEN,0.0,JPO3,0.3,JPHO2,0.35,JPOLT)
Zindice_4(JP4_DIEN_O3_iii)=zloc2
zloc2=Z4SPEC(0.0,JPDIEN,0.0,JPO3,0.28,JPOH,0.15,JPACO3)
Zindice_4(JP4_DIEN_O3_iv)=zloc2
zloc2=Z4SPEC(0.0,JPDIEN,0.0,JPO3,0.03,JPMO2,0.02,JPKETP)
Zindice_4(JP4_DIEN_O3_v)=zloc2
zloc2=Z4SPEC(0.0,JPDIEN,0.0,JPO3,0.13,JPXO2,0.001,JPH2O2)
Zindice_4(JP4_DIEN_O3_vi)=zloc2
!
zloc2=Z4SPEC(1.0,JPISO,1.0,JPO3,0.9,JPHCHO,0.39,JPMACR)
Zindice_4(JP4_ISO_O3_i)=zloc2
zloc2=Z4SPEC(0.0,JPISO,0.0,JPO3,0.36,JPCO,0.001,JPH2O2)
Zindice_4(JP4_ISO_O3_ii)=zloc2
zloc2=Z4SPEC(0.0,JPISO,0.0,JPO3,0.09,JPO3P,0.3,JPHO2)
Zindice_4(JP4_ISO_O3_iii)=zloc2
zloc2=Z4SPEC(0.0,JPISO,0.0,JPO3,0.35,JPOLT,0.28,JPOH)
Zindice_4(JP4_ISO_O3_iv)=zloc2
zloc2=Z4SPEC(0.0,JPISO,0.0,JPO3,0.15,JPACO3,0.03,JPMO2)
Zindice_4(JP4_ISO_O3_v)=zloc2
zloc2=Z4SPEC(0.0,JPISO,0.0,JPO3,0.02,JPKETP,0.13,JPXO2)
Zindice_4(JP4_ISO_O3_vi)=zloc2
!
zloc2=Z4SPEC(1.0,JPAPI,1.0,JPO3,0.65,JPALD,0.53,JPKET)
Zindice_4(JP4_API_O3_i)=zloc2
zloc2=Z4SPEC(0.0,JPAPI,0.0,JPO3,0.14,JPCO,0.2,JPETHP)
Zindice_4(JP4_API_O3_ii)=zloc2
zloc2=Z4SPEC(0.0,JPAPI,0.0,JPO3,0.42,JPKETP,0.85,JPOH)
Zindice_4(JP4_API_O3_iii)=zloc2
zloc2=Z4SPEC(0.0,JPAPI,0.0,JPO3,0.1,JPHO2,0.02,JPH2O2)
Zindice_4(JP4_API_O3_iv)=zloc2
!
zloc2=Z4SPEC(1.0,JPLIM,1.0,JPO3,0.04,JPHCHO,0.46,JPOLT)
Zindice_4(JP4_LIM_O3_i)=zloc2
zloc2=Z4SPEC(0.0,JPLIM,0.0,JPO3,0.14,JPCO,0.16,JPETHP)
Zindice_4(JP4_LIM_O3_ii)=zloc2
zloc2=Z4SPEC(0.0,JPLIM,0.0,JPO3,0.42,JPKETP,0.85,JPOH)
Zindice_4(JP4_LIM_O3_iii)=zloc2
zloc2=Z4SPEC(0.0,JPLIM,0.0,JPO3,0.1,JPHO2,0.02,JPH2O2)
Zindice_4(JP4_LIM_O3_iv)=zloc2
zloc2=Z4SPEC(0.0,JPLIM,0.0,JPO3,0.79,JPMACR,0.0,JPDUMMY)
Zindice_4(JP4_LIM_O3_v)=zloc2
!
zloc2=Z4SPEC(1.0,JPMACR,1.0,JPO3,0.4,JPHCHO,0.6,JPMGLY)
Zindice_4(JP4_MACR_O3_i)=zloc2
zloc2=Z4SPEC(0.0,JPMACR,0.0,JPO3,0.0,JPDUMMY,0.54,JPCO)
Zindice_4(JP4_MACR_O3_ii)=zloc2
zloc2=Z4SPEC(0.0,JPMACR,0.0,JPO3,0.0,JPDUMMY,0.29,JPHO2)
Zindice_4(JP4_MACR_O3_iii)=zloc2
zloc2=Z4SPEC(0.0,JPMACR,0.0,JPO3,0.07,JPOH,0.13,JPOP2)
Zindice_4(JP4_MACR_O3_iv)=zloc2
zloc2=Z4SPEC(0.0,JPMACR,0.0,JPO3,0.13,JPACO3,0.0,JPDUMMY)
Zindice_4(JP4_MACR_O3_v)=zloc2
!
zloc2=Z4SPEC(1.0,JPDCB,1.0,JPO3,0.21,JPOH,0.29,JPHO2)
Zindice_4(JP4_DCB_O3_i)=zloc2
zloc2=Z4SPEC(0.0,JPDCB,0.0,JPO3,0.66,JPCO,00.50,JPGLY)
Zindice_4(JP4_DCB_O3_ii)=zloc2
zloc2=Z4SPEC(0.0,JPDCB,0.0,JPO3,0.62,JPMGLY,0.28,JPACO3)
Zindice_4(JP4_DCB_O3_iii)=zloc2
zloc2=Z4SPEC(0.0,JPDCB,0.0,JPO3,0.16,JPALD,0.11,JPPAA)
Zindice_4(JP4_DCB_O3_iv)=zloc2
!
zloc2=Z4SPEC(1.0,JPTPAN,1.0,JPO3,0.7,JPHCHO,0.3,JPPAN)
Zindice_4(JP4_TPAN_O3_i)=zloc2
zloc2=Z4SPEC(0.0,JPTPAN,0.0,JPO3,0.7,JPNO2,0.13,JPCO)
Zindice_4(JP4_TPAN_O3_ii)=zloc2
zloc2=Z4SPEC(0.0,JPTPAN,0.0,JPO3,0.08,JPHO2,0.036,JPOH)
Zindice_4(JP4_TPAN_O3_iii)=zloc2
zloc2=Z4SPEC(0.0,JPTPAN,0.0,JPO3,0.7,JPACO3,0.0,JPDUMMY)
Zindice_4(JP4_TPAN_O3_iv)=zloc2
!
zloc2=Z4SPEC(1.0,JPPHO,1.0,JPNO2,0.1,JPCSL,1.0,JPONIT)
Zindice_4(JP4_PHO_NO2)=zloc2
!
zloc2=Z4SPEC(1.0,JPPHO,1.0,JPHO2,1.0,JPCSL,0.0,JPDUMMY)
Zindice_4(JP4_PHO_HO2)=zloc2
!
zloc2=Z4SPEC(1.0,JPADDT,1.0,JPNO2,1.0,JPCSL,1.0,JPHONO)
Zindice_4(JP4_ADDT_NO2)=zloc2
!
zloc2=Z4SPEC(1.0,JPADDT,1.0,JPO2,1.0,JPTOLP,0.0,JPDUMMY)
Zindice_4(JP4_ADDT_O2a)=zloc2
!
zloc2=Z4SPEC(1.0,JPADDT,1.0,JPO2,1.0,JPCSL,1.0,JPHO2)
Zindice_4(JP4_ADDT_O2b)=zloc2
!
zloc2=Z4SPEC(1.0,JPADDT,1.0,JPO3,1.0,JPCSL,1.0,JPOH)
Zindice_4(JP4_ADDT_O3)=zloc2
!
zloc2=Z4SPEC(1.0,JPADDX,1.0,JPNO2,1.0,JPCSL,1.0,JPHONO)
Zindice_4(JP4_ADDX_NO2)=zloc2
!
zloc2=Z4SPEC(1.0,JPADDX,1.0,JPO2,1.0,JPXYLP,0.0,JPDUMMY)
Zindice_4(JP4_ADDX_O2a)=zloc2
!
zloc2=Z4SPEC(1.0,JPADDX,1.0,JPO2,1.0,JPCSL,1.0,JPHO2)
Zindice_4(JP4_ADDX_O2b)=zloc2
!
zloc2=Z4SPEC(1.0,JPADDX,1.0,JPO3,1.0,JPCSL,1.0,JPOH)
Zindice_4(JP4_ADDX_O3)=zloc2
!
zloc2=Z4SPEC(1.0,JPADDC,1.0,JPNO2,1.0,JPCSL,1.0,JPHONO)
Zindice_4(JP4_ADDC_NO2)=zloc2
!
zloc2=Z4SPEC(1.0,JPADDC,1.0,JPO2,1.0,JPCSLP,0.0,JPDUMMY)
Zindice_4(JP4_ADDC_O2a)=zloc2
!
zloc2=Z4SPEC(1.0,JPADDC,1.0,JPO2,1.0,JPCSL,1.0,JPHO2)
Zindice_4(JP4_ADDC_O2b)=zloc2
!
zloc2=Z4SPEC(1.0,JPADDC,1.0,JPO3,1.0,JPCSL,1.0,JPOH)
Zindice_4(JP4_ADDC_O3)=zloc2
!
zloc2=Z4SPEC(1.0,JPACO3,1.0,JPNO2,1.0,JPPAN,0.0,JPDUMMY)
Zindice_4(JP4_ACO3_NO2)=zloc2
!
zloc2=Z4SPEC(1.0,JPTCO3,1.0,JPNO2,1.0,JPTPAN,0.0,JPDUMMY)
Zindice_4(JP4_TCO3_NO2)=zloc2
!
zloc2=Z4SPEC(1.0,JPHC8P,1.0,JPNO,0.150,JPALD,0.642,JPKET)
Zindice_4(JP4_HC8P_NO_i)=zloc2
zloc2=Z4SPEC(0.0,JPHC8P,0.0,JPNO,0.133,JPETHP,0.261,JPONIT)
Zindice_4(JP4_HC8P_NO_ii)=zloc2
zloc2=Z4SPEC(0.0,JPHC8P,0.0,JPNO,0.739,JPNO2,0.606,JPHO2)
Zindice_4(JP4_HC8P_NO_iii)=zloc2
zloc2=Z4SPEC(0.0,JPHC8P,0.0,JPNO,0.416,JPXO2,0.0,JPDUMMY)
Zindice_4(JP4_HC8P_NO_iv)=zloc2
!
zloc2=Z4SPEC(1.0,JPETEP,1.0,JPNO,1.6,JPHCHO,1.0,JPHO2)
Zindice_4(JP4_ETEP_NO_i)=zloc2
zloc2=Z4SPEC(0.0,JPETEP,0.0,JPNO,1.0,JPNO2,0.2,JPALD)
Zindice_4(JP4_ETEP_NO_ii)=zloc2
!
zloc2=Z4SPEC(1.0,JPOLTP,1.0,JPNO,0.94,JPALD,1.0,JPHCHO)
Zindice_4(JP4_OLTP_NO_i)=zloc2
zloc2=Z4SPEC(0.0,JPOLTP,0.0,JPNO,1.0,JPHO2,1.0,JPNO2)
Zindice_4(JP4_OLTP_NO_ii)=zloc2
zloc2=Z4SPEC(0.0,JPOLTP,0.0,JPNO,0.06,JPKET,0.0,JPDUMMY)
Zindice_4(JP4_OLTP_NO_iii)=zloc2
!
zloc2=Z4SPEC(1.0,JPOLIP,1.0,JPNO,1.0,JPHO2,1.71,JPALD)
Zindice_4(JP4_OLIP_NO_i)=zloc2
zloc2=Z4SPEC(0.0,JPOLIP,0.0,JPNO,0.29,JPKET,1.0,JPNO2)
Zindice_4(JP4_OLIP_NO_ii)=zloc2
!
zloc2=Z4SPEC(1.0,JPISOP,1.0,JPNO,0.446,JPMACR,0.354,JPOLT)
Zindice_4(JP4_ISOP_NO_i)=zloc2
zloc2=Z4SPEC(0.0,JPISOP,0.0,JPNO,0.847,JPHO2,0.847,JPNO2)
Zindice_4(JP4_ISOP_NO_ii)=zloc2
zloc2=Z4SPEC(0.0,JPISOP,0.0,JPNO,0.153,JPONIT,0.606,JPHCHO)
Zindice_4(JP4_ISOP_NO_iii)=zloc2
!
zloc2=Z4SPEC(1.0,JPAPIP,1.0,JPNO,0.8,JPHO2,0.8,JPALD)
Zindice_4(JP4_APIP_NO_i)=zloc2
zloc2=Z4SPEC(0.0,JPAPIP,0.0,JPNO,0.8,JPNO2,0.8,JPKET)
Zindice_4(JP4_APIP_NO_ii)=zloc2
zloc2=Z4SPEC(0.0,JPAPIP,0.0,JPNO,0.2,JPONIT,0.0,JPDUMMY)
Zindice_4(JP4_APIP_NO_iii)=zloc2
!
zloc2=Z4SPEC(1.0,JPLIMP,1.0,JPNO,0.65,JPHO2,0.4,JPMACR)
Zindice_4(JP4_LIMP_NO_i)=zloc2
zloc2=Z4SPEC(0.0,JPLIMP,0.0,JPNO,0.25,JPOLI,0.25,JPHCHO)
Zindice_4(JP4_LIMP_NO_ii)=zloc2
zloc2=Z4SPEC(0.0,JPLIMP,0.0,JPNO,0.65,JPNO2,0.35,JPONIT)
Zindice_4(JP4_LIMP_NO_iii)=zloc2
!
zloc2=Z4SPEC(1.0,JPTOLP,1.0,JPNO,0.95,JPNO2,0.95,JPHO2)
Zindice_4(JP4_TOLP_NO_i)=zloc2
zloc2=Z4SPEC(0.0,JPTOLP,0.0,JPNO,1.2,JPGLY,0.5,JPDCB)
Zindice_4(JP4_TOLP_NO_ii)=zloc2
zloc2=Z4SPEC(0.0,JPTOLP,0.0,JPNO,0.05,JPONIT,0.65,JPMGLY)
Zindice_4(JP4_TOLP_NO_iii)=zloc2
!
zloc2=Z4SPEC(1.0,JPXYLP,1.0,JPNO,0.95,JPNO2,0.95,JPHO2)
Zindice_4(JP4_XYLP_NO_i)=zloc2
zloc2=Z4SPEC(0.0,JPXYLP,0.0,JPNO,0.35,JPGLY,0.95,JPDCB)
Zindice_4(JP4_XYLP_NO_ii)=zloc2
zloc2=Z4SPEC(0.0,JPXYLP,0.0,JPNO,0.05,JPONIT,0.60,JPMGLY)
Zindice_4(JP4_XYLP_NO_iii)=zloc2
!
zloc2=Z4SPEC(1.0,JPCSLP,1.0,JPNO,1.0,JPGLY,1.0,JPMGLY)
Zindice_4(JP4_CSLP_NO_i)=zloc2
zloc2=Z4SPEC(0.0,JPCSLP,0.0,JPNO,1.0,JPNO2,1.0,JPHO2)
Zindice_4(JP4_CSLP_NO_ii)=zloc2
!
zloc2=Z4SPEC(1.0,JPACO3,1.0,JPNO,1.0,JPMO2,1.0,JPNO2)
Zindice_4(JP4_ACO3_NO)=zloc2
!
zloc2=Z4SPEC(1.0,JPTCO3,1.0,JPNO,1.0,JPNO2,1.0,JPACO3)
Zindice_4(JP4_TCO3_NO_i)=zloc2
zloc2=Z4SPEC(0.0,JPTCO3,0.0,JPNO,1.0,JPHCHO,0.0,JPDUMMY)
Zindice_4(JP4_TCO3_NO_ii)=zloc2
!
zloc2=Z4SPEC(1.0,JPKETP,1.0,JPNO,0.54,JPMGLY,0.46,JPALD)
Zindice_4(JP4_KETP_NO_i)=zloc2
zloc2=Z4SPEC(0.0,JPKETP,0.0,JPNO,0.23,JPACO3,0.77,JPHO2)
Zindice_4(JP4_KETP_NO_ii)=zloc2
zloc2=Z4SPEC(0.0,JPKETP,0.0,JPNO,0.16,JPXO2,1.0,JPNO2)
Zindice_4(JP4_KETP_NO_iii)=zloc2
!
zloc2=Z4SPEC(1.0,JPOLNN,1.0,JPNO,1.0,JPONIT,1.0,JPNO2)
Zindice_4(JP4_OLNN_NO_i)=zloc2
zloc2=Z4SPEC(0.0,JPOLNN,0.0,JPNO,1.0,JPHO2,0.0,JPDUMMY)
Zindice_4(JP4_OLNN_NO_ii)=zloc2
!
zloc2=Z4SPEC(1.0,JPOLND,1.0,JPNO,0.287,JPHCHO,1.24,JPALD)
Zindice_4(JP4_OLND_NO_i)=zloc2
zloc2=Z4SPEC(0.0,JPOLND,0.0,JPNO,0.464,JPKET,2.0,JPNO2)
Zindice_4(JP4_OLND_NO_ii)=zloc2
!
zloc2=Z4SPEC(1.0,JPHC8P,1.0,JPHO2,1.0,JPOP2,0.0,JPDUMMY)
Zindice_4(JP4_HC8P_HO2)=zloc2
!
zloc2=Z4SPEC(1.0,JPOLIP,1.0,JPHO2,1.0,JPOP2,0.0,JPDUMMY)
Zindice_4(JP4_OLIP_HO2)=zloc2
!
zloc2=Z4SPEC(1.0,JPISOP,1.0,JPHO2,1.0,JPOP2,0.0,JPDUMMY)
Zindice_4(JP4_ISOP_HO2)=zloc2
!
zloc2=Z4SPEC(1.0,JPAPIP,1.0,JPHO2,1.0,JPOP2,0.0,JPDUMMY)
Zindice_4(JP4_APIP_HO2)=zloc2
!
zloc2=Z4SPEC(1.0,JPLIMP,1.0,JPHO2,1.0,JPOP2,0.0,JPDUMMY)
Zindice_4(JP4_LIMP_HO2)=zloc2
!
zloc2=Z4SPEC(1.0,JPTOLP,1.0,JPHO2,1.0,JPOP2,0.0,JPDUMMY)
Zindice_4(JP4_TOLP_HO2)=zloc2
!
zloc2=Z4SPEC(1.0,JPXYLP,1.0,JPHO2,1.0,JPOP2,0.0,JPDUMMY)
Zindice_4(JP4_XYLP_HO2)=zloc2
!
zloc2=Z4SPEC(1.0,JPCSLP,1.0,JPHO2,1.0,JPOP2,0.0,JPDUMMY)
Zindice_4(JP4_CSLP_HO2)=zloc2
!
zloc2=Z4SPEC(1.0,JPACO3,1.0,JPHO2,1.0,JPPAA,0.0,JPDUMMY)
Zindice_4(JP4_ACO3_HO2a)=zloc2
!
zloc2=Z4SPEC(1.0,JPACO3,1.0,JPHO2,1.0,JPO3,1.0,JPDUMMY)
Zindice_4(JP4_ACO3_HO2b)=zloc2
!
zloc2=Z4SPEC(1.0,JPTCO3,1.0,JPHO2,1.0,JPOP2,0.0,JPDUMMY)
Zindice_4(JP4_TCO3_HO2a)=zloc2
!
zloc2=Z4SPEC(1.0,JPTCO3,1.0,JPHO2,1.0,JPO3,0.0,JPDUMMY)
Zindice_4(JP4_TCO3_HO2b)=zloc2
!
zloc2=Z4SPEC(1.0,JPKETP,1.0,JPHO2,1.0,JPOP2,0.0,JPDUMMY)
Zindice_4(JP4_KETP_HO2)=zloc2
!
zloc2=Z4SPEC(1.0,JPOLNN,1.0,JPHO2,1.0,JPONIT,0.0,JPDUMMY)
Zindice_4(JP4_OLNN_HO2)=zloc2
!
zloc2=Z4SPEC(1.0,JPOLND,1.0,JPHO2,1.0,JPONIT,0.0,JPDUMMY)
Zindice_4(JP4_OLND_HO2)=zloc2
!
zloc2=Z4SPEC(1.0,JPETHP,1.0,JPMO2,0.75,JPHCHO,1.0,JPHO2)
Zindice_4(JP4_ETHP_MO2_i)=zloc2
zloc2=Z4SPEC(0.0,JPETHP,0.0,JPMO2,0.75,JPALD,0.0,JPDUMMY)
Zindice_4(JP4_ETHP_MO2_ii)=zloc2
!
zloc2=Z4SPEC(1.0,JPHC3P,0.993,JPMO2,0.81,JPHCHO,0.992,JPHO2)
Zindice_4(JP4_HC3P_MO2_i)=zloc2
zloc2=Z4SPEC(0.0,JPHC3P,0.0,JPMO2,0.58,JPALD,0.018,JPKET)
Zindice_4(JP4_HC3P_MO2_ii)=zloc2
zloc2=Z4SPEC(0.0,JPHC3P,0.0,JPMO2,0.00,JPMO2,0.005,JPMGLY)
Zindice_4(JP4_HC3P_MO2_iii)=zloc2
zloc2=Z4SPEC(0.0,JPHC3P,0.0,JPMO2,0.085,JPXO2,0.119,JPGLY)
Zindice_4(JP4_HC3P_MO2_iv)=zloc2
!
zloc2=Z4SPEC(1.0,JPHC5P,0.951,JPMO2,0.829,JPHCHO,0.946,JPHO2)
Zindice_4(JP4_HC5P_MO2_i)=zloc2
zloc2=Z4SPEC(0.0,JPHC5P,0.0,JPMO2,0.523,JPALD,0.24,JPKET)
Zindice_4(JP4_HC5P_MO2_ii)=zloc2
zloc2=Z4SPEC(0.0,JPHC5P,0.0,JPMO2,0.014,JPETHP,0.245,JPXO2)
Zindice_4(JP4_HC5P_MO2_iii)=zloc2
zloc2=Z4SPEC(0.0,JPHC5P,0.0,JPMO2,0.00,JPMO2,0.0,JPDUMMY)
Zindice_4(JP4_HC5P_MO2_iv)=zloc2
!
zloc2=Z4SPEC(1.0,JPHC8P,1.0,JPMO2,0.753,JPHCHO,0.993,JPHO2)
Zindice_4(JP4_HC8P_MO2_i)=zloc2
zloc2=Z4SPEC(0.0,JPHC8P,0.0,JPMO2,0.411,JPALD,0.419,JPKET)
Zindice_4(JP4_HC8P_MO2_ii)=zloc2
zloc2=Z4SPEC(0.0,JPHC8P,0.0,JPMO2,0.322,JPXO2,0.013,JPETHP)
Zindice_4(JP4_HC8P_MO2_iii)=zloc2
!
zloc2=Z4SPEC(1.0,JPETEP,1.0,JPMO2,1.55,JPHCHO,1.0,JPHO2)
Zindice_4(JP4_ETEP_MO2_i)=zloc2
zloc2=Z4SPEC(0.0,JPETEP,0.0,JPMO2,0.35,JPALD,0.0,JPDUMMY)
Zindice_4(JP4_ETEP_MO2_ii)=zloc2
!
zloc2=Z4SPEC(1.0,JPOLTP,1.0,JPMO2,1.25,JPHCHO,1.0,JPHO2)
Zindice_4(JP4_OLTP_MO2_i)=zloc2
zloc2=Z4SPEC(0.0,JPOLTP,0.0,JPMO2,0.669,JPALD,0.081,JPKET)
Zindice_4(JP4_OLTP_MO2_ii)=zloc2
!
zloc2=Z4SPEC(1.0,JPOLIP,1.0,JPMO2,0.755,JPHCHO,1.0,JPHO2)
Zindice_4(JP4_OLIP_MO2_i)=zloc2
zloc2=Z4SPEC(0.0,JPOLIP,0.0,JPMO2,0.932,JPALD,0.313,JPKET)
Zindice_4(JP4_OLIP_MO2_ii)=zloc2
!
zloc2=Z4SPEC(1.0,JPISOP,1.0,JPMO2,0.55,JPMACR,0.37,JPOLT)
Zindice_4(JP4_ISOP_MO2_i)=zloc2
zloc2=Z4SPEC(0.0,JPISOP,0.0,JPMO2,1.0,JPHO2,0.08,JPOLI)
Zindice_4(JP4_ISOP_MO2_ii)=zloc2
zloc2=Z4SPEC(0.0,JPISOP,0.0,JPMO2,1.09,JPHCHO,0.0,JPDUMMY)
Zindice_4(JP4_ISOP_MO2_iii)=zloc2
!
zloc2=Z4SPEC(1.0,JPAPIP,1.0,JPMO2,2.0,JPHO2,1.0,JPALD)
Zindice_4(JP4_APIP_MO2_i)=zloc2
zloc2=Z4SPEC(0.0,JPAPIP,0.0,JPMO2,1.0,JPHCHO,1.0,JPKET)
Zindice_4(JP4_APIP_MO2_ii)=zloc2
!
zloc2=Z4SPEC(1.0,JPLIMP,1.0,JPMO2,2.0,JPHO2,0.6,JPMACR)
Zindice_4(JP4_LIMP_MO2_i)=zloc2
zloc2=Z4SPEC(0.0,JPLIMP,0.0,JPMO2,0.4,JPOLI,1.4,JPHCHO)
Zindice_4(JP4_LIMP_MO2_ii)=zloc2
!
zloc2=Z4SPEC(1.0,JPTOLP,1.0,JPMO2,1.0,JPHCHO,1.0,JPHO2)
Zindice_4(JP4_TOLP_MO2_i)=zloc2
zloc2=Z4SPEC(0.0,JPTOLP,0.0,JPMO2,0.65,JPGLY,1.0,JPDCB)
Zindice_4(JP4_TOLP_MO2_ii)=zloc2
zloc2=Z4SPEC(0.0,JPTOLP,0.0,JPMO2,0.35,JPMGLY,0.0,JPDUMMY)
Zindice_4(JP4_TOLP_MO2_iii)=zloc2
!
zloc2=Z4SPEC(1.0,JPXYLP,1.0,JPMO2,1.0,JPHCHO,1.0,JPHO2)
Zindice_4(JP4_XYLP_MO2_i)=zloc2
zloc2=Z4SPEC(0.0,JPXYLP,0.0,JPMO2,0.37,JPGLY,1.0,JPDCB)
Zindice_4(JP4_XYLP_MO2_ii)=zloc2
zloc2=Z4SPEC(0.0,JPXYLP,0.0,JPMO2,0.63,JPMGLY,0.0,JPDUMMY)
Zindice_4(JP4_XYLP_MO2_iii)=zloc2
!
zloc2=Z4SPEC(1.0,JPCSLP,1.0,JPMO2,1.0,JPGLY,1.0,JPMGLY)
Zindice_4(JP4_CSLP_MO2_i)=zloc2
zloc2=Z4SPEC(0.0,JPCSLP,0.0,JPMO2,2.0,JPHO2,1.0,JPHCHO)
Zindice_4(JP4_CSLP_MO2_ii)=zloc2
!
zloc2=Z4SPEC(1.0,JPACO3,0.0,JPMO2,1.0,JPHCHO,1.0,JPHO2)
Zindice_4(JP4_ACO3_MO2a_i)=zloc2
zloc2=Z4SPEC(0.0,JPACO3,0.0,JPMO2,0.0,JPMO2,0.0,JPDUMMY)
Zindice_4(JP4_ACO3_MO2a_ii)=zloc2
!
zloc2=Z4SPEC(1.0,JPACO3,1.0,JPMO2,1.0,JPHCHO,0.0,JPDUMMY)
Zindice_4(JP4_ACO3_MO2b)=zloc2
!
zloc2=Z4SPEC(1.0,JPTCO3,1.0,JPMO2,1.0,JPACO3,2.0,JPHCHO)
Zindice_4(JP4_TCO3_MO2a_i)=zloc2
zloc2=Z4SPEC(0.0,JPTCO3,0.0,JPMO2,1.0,JPHO2,0.0,JPDUMMY)
Zindice_4(JP4_TCO3_MO2a_ii)=zloc2
!
zloc2=Z4SPEC(1.0,JPTCO3,1.0,JPMO2,1.0,JPHCHO,0.0,JPDUMMY)
Zindice_4(JP4_TCO3_MO2b)=zloc2
!
zloc2=Z4SPEC(1.0,JPKETP,1.0,JPMO2,0.4,JPMGLY,0.3,JPALD)
Zindice_4(JP4_KETP_MO2_i)=zloc2
zloc2=Z4SPEC(0.0,JPKETP,0.0,JPMO2,0.3,JPHKET,0.88,JPHO2)
Zindice_4(JP4_KETP_MO2_ii)=zloc2
zloc2=Z4SPEC(0.0,JPKETP,0.0,JPMO2,0.08,JPXO2,0.12,JPACO3)
Zindice_4(JP4_KETP_MO2_iii)=zloc2
zloc2=Z4SPEC(0.0,JPKETP,0.0,JPMO2,0.75,JPHCHO,0.0,JPDUMMY)
Zindice_4(JP4_KETP_MO2_iv)=zloc2
!
zloc2=Z4SPEC(1.0,JPOLNN,1.0,JPMO2,0.75,JPHCHO,1.0,JPHO2)
Zindice_4(JP4_OLNN_MO2_i)=zloc2
zloc2=Z4SPEC(0.0,JPOLNN,0.0,JPMO2,1.0,JPONIT,0.0,JPDUMMY)
Zindice_4(JP4_OLNN_MO2_ii)=zloc2
!
zloc2=Z4SPEC(1.0,JPOLND,1.0,JPMO2,0.96,JPHCHO,0.5,JPHO2)
Zindice_4(JP4_OLND_MO2_i)=zloc2
zloc2=Z4SPEC(0.0,JPOLND,0.0,JPMO2,0.64,JPALD,0.149,JPKET)
Zindice_4(JP4_OLND_MO2_ii)=zloc2
zloc2=Z4SPEC(0.0,JPOLND,0.0,JPMO2,0.5,JPNO2,0.5,JPONIT)
Zindice_4(JP4_OLND_MO2_iii)=zloc2
!
zloc2=Z4SPEC(1.0,JPETHP,1.0,JPACO3,1.0,JPALD,0.5,JPHO2)
Zindice_4(JP4_ETHP_ACO3_i)=zloc2
zloc2=Z4SPEC(0.0,JPETHP,0.0,JPACO3,0.5,JPMO2,0.0,JPDUMMY)
Zindice_4(JP4_ETHP_ACO3_ii)=zloc2
!
zloc2=Z4SPEC(1.0,JPHC3P,1.0,JPACO3,0.724,JPALD,0.127,JPKET)
Zindice_4(JP4_HC3P_ACO3_i)=zloc2
zloc2=Z4SPEC(0.0,JPHC3P,0.0,JPACO3,0.488,JPHO2,0.508,JPMO2)
Zindice_4(JP4_HC3P_ACO3_ii)=zloc2
zloc2=Z4SPEC(0.0,JPHC3P,0.0,JPACO3,0.091,JPHCHO,0.006,JPETHP)
Zindice_4(JP4_HC3P_ACO3_iii)=zloc2
zloc2=Z4SPEC(0.0,JPHC3P,0.0,JPACO3,0.071,JPXO2,0.004,JPMGLY)
Zindice_4(JP4_HC3P_ACO3_iv)=zloc2
zloc2=Z4SPEC(0.0,JPHC3P,0.0,JPACO3,0.1,JPGLY,0.0,JPDUMMY)
Zindice_4(JP4_HC3P_ACO3_v)=zloc2
!
zloc2=Z4SPEC(1.0,JPHC5P,1.0,JPACO3,0.677,JPALD,0.33,JPKET)
Zindice_4(JP4_HC5P_ACO3_i)=zloc2
zloc2=Z4SPEC(0.0,JPHC5P,0.0,JPACO3,0.438,JPHO2,0.554,JPMO2)
Zindice_4(JP4_HC5P_ACO3_ii)=zloc2
zloc2=Z4SPEC(0.0,JPHC5P,0.0,JPACO3,0.0,JPDUMMY,0.076,JPHCHO)
Zindice_4(JP4_HC5P_ACO3_iii)=zloc2
zloc2=Z4SPEC(0.0,JPHC5P,0.0,JPACO3,0.018,JPETHP,0.237,JPXO2)
Zindice_4(JP4_HC5P_ACO3_iv)=zloc2
!
zloc2=Z4SPEC(1.0,JPHC8P,1.0,JPACO3,0.497,JPALD,0.581,JPKET)
Zindice_4(JP4_HC8P_ACO3_i)=zloc2
zloc2=Z4SPEC(0.0,JPHC8P,0.0,JPACO3,0.489,JPHO2,0.507,JPMO2)
Zindice_4(JP4_HC8P_ACO3_ii)=zloc2
zloc2=Z4SPEC(0.0,JPHC8P,0.0,JPACO3,0.0,JPDUMMY,0.015,JPETHP)
Zindice_4(JP4_HC8P_ACO3_iii)=zloc2
zloc2=Z4SPEC(0.0,JPHC8P,0.0,JPACO3,0.318,JPXO2,0.0,JPDUMMY)
Zindice_4(JP4_HC8P_ACO3_iv)=zloc2
!
zloc2=Z4SPEC(1.0,JPETEP,1.0,JPACO3,0.8,JPHCHO,0.6,JPALD)
Zindice_4(JP4_ETEP_ACO3_i)=zloc2
zloc2=Z4SPEC(0.0,JPETEP,0.0,JPACO3,0.5,JPHO2,0.5,JPMO2)
Zindice_4(JP4_ETEP_ACO3_ii)=zloc2
!
zloc2=Z4SPEC(1.0,JPOLTP,1.0,JPACO3,0.859,JPALD,0.501,JPHCHO)
Zindice_4(JP4_OLTP_ACO3_i)=zloc2
zloc2=Z4SPEC(0.0,JPOLTP,0.0,JPACO3,0.501,JPHO2,0.501,JPMO2)
Zindice_4(JP4_OLTP_ACO3_ii)=zloc2
zloc2=Z4SPEC(0.0,JPOLTP,0.0,JPACO3,0.0,JPDUMMY,0.141,JPKET)
Zindice_4(JP4_OLTP_ACO3_iii)=zloc2
!
zloc2=Z4SPEC(1.0,JPOLIP,1.0,JPACO3,0.941,JPALD,0.51,JPHO2)
Zindice_4(JP4_OLIP_ACO3_i)=zloc2
zloc2=Z4SPEC(0.0,JPOLIP,0.0,JPACO3,0.51,JPMO2,0.569,JPKET)
Zindice_4(JP4_OLIP_ACO3_ii)=zloc2
!
zloc2=Z4SPEC(1.0,JPISOP,1.0,JPACO3,0.771,JPMACR,0.229,JPOLT)
Zindice_4(JP4_ISOP_ACO3_i)=zloc2
zloc2=Z4SPEC(0.0,JPISOP,0.0,JPACO3,0.506,JPHO2,0.0,JPDUMMY)
Zindice_4(JP4_ISOP_ACO3_ii)=zloc2
zloc2=Z4SPEC(0.0,JPISOP,0.0,JPACO3,0.34,JPHCHO,0.506,JPMO2)
Zindice_4(JP4_ISOP_ACO3_iii)=zloc2
!
zloc2=Z4SPEC(1.0,JPAPIP,1.0,JPACO3,1.0,JPHO2,1.0,JPALD)
Zindice_4(JP4_APIP_ACO3_i)=zloc2
zloc2=Z4SPEC(0.0,JPAPIP,0.0,JPACO3,1.0,JPMO2,1.0,JPKET)
Zindice_4(JP4_APIP_ACO3_ii)=zloc2
!
zloc2=Z4SPEC(1.0,JPLIMP,1.0,JPACO3,1.0,JPHO2,0.6,JPMACR)
Zindice_4(JP4_LIMP_ACO3_i)=zloc2
zloc2=Z4SPEC(0.0,JPLIMP,0.0,JPACO3,0.4,JPOLI,0.4,JPHCHO)
Zindice_4(JP4_LIMP_ACO3_ii)=zloc2
zloc2=Z4SPEC(0.0,JPLIMP,0.0,JPACO3,1.0,JPMO2,0.0,JPDUMMY)
Zindice_4(JP4_LIMP_ACO3_iii)=zloc2
!
zloc2=Z4SPEC(1.0,JPTOLP,1.0,JPACO3,1.0,JPMO2,1.0,JPHO2)
Zindice_4(JP4_TOLP_ACO3_i)=zloc2
zloc2=Z4SPEC(0.0,JPTOLP,0.0,JPACO3,1.0,JPDCB,0.65,JPGLY)
Zindice_4(JP4_TOLP_ACO3_ii)=zloc2
zloc2=Z4SPEC(0.0,JPTOLP,0.0,JPACO3,0.35,JPMGLY,0.0,JPDUMMY)
Zindice_4(JP4_TOLP_ACO3_iii)=zloc2
!
zloc2=Z4SPEC(1.0,JPXYLP,1.0,JPACO3,1.0,JPMO2,1.0,JPHO2)
Zindice_4(JP4_XYLP_ACO3_i)=zloc2
zloc2=Z4SPEC(0.0,JPXYLP,0.0,JPACO3,1.0,JPDCB,0.37,JPGLY)
Zindice_4(JP4_XYLP_ACO3_ii)=zloc2
zloc2=Z4SPEC(0.0,JPXYLP,0.0,JPACO3,0.63,JPMGLY,0.0,JPDUMMY)
Zindice_4(JP4_XYLP_ACO3_iii)=zloc2
!
zloc2=Z4SPEC(1.0,JPCSLP,1.0,JPACO3,1.0,JPGLY,1.0,JPMGLY)
Zindice_4(JP4_CSLP_ACO3_i)=zloc2
zloc2=Z4SPEC(0.0,JPCSLP,0.0,JPACO3,1.0,JPHO2,1.0,JPMO2)
Zindice_4(JP4_CSLP_ACO3_ii)=zloc2
!
zloc2=Z4SPEC(1.0,JPTCO3,1.0,JPACO3,1.0,JPMO2,1.0,JPACO3)
Zindice_4(JP4_TCO3_ACO3_i)=zloc2
zloc2=Z4SPEC(0.0,JPTCO3,0.0,JPACO3,1.0,JPHCHO,0.0,JPDUMMY)
Zindice_4(JP4_TCO3_ACO3_ii)=zloc2
!
zloc2=Z4SPEC(1.0,JPKETP,1.0,JPACO3,0.54,JPMGLY,0.35,JPALD)
Zindice_4(JP4_KETP_ACO3_i)=zloc2
zloc2=Z4SPEC(0.0,JPKETP,0.0,JPACO3,0.11,JPKET,0.12,JPACO3)
Zindice_4(JP4_KETP_ACO3_ii)=zloc2
zloc2=Z4SPEC(0.0,JPKETP,0.0,JPACO3,0.38,JPHO2,0.08,JPXO2)
Zindice_4(JP4_KETP_ACO3_iii)=zloc2
zloc2=Z4SPEC(0.0,JPKETP,0.0,JPACO3,0.5,JPMO2,0.0,JPDUMMY)
Zindice_4(JP4_KETP_ACO3_iv)=zloc2
!
zloc2=Z4SPEC(1.0,JPOLNN,1.0,JPACO3,1.0,JPONIT,0.5,JPMO2)
Zindice_4(JP4_OLNN_ACO3_i)=zloc2
zloc2=Z4SPEC(0.0,JPOLNN,0.0,JPACO3,0.0,JPDUMMY,0.5,JPHO2)
Zindice_4(JP4_OLNN_ACO3_ii)=zloc2
!
zloc2=Z4SPEC(1.0,JPOLND,1.0,JPACO3,0.207,JPHCHO,0.65,JPALD)
Zindice_4(JP4_OLND_ACO3_i)=zloc2
zloc2=Z4SPEC(0.0,JPOLND,0.0,JPACO3,0.167,JPKET,0.516,JPMO2)
Zindice_4(JP4_OLND_ACO3_ii)=zloc2
zloc2=Z4SPEC(0.0,JPOLND,0.0,JPACO3,0.484,JPONIT,0.516,JPNO2)
Zindice_4(JP4_OLND_ACO3_iii)=zloc2
!
! OLNN +OLND - 0.202*HCHO + 0.640*ALD + 0.149*KET +0.500*HO2 + 0.500*NO2
! + 1.50*ONIT
zloc2=Z4SPEC(1.0,JPOLNN,1.0,JPOLND,0.202,JPHCHO,0.640,JPALD)
Zindice_4(JP4_OLNN_OLND_i)=zloc2
zloc2=Z4SPEC(0.0,JPOLNN,0.0,JPOLND,0.149,JPKET,0.5,JPHO2)
Zindice_4(JP4_OLNN_OLND_ii)=zloc2
zloc2=Z4SPEC(0.0,JPOLNN,0.0,JPOLND,0.5,JPNO2,1.5,JPONIT)
Zindice_4(JP4_OLNN_OLND_iii)=zloc2
!
zloc2=Z4SPEC(1.0,JPHC8P,1.0,JPNO3,0.187,JPALD,0.880,JPKET)
Zindice_4(JP4_HC8P_NO3_i)=zloc2
zloc2=Z4SPEC(0.0,JPHC8P,0.0,JPNO3,0.155,JPETHP,1.0,JPNO2)
Zindice_4(JP4_HC8P_NO3_ii)=zloc2
zloc2=Z4SPEC(0.0,JPHC8P,0.0,JPNO3,0.845,JPHO2,0.587,JPXO2)
Zindice_4(JP4_HC8P_NO3_iii)=zloc2
!
zloc2=Z4SPEC(1.0,JPETEP,1.0,JPNO3,1.6,JPHCHO,1.0,JPHO2)
Zindice_4(JP4_ETEP_NO3_i)=zloc2
zloc2=Z4SPEC(0.0,JPETEP,0.0,JPNO3,1.0,JPNO2,0.2,JPALD)
Zindice_4(JP4_ETEP_NO3_ii)=zloc2
!
zloc2=Z4SPEC(1.0,JPOLTP,1.0,JPNO3,0.94,JPALD,1.0,JPHCHO)
Zindice_4(JP4_OLTP_NO3_i)=zloc2
zloc2=Z4SPEC(0.0,JPOLTP,0.0,JPNO3,1.0,JPHO2,1.0,JPNO2)
Zindice_4(JP4_OLTP_NO3_ii)=zloc2
zloc2=Z4SPEC(0.0,JPOLTP,0.0,JPNO3,0.06,JPKET,0.0,JPDUMMY)
Zindice_4(JP4_OLTP_NO3_iii)=zloc2
!
zloc2=Z4SPEC(1.0,JPOLIP,1.0,JPNO3,1.0,JPHO2,1.71,JPALD)
Zindice_4(JP4_OLIP_NO3_i)=zloc2
zloc2=Z4SPEC(0.0,JPOLIP,0.0,JPNO3,0.29,JPKET,1.0,JPNO2)
Zindice_4(JP4_OLIP_NO3_ii)=zloc2
!
zloc2=Z4SPEC(1.0,JPISOP,1.0,JPNO3,0.60,JPMACR,0.4,JPOLT)
Zindice_4(JP4_ISOP_NO3_i)=zloc2
zloc2=Z4SPEC(0.0,JPISOP,0.0,JPNO3,1.0,JPHO2,1.0,JPNO2)
Zindice_4(JP4_ISOP_NO3_ii)=zloc2
zloc2=Z4SPEC(0.0,JPISOP,0.0,JPNO3,0.686,JPHCHO,0.0,JPDUMMY)
Zindice_4(JP4_ISOP_NO3_iii)=zloc2
!
zloc2=Z4SPEC(1.0,JPAPIP,1.0,JPNO3,1.0,JPHO2,1.0,JPALD)
Zindice_4(JP4_APIP_NO3_i)=zloc2
zloc2=Z4SPEC(0.0,JPAPIP,0.0,JPNO3,1.0,JPNO2,1.0,JPKET)
Zindice_4(JP4_APIP_NO3_ii)=zloc2
! LIMP + NO3 --> HO2 + 0.60*MACR + 0.40*OLI + 0.40*HCHO + NO2
zloc2=Z4SPEC(1.0,JPLIMP,1.0,JPNO3,1.0,JPHO2,0.6,JPMACR)
Zindice_4(JP4_LIMP_NO3_i)=zloc2
zloc2=Z4SPEC(0.0,JPLIMP,0.0,JPNO3,0.4,JPOLI,0.4,JPHCHO)
Zindice_4(JP4_LIMP_NO3_ii)=zloc2
zloc2=Z4SPEC(0.0,JPLIMP,0.0,JPNO3,1.0,JPNO2,0.0,JPDUMMY)
Zindice_4(JP4_LIMP_NO3_iii)=zloc2
!
zloc2=Z4SPEC(1.0,JPTOLP,1.0,JPNO3,1.0,JPNO2,1.0,JPHO2)
Zindice_4(JP4_TOLP_NO3_i)=zloc2
zloc2=Z4SPEC(0.0,JPTOLP,0.0,JPNO3,0.5,JPDCB,1.3,JPGLY)
Zindice_4(JP4_TOLP_NO3_ii)=zloc2
zloc2=Z4SPEC(0.0,JPTOLP,0.0,JPNO3,0.7,JPMGLY,0.0,JPDUMMY)
Zindice_4(JP4_TOLP_NO3_iii)=zloc2
!
zloc2=Z4SPEC(1.0,JPXYLP,1.0,JPNO3,1.0,JPNO2,1.0,JPHO2)
Zindice_4(JP4_XYLP_NO3_i)=zloc2
zloc2=Z4SPEC(0.0,JPXYLP,0.0,JPNO3,1.0,JPDCB,0.74,JPGLY)
Zindice_4(JP4_XYLP_NO3_ii)=zloc2
zloc2=Z4SPEC(0.0,JPXYLP,0.0,JPNO3,1.26,JPMGLY,0.0,JPDUMMY)
Zindice_4(JP4_XYLP_NO3_iii)=zloc2
!
zloc2=Z4SPEC(1.0,JPCSLP,1.0,JPNO3,1.0,JPGLY,1.0,JPMGLY)
Zindice_4(JP4_CSLP_NO3_i)=zloc2
zloc2=Z4SPEC(0.0,JPCSLP,0.0,JPNO3,1.0,JPNO2,1.0,JPHO2)
Zindice_4(JP4_CSLP_NO3_ii)=zloc2
!
zloc2=Z4SPEC(1.0,JPACO3,1.0,JPNO3,1.0,JPMO2,1.0,JPNO2)
Zindice_4(JP4_ACO3_NO3)=zloc2
!
zloc2=Z4SPEC(1.0,JPTCO3,1.0,JPNO3,1.0,JPNO2,1.0,JPACO3)
Zindice_4(JP4_TCO3_NO3_i)=zloc2
zloc2=Z4SPEC(0.0,JPTCO3,0.0,JPNO3,1.0,JPHCHO,0.0,JPDUMMY)
Zindice_4(JP4_TCO3_NO3_ii)=zloc2
!
zloc2=Z4SPEC(1.0,JPKETP,1.0,JPNO3,0.54,JPMGLY,0.46,JPALD)
Zindice_4(JP4_KETP_NO3_i)=zloc2
zloc2=Z4SPEC(0.0,JPKETP,0.0,JPNO3,0.23,JPACO3,0.77,JPHO2)
Zindice_4(JP4_KETP_NO3_ii)=zloc2
zloc2=Z4SPEC(0.0,JPKETP,0.0,JPNO3,0.16,JPXO2,1.0,JPNO2)
Zindice_4(JP4_KETP_NO3_iii)=zloc2
!
zloc2=Z4SPEC(1.0,JPOLNN,1.0,JPNO3,1.0,JPONIT,1.0,JPNO2)
Zindice_4(JP4_OLNN_NO3_i)=zloc2
zloc2=Z4SPEC(0.0,JPOLNN,0.0,JPNO3,1.0,JPHO2,0.0,JPDUMMY)
Zindice_4(JP4_OLNN_NO3_ii)=zloc2
!
zloc2=Z4SPEC(1.0,JPOLND,1.0,JPNO3,0.28,JPHCHO,1.24,JPALD)
Zindice_4(JP4_OLND_NO3_i)=zloc2
zloc2=Z4SPEC(0.0,JPOLND,0.0,JPNO3,0.469,JPKET,2.0,JPNO2)
Zindice_4(JP4_OLND_NO3_ii)=zloc2
!
zloc2=Z4SPEC(1.0,JPXO2,1.0,JPHO2,1.0,JPOP2,0.0,JPDUMMY)
Zindice_4(JP4_XO2_HO2)=zloc2
!
zloc2=Z4SPEC(1.0,JPXO2,1.0,JPMO2,1.0,JPHCHO,1.0,JPHO2)
Zindice_4(JP4_XO2_MO2)=zloc2
!
zloc2=Z4SPEC(1.0,JPXO2,1.0,JPACO3,1.0,JPMO2,0.0,JPDUMMY)
Zindice_4(JP4_XO2_ACO3)=zloc2
!
zloc2=Z4SPEC(1.0,JPXO2,1.0,JPNO,1.0,JPNO2,0.0,JPDUMMY)
Zindice_4(JP4_XO2_NO)=zloc2
!
zloc2=Z4SPEC(1.0,JPXO2,1.0,JPNO3,1.0,JPNO2,0.0,JPDUMMY)
Zindice_4(JP4_XO2_NO3)=zloc2
!
zloc2=Z4SPEC(1.0,JPDMS,1.0,JPNO3,1.0,JPSO2,0.0,JPDUMMY)
Zindice_4(JP4_DMS_NO3)=zloc2
!
zloc2=Z4SPEC(1.0,JPDMS,1.0,JPO3P,1.0,JPSO2,0.0,JPDUMMY)
Zindice_4(JP4_DMS_O3P)=zloc2
!
zloc2=Z4SPEC(1.0,JPDMS,1.0,JPOH,0.8,JPSO2,0.0,JPDUMMY)
Zindice_4(JP4_DMS_OH)=zloc2
!
write(*,*) 'Number of reactions of each kind :'
write(*,*) '        NB_PHOT=',NB_PHOT
write(*,*) '        NB_REACTION_3=',NB_REACTION_3
write(*,*) '        NB_REACTION_4=',NB_REACTION_4
!
END SUBROUTINE indices_reactions

END MODULE indices_reactions_module
